local L = LibStub("AceLocale-3.0"):GetLocale("SPTimers")
local options
local font_list
local statusbar_list

local myGUID

local parent = SPTimers.Parent

SPTimers.Anchors = {}
SPTimers.Bars = {}
local anchors = SPTimers.Anchors
local bars_list = SPTimers.Bars
local tinsert = tinsert

-- debug print ------------------
local old_print = print
local print = function(...)
	if SPTimers.dodebugging then	
		old_print(GetTime(), "SPTimers_BARS, ", ...)
	end
end

local old_assert = assert
local assert = function(...)
	if SPTimers.dodebugging then	
	--	old_assert(...)
	end
end

local function GetDefaultFont(name)
	if font_list[name] then
		return font_list[name]
	else
		return STANDARD_TEXT_FONT
	end
end

local function GetDefaultTexture(name)
	if statusbar_list[name] then
		return statusbar_list[name]
	else
		return "Interface\\ChatFrame\\ChatFrameBackground"
	end
end

local function deepcopy(t)
	if type(t) ~= 'table' then return t end
		local mt = getmetatable(t)
		local res = {}
		for k,v in pairs(t) do
			if type(v) == 'table' then
				v = deepcopy(v)


			end
		res[k] = v
		end
		setmetatable(res,mt)
	return res
end
	
local backdrop = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	edgeFile = "Interface\\ChatFrame\\ChatFrameBackground",
	tile = true, tileSize = 1,
	insets = {left = -2, right = -2, top = -2, bottom = -2},
}

local backdrop_mover = {
	bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	edgeFile = "Interface\\ChatFrame\\ChatFrameBackground",
	tile = true, tileSize = 0,
	insets = {left = -2, right = -2, top = -2, bottom = -2},
}

function SPTimers:InitFrames()

	self.myGUID = UnitGUID("player")

	local _,class = UnitClass("player")	
	self.myCLASS = class

	options = self.db.profile
	font_list = SPTimers.SharedMedia:HashTable("font")
	statusbar_list =  SPTimers.SharedMedia:HashTable("statusbar")
	
	print("InitFrames", #anchors, #options.bars_anchors)
	if #anchors > 0 and #anchors > #options.bars_anchors then

		for i=1, #anchors-#options.bars_anchors do
		
			SPTimers:DeleteAnchor(#anchors)
			print("remove unusable anchor", #anchors)
		end
	end

	
	for i, opts in pairs(options.bars_anchors) do	
		self:InitBarAnchor(i,opts)
	end
end

do
	local mark = 0
	local testbar = false
	
	function SPTimers:TestBars(force)
		if testbar or force then
			self.testbar_shown = nil
			self.onUpdateHandler.elapsed = 0
			self.onUpdateHandler:Show()
			mark = 0
			for k,v in pairs(bars_list) do
				v:EndTimer()
			end
		else
			self.testbar_shown = true
			self.onUpdateHandler.elapsed = 99999
			self.onUpdateHandler:Hide()			

			for k,v in pairs(anchors) do
				local a = 0
				local group = 0
				local group_guid = 1
				
				for f in pairs(v.free) do
					local t = math.random(1, 100)
					local name = "Test Bar №"..a.." of "..( v.opts.name or k )
					a = a +1
					 
					group = group + 1
					
					if mark >= 8 then
						mark = 1
					else
						mark = mark + 1
					end
					
					if group >= 5 then 
						group = 0
						group_guid = group_guid + 1
					end
					
					v:ActivateTimer(t, GetTime()+t ,name ,a ,"Interface\\Icons\\spell_shadow_shadowwordpain","BUFF",a, "group"..group_guid, "group"..group_guid, unit,sUnit,name,name,mark,"TEST_BAR")
				end
			end
		end
		testbar = not testbar
	end
end

do
	local default_anchor = {
		name = 1,
		bar_number = 20,
		left_icon = true,
		right_icon = false,
		add_up = true,
		point = { 0, 0},
		w = 250,
		h = 14,
		target_name =true,
		gap = 4,
		icon_gap = 5,
		fortam_s = 1,
		border = "Flat",
		bordersize = 1, -- Added defaults
		borderinset = 0, -- Added defaults
		bordercolor = {80/255,80/255,80/255,1},
		bar = { 
			color = {118/255, 0, 0, 1},
			texture = "Flat",
			bgcolor = {0, 0, 0, 0.5},
			bgtexture = "Flat",
		},
		stack = {
			textcolor = {1, 1, 1},
			font = STANDARD_TEXT_FONT,
			size = 14,
			flags = "OUTLINE",
			justify = "RIGHT",
			shadow =  { 0, 0, 0, 1},
			offset = { 0, 0 },
		},
		timer ={
			textcolor = {1, 1, 1},
			font = STANDARD_TEXT_FONT,
			size = 14,
			flags = "OUTLINE",
			justify = "RIGHT",
			shadow =  { 0, 0, 0, 1},
			offset = { 0, 0 },
		},
		
		raidicon_x = 0,
		raidicon_y = 5, 
		raidiconsize = 10,
		raidicon_alpha = 1,
						
		spell ={
			textcolor = {1, 1, 1},
			font = STANDARD_TEXT_FONT,
			size = 14,
			flags = "OUTLINE",
			justify = "LEFT",
			offsetx = 0,
			shadow =  { 0, 0, 0, 1},
			offset = { 0, 0 },
		},
		castspark = {
			color = {1, 1, 1, 1},	
			alpha = 1,
		},
		dotticks = {
			color = {1, 1, 1, 1},	
			alpha = 1,
		},
		sorting = {
			{name = "target", 		gap = 10, alpha = 1,  sort = 1 },
			{name = "player", 		gap = 10, alpha = 1,  sort = 2 },
			{name = "procs",		gap = 15, alpha = .7, sort = 3 },
			{name = "offtargets",	gap = 6,  alpha = .7, sort = 4 },
		},	
	}

	function SPTimers:CreateNewAnhors()
		local anhor_number = #options.bars_anchors+1
		
		options.totalanchor = options.totalanchor+1
		
		options.bars_anchors[anhor_number] = deepcopy(default_anchor)
		options.bars_anchors[anhor_number].name = options.totalanchor
		
		self:InitBarAnchor(anhor_number, options.bars_anchors[anhor_number])		
		self:SetAnchorTable(anhor_number)
	end
end

function SPTimers:DeleteAnchor(number)
	if anchors[number] then
	
		anchors[number]:Delete()
		
		table.remove(options.bars_anchors, number)
	end
end

function SPTimers:CopySettings(from, to)
	local a1 = options.bars_anchors[to].point
	local a2 = options.bars_anchors[to].name
	
	options.bars_anchors[to] = deepcopy(options.bars_anchors[from])
	options.bars_anchors[to].point = deepcopy(a1)
	options.bars_anchors[to].name = a2
	
--	print(to, options.bars_anchors[to], #options.bars_anchors)
	self:InitBarAnchor(to, options.bars_anchors[to])	
	
	self:Visibility()
	self:Update_StackText()
	self:Update_TimeText()
	self:Update_SpellText()
	self:UpdateAllBorder()
end

function SPTimers:UpdateSmothBars()
	for k,v in ipairs(bars_list) do
		v:SmoothBars()	
	end
end

function SPTimers:Visibility()	
	for k,v in ipairs(bars_list) do
		v:Update()	
	end
end

function SPTimers:Update_StackText()	
	for k,v in ipairs(bars_list) do
		v:UpdateStackText()	
	end
end
function SPTimers:Update_TimeText()	
	for k,v in ipairs(bars_list) do
		v:UpdateTimeText()	
	end
end
function SPTimers:Update_SpellText()	
	for k,v in ipairs(bars_list) do
		v:UpdateSpellText()	
	end
end
function SPTimers:UpdateAllBorder()
	for k,v in ipairs(bars_list) do
		v:UpdateBorder()	
	end
end

function SPTimers:UpdateRaidIcons()
	for k,v in ipairs(bars_list) do
		v:UpdateRaidIcon()	
	end
end


function SPTimers:UpdateStatusBars()
	for k,v in ipairs(bars_list) do
		v:EndTimer()	
	end
end

function SPTimers:UpdateBarsSize()
	for k,v in ipairs(bars_list) do
		v:UpdateBarSize()	
	end
end

function SPTimers:UpdateBackgroundBarColor()
	for k,v in pairs(anchors) do	
		for f in pairs(v.active) do
			f:UpdateBarColor()
		end		
	end			
end

function SPTimers:UpdateSortingFunction()
	for k,v in pairs(anchors) do	
		 v:SortTimers()
	end			
end

function SPTimers:UpdateAllSparks()
	for k,v in pairs(anchors) do	
		for f in pairs(v.active) do
			f:UpdateSpark_Color()
		end		
	end			
end

function SPTimers:UpdateAllTiks()
	for k,v in pairs(anchors) do	
		for f in pairs(v.active) do
			f:UpdateTick_Color()
		end		
	end			
end

function SPTimers:UpdateMovers()
	if options.locked then
		for k,v in pairs(anchors) do
			v:Lock()
		end
	else 
		for k,v in pairs(anchors) do
			v:Unlock()
		end
	end
end

local main_bg_color = {0/255 , 0/255 , 0/255 , 1}
local main_border_color = {0, 0, 0 , 0}

local main_bg_color1 = {255/255 , 0/255 , 0/255 , 1}
local main_border_color1 = {0, 0, 0 , 0}

local main_bg_color2 = {0/255 , 0/255 , 0/255 , 1}
local main_border_color2 = {0, 0, 0 , 0}


function SPTimers:InitBarAnchor(i, opts)

	local f
	
	if opts and opts.sorting then
		for k,v in ipairs(opts.sorting) do
			if not v.sort then
				if v.name == "offtargets" then 
					v.sort = 4 
				else
					v.sort = k
				end
			end
		end
	end
	
	if not anchors[i] then
		f = CreateFrame("Frame", "SPTimers_Bar_Anchor_"..i ,parent)
		f.id = i
		--[[
		f:SetBackdrop(backdrop)
		f:SetBackdropColor(unpack(main_bg_color1)) --цвет фона
		f:SetBackdropBorderColor(unpack(main_bg_color1)) --цвет краев
		]]
		f.bars = {}		
		f.active = {}
		f.free = {}
		f.queue = {}
		
		f.groups = { player = {}, target = {}, procs = {} }
		f.guid_groups = {}


		setmetatable(f.active,{ __newindex = function(t,k,v)
			rawset(f.free,k,nil)
			rawset(t,k,v)
		end})
		setmetatable(f.free,{ __newindex = function(t,k,v)
		
	--		print("Free bar", f.name)
			k:Hide()
			rawset(f.active,k,nil)
			rawset(t,k,v)
		end})
		
	--	assert(opts, "NO OPTS FOUND")
		f.opts = opts
		
		f.Delete = function(self)
			wipe(self.bars)
			for k in pairs(self.active) do		   
				k:EndTimer()
			end
			wipe(self.active)
			wipe(self.free)
			wipe(self.groups)
			wipe(self.queue)
			wipe(self.guid_groups)
			
			self.mover:Hide()
			self:Hide()

			tremove(anchors, self.id)
		end
		
		f.GetTimer = function(self,targetGUID,spellID,sourceGUID,aura_type, aura_number, duration, endTime)
			if aura_number then
				for k in pairs(self.active) do
					if k.targetGUID == targetGUID and
					   k.spellID	== spellID	  and
					 --  k.duration   == duration	  and
					   k.aura_type  == aura_type  and
					 --  k.endTime	== endTime	  then
					   k.aura_number == aura_number then
						return k
					end
				end
			elseif sourceGUID then
				for k in pairs(self.active) do
					if k.targetGUID == targetGUID and
					   k.spellID	== spellID	  and
					   k.sourceGUID == sourceGUID and
					   k.aura_type  == aura_type  then
					   
						return k
					end
				end
			else
				for k in pairs(self.active) do
					if k.targetGUID == targetGUID and
					   k.spellID	== spellID	  and
					   k.aura_type  == aura_type  then
					   
						return k
					end
				end
			end
			return false
		end
		
		f.GetTimerSourceGUID = function(self,spellID,sourceGUID)
			for k in pairs(self.active) do
			
			--	print( spellID, (k.spellID	== spellID), ( k.sourceGUID == sourceGUID ), ( k.targetGUID ~= sourceGUID ))
				if k.spellID	== spellID	  and
				   k.sourceGUID == sourceGUID and
				   k.targetGUID ~= sourceGUID then		   
					return k
				end
			end
			return false
		end
		
		f.GetHighestStartTime = function(self)
			local m = 0

			for k in pairs(self.active) do
				if ( k.duration < 60 ) and ( k.duration > m or m == 0 ) then	
					m = k.duration
				end
			end
			
			if m > 0 then
				for k in pairs(self.active) do
					
					if k.duration > 0 and k.endTime > 0 then
					--	k.startTime = m
						if options.maximumtime then
							k.bar:SetMinMaxValues(0, options.maximumtime_value)
						else
							k.bar:SetMinMaxValues(0, m)
						end
						
						if k.showticks then
							k:SetTicks() 




						end
					end
				end			
			end

			return m
		end
		f.GetTimerByGUID = function(self,targetGUID, auratype)
			local t = {}
			local i = 0
			for k in pairs(self.active) do
				if k.targetGUID == targetGUID then
					if auratype then
						if auratype == f.aura_type then
							i = i+1
							t[k] = true
						end
					else
						i = i+1
						t[k] = true
					end
				end
			end
			return i > 0 and t or false
		end
		
	
		f.Unlock = function(self)
			self.mover:Show()
			self.mover:EnableMouse(true)
		end		
		f.Lock = function(self)
			self.mover:Hide()
			self.mover:EnableMouse(false)
		end
		
		f.ActivateTimer = SPTimers.ActivateTimer
		f.ActivateTimerSkip = SPTimers.ActivateTimer2
		f.AppliedDose	= SPTimers.AppliedDose
		f.QueueBars = SPTimers.QueueBars
		f.SwapTimers = SPTimers.SwapTimers
		
		
		f.SortTimers = SPTimers.SortTimers
		f.ResetBarPositions = function(self)
			for k,v in pairs(self.bars) do		
				v:ClearAllPoints()
			end
		end
		
		
		f.mover = CreateFrame("Frame", f:GetName().."_Mover", f)
		f.mover.text = f.mover:CreateFontString(nil, "OVERLAY", "GameFontNormal");
		f.mover.text:SetPoint("CENTER", f.mover, "CENTER",0,0)
		f.mover.text:SetTextColor(1,1,1,1)
		f.mover.text:SetFont(STANDARD_TEXT_FONT,
							12,
							"OUTLINE")
		f.mover.text:SetJustifyH("CENTER")
		f.mover.text:SetText(L["Unlocked. Move group №"]..i)
		
		
		f.mover.parent = f
		
		f.mover:SetMovable(true)
		f.mover:RegisterForDrag("LeftButton")
		f.mover:SetScript("OnDragStart", function(self) self:StartMoving() end)
		f.mover:SetScript("OnDragStop", function(self) 
			self:StopMovingOrSizing()
			local x, y = self:GetCenter()
			local ux, uy = parent:GetCenter()

			self.parent.opts.point = { floor(x - ux + 0.5),floor(y - uy + 0.5) }

			for k,v in pairs(self.mover_add_button.editboxes) do
				v:UpdateText()
			end
		end)

		f.mover:SetClampedToScreen(true)
		
		f.mover:SetBackdrop(backdrop_mover)
		f.mover:SetBackdropColor(unpack(main_bg_color)) --цвет фона
		f.mover:SetBackdropBorderColor(unpack(main_border_color)) --цвет краев	
		f.mover:SetAlpha(.6)
		f.mover:Hide()
		
		SPTimers.AddMoverButtons(f.mover, f.opts)
		
		f.CreateBar = SPTimers.Create_Bar
		
		anchors[i] = f

		f:SetSize( f.opts.w or 100 , f.opts.h or 20)
		f:Show()

		f.mover:ClearAllPoints()
		f.mover:SetPoint("CENTER", parent, "CENTER",f.opts.point[1] or 0,f.opts.point[2] or 0)					
		f.mover:SetSize( f.opts.w or 100 , f.opts.h or 20)
		
		f:ClearAllPoints()
		f:SetPoint("CENTER", f.mover,"CENTER", 0, 0)
	else	
		f = anchors[i]
		
	--	assert(opts, "NO OPTS FOUND")
		
		f.opts = opts
		
		f.mover:ClearAllPoints()
		f.mover:SetPoint("CENTER", parent, "CENTER",f.opts.point[1] or 0,f.opts.point[2] or 0)				
		f.mover:SetSize( f.opts.w or 100 , f.opts.h or 20)
		
		f:ClearAllPoints()
		f:SetPoint("CENTER", f.mover,"CENTER", 0, 0)
		f:SetSize( f.opts.w or 100 , f.opts.h or 20)
		
		SPTimers.AddMoverButtons(f.mover, f.opts)

		for i=1, #f.bars do
			f.free[f.bars[i]] = nil
		end
	end

	for i=1, f.opts.bar_number do
		local bar = f:CreateBar(i)
		f.bars[i] = bar
		f.free[bar] = true
	end

	SPTimers:UpdateMovers()
end


do
    local hour, minute = 3600, 60
    local format = string.format
    local ceil = math.ceil
	local floor = math.floor
	local fmod = math.fmod
	
	local formats = {
		function(s)  -- 1h, 2m, 119s, 29.9
			if s >= hour then
				return " %dh ", ceil(s / hour)
			elseif s >= minute*2 then
				return " %dm ", ceil(s / minute)
			elseif s >= 30 then
				return " %ds ", floor(s)
			end
			return " %.1f ", s
		end,
		function(s, dur) -- 1h, 2m, 119s / 300 , 29.99 / 300
			if s >= hour then
				return " %dh ", ceil(s / hour)
			elseif s >= minute*2 then
				return " %dm ", ceil(s / minute), dur
			elseif s >= 30 then
				return " %ds / %.0f ", floor(s), dur
			end
			return " %.2f / %.0f ", s, dur
		end,
		function(s) -- 1:11m, 59s, 10s, 1s
			if s <= 60 then
				return (" %.0fs "):format(s+0.1)
			else
				return (" %d:%0.2dm "):format(s/60, fmod(s, 60))
			end
		end,
		function(s) -- 1:11m, 59.1s, 10.2s, 1.1s
			if s <= 60 then
				return (" %.1fs "):format(s+0.1)
			else
				return (" %d:%0.2dm "):format(s/60, fmod(s, 60))
			end
		end,
		
		function(s)  -- 1, 2, 119, 29.9
			if s >= hour then
				return " %d ", ceil(s / hour)
			elseif s >= minute*2 then
				return " %d ", ceil(s / minute)
			elseif s >= 30 then
				return " %d ", floor(s)
			end
			return " %.1f ", s
		end,
		function(s, dur) -- 1, 2, 119 / 300 , 29.99 / 300
			if s >= hour then
				return " %d ", ceil(s / hour)
			elseif s >= minute*2 then
				return " %d ", ceil(s / minute), dur
			elseif s >= 30 then
				return " %d / %.2f ", floor(s), dur
			end
			return " %.2f / %.2f ", s, dur
		end,
		function(s) -- 1:11, 59, 10, 1
			if s <= 60 then
				return (" %.0f "):format(s+0.1)
			else
				return (" %d:%0.2d "):format(s/60, fmod(s, 60))
			end
		end,
		function(s) -- 1:11, 59.1, 10.2, 1.1
			if s <= 60 then
				return (" %.1f "):format(s+0.1)
			else
				return (" %d:%0.2d "):format(s/60, fmod(s, 60))
			end
		end,
	}
	
    function SPTimers.FormatTime(f, s, dur)
		local t = f.parent.opts.fortam_s or 1
		
		return formats[t](s, dur)
    end
end

do
	local gsub = gsub
	local find = string.find

	local hidegametooltip = CreateFrame("Frame")
	hidegametooltip:Hide()
	local gametooltip = CreateFrame("GameTooltip", "SPTimersGameToolTip2", nil, "GameTooltipTemplate");
	gametooltip:SetOwner( hidegametooltip,"ANCHOR_NONE");
	
	local auratypes = {
		['DEBUFF']	= 'HARMFUL',
		['BUFF']	= 'HELPFUL',
		['HARMFUL'] = 'HARMFUL',
		['HELPFUL'] = 'HELPFUL',
	}
	
	local function CacheValues(timer)
		
		gametooltip:ClearLines()
		
		local guid = timer.destUnit and UnitGUID(timer.destUnit)
			
		if guid == timer.targetGUID then
			gametooltip:SetUnitAura(timer.destUnit, timer.spellName, nil, auratypes[timer.aura_type]) --, auratypes[timer.aura_type]

			local line = _G[gametooltip:GetName().."TextLeft2"]:GetText()
			
			if line then 
				timer.value_cache = timer.value_cache or {}
				
				for val in gmatch(line, "(%d+)") do
					tinsert(timer.value_cache, val)
				end	
			end
		end
	end
	
	local function GetValues(timer, i)
		
		if not timer.value_cache then
			CacheValues(timer)
		end
		
		if timer.value_cache and timer.value_cache[i] then return timer.value_cache[i] end
		return "No Value"
	end

	function SPTimers.CustomTextCreate(self)		
		local text = self.customText
	--	if find(text, "%%tN") then
			if self.parent.opts.short then		
				text = text:gsub("%%tN", SPTimers:getShort(self.targetName) or UNKNOWN)
			else
				text = text:gsub("%%tN", self.targetName or UNKNOWN)
			end
	--	end
	--	if find(text, "%%spell") then
			if self.parent.opts.short then		
				text = text:gsub("%%spell", SPTimers:getShort(self.spellName))
			else
				text = text:gsub("%%spell", self.spellName)
			end
	--	end
	--	if find(text, "%%sN") then
			if self.parent.opts.short then		
				text = text:gsub("%%sN", SPTimers:getShort(self.sourceName) or UNKNOWN)
			else
				text = text:gsub("%%sN", self.sourceName or UNKNOWN)
			end
	--	end
		
		if SPTimers.AffDotsParse then
--		and find(text, "%%affdots") then
			text = text:gsub("%%affdots", SPTimers:AffDotsParse(self))
		end
		
	--	if find(text, "%%stacks") then
			text = text:gsub("%%stacks", self.stacks_string)
	--	end
		
	--	if find(text, "%%val1") then
			text = text:gsub("%%val1", GetValues(self, 1))
	--	end
		
	--	if find(text, "%%val2") then
			text = text:gsub("%%val2", GetValues(self, 2))
	--	end
		
	--	if find(text, "%%val3") then
			text = text:gsub("%%val3", GetValues(self, 3))
	--	end

		
	--	if find(text, "%%tickcount") and 
		
		if self._ticks_left then
			text = text:gsub("%%tickcount", self._ticks_left)
		end
		
		return text
	end
end

























































































































































function SPTimers.UpdateStatusBar(self, elapsed)
	self.elapsed = self.elapsed + elapsed



	if self.duration > 0 then
	
		local curtime = GetTime()
		
		local val = self.endTime - curtime




		self.bar:SetValue(val)


























		if val > 0 then


			if ( not self.do_onshow ) and floor(self.startTime+0.5) == floor(curtime+0.5) then
				self.do_onhide = nil
				SPTimers:PlaySound(self.spellID, "sound_onshow", self)
			end


				
			self.timeText:SetFormattedText(SPTimers.FormatTime(self, val, self.duration))
			
			if ( options.background_fading ) then
				local c = self.duration * 0.2 
				if c > val then
					self:bgFade(val, c) --val/c)
				end
			end
			
			if options.tick_count_on_stacks then
				self:SetCount("tick")
			end
		
		else
			if ( options.background_fading ) then
				self.bar.bg3:Hide()
				self.bg3_fade:Stop()
			end
			self.timeText:SetFormattedText(SPTimers.FormatTime(self, 0.00, self.duration))




		end


		if SPTimers:IsChanneling(self.spellID) then
			local spellName = UnitChannelInfo("player")
			
			if not spellName or self.spellName ~= spellName then
				self:EndTimer()
				self.parent:SortTimers("UpdateStatusBar - IsChanneling")
				return
			end
		end
		
		if (val <= 0) then
			
			self.do_onshow = nil
			SPTimers:PlaySound(self.spellID, "sound_onhide", self)
			
			self:EndTimer("play")
			if not options.delayfading then self.parent:SortTimers("UpdateStatusBar - EndTimer by endTime") end
			return
		end

		if self.elapsed > 0.25 then
			self.elapsed = 0
			
			if self.customText then
				self.spellText:SetText(SPTimers.CustomTextCreate(self))
			end
		end
	end
end

function SPTimers:UpdateTargetBars()

	self:swaptimers()

	for k,v in pairs(anchors) do
		v:SortTimers("UpdateTargetBars")
	end
end

local function GQB(anchor)
	local a = 0	
	for _, v in pairs(anchor.queue) do
		if v then a = a+ 1 end
	end
	return a
end

local function GAB(anchor)
	local a = 0
	for _, v in pairs(anchor.active) do
		if v then a = a+ 1 end
	end
	return a
end
local function GFB(anchor)
	local a = 0
	for _, v in pairs(anchor.free) do
		if v then a = a+ 1 end
	end
	return a
end


function SPTimers.GetQueueBars(self)

	for guid, tbl in pairs(self.guid_groups) do		
		for i, frame in ipairs(tbl) do

			if frame.targetGUID ~= SPTimers.myTarget and
			   frame.targetGUID ~= SPTimers.myGUID  and
			   frame.targetGUID ~= SPTimers.lastmyTarget then

				self.queue[frame] = frame
				self.free[frame] = true
				
				return frame
			end
		end
	end		
end

function SPTimers.RestoreQueue(self)
	
	if self.endTime < GetTime() then
		
		self.parent.active[self] = true
		
		self.parent:SortTimers("RestoreQueue Timer")
	end
	
	self.parent.queue[self] = nil
end

function SPTimers.ActivateTimer(self,duration,endTime,name,spellID,icon,aura_type,count,destGUID,sourceGUID,destUnit,sourceUnit,targetName,sourceName,raidIndex,sourceFunc,showticks,aura_number, realindex)
	
	if not options.bar_module_enabled then return end
	if not duration then return end
	local f
	
	if aura_number and aura_number > 0 then
		f = self:GetTimer(destGUID, spellID, sourceGUID, aura_type, aura_number, duration, endTime)
	else
		f = self:GetTimer(destGUID, spellID, sourceGUID, aura_type)
	end
	
--	print("ActivateTimer", name, aura_type, f, sourceFunc,spellID, f and f.spellID, f and f.spellName)
	
	if f then	
		if duration ~= 0 and endTime ~= 0 then
			f:SetTimer(duration,endTime,name,spellID,icon,aura_type,count,destGUID,sourceGUID,destUnit,sourceUnit,targetName,sourceName,raidIndex,sourceFunc,showticks,aura_number, realindex)
		else
		--	count = SPTimers:GetCheckStacks(spellID) or count
			f:ResetBarSize()
			f:SetCount(count)
			f:SetMark(raidIndex)
			f.castTime	= SPTimers:GetCastTime(spellID)
			f.updated = true
		end
	else
		f = next(self.free)
		
		if not f then
			
			if destGUID == SPTimers.myTarget or 
			   destGUID == SPTimers.myGUID or
			   destUnit == "target" or
			   destUnit == "player" then			

				f = SPTimers.GetQueueBars(self)
				
				if not f then return end				
			--	print("QUEUE BAR MAKE FREE", GQB(self), GAB(self), GFB(self) )
			else
			--[[
				if sourceFunc ~= "CLEU" and destUnit ~= "mouseover" then
					assert(false, format("Not found free bar for %s,%s,%s,%s,%s,%s,%s", tostring(name), tostring(destUnit), tostring(sourceUnit), tostring(aura_type),tostring(sourceFunc), tostring(targetName), tostring(sourceName)))
				end
				]]
				return
			end
		end
		
		self.active[f] = true
		
		if duration == 0 and endTime == 0 then
			f:ToInfinite(duration,endTime,name,spellID,icon,aura_type,count,destGUID,sourceGUID,destUnit,sourceUnit,targetName,sourceName,raidIndex,sourceFunc,showticks,aura_number, realindex)
		else
			f:SetTimer(duration,endTime,name,spellID,icon,aura_type,count,destGUID,sourceGUID,destUnit,sourceUnit,targetName,sourceName,raidIndex,sourceFunc,showticks,aura_number, realindex)
		end
		
		self:SortTimers("ActivateTimer - new timer")
	end
	
	return f
end


function SPTimers.ActivateTimer2(self,duration,endTime,name,spellID,icon,aura_type,count,destGUID,sourceGUID,destUnit,sourceUnit,targetName,sourceName,raidIndex,sourceFunc,showticks,aura_number, realindex)
	
	if not options.bar_module_enabled then return end
	if not duration then return end
	
	local f  = next(self.free)
	
	if not f then		
		if	destGUID == SPTimers.myTarget or 
			destGUID == SPTimers.myGUID or
			destUnit == "target" or
			destUnit == "player" then			
			f = SPTimers.GetQueueBars(self)
					
			if not f then return end
					
		--	print("QUEUE BAR MAKE FREE", GQB(self), GAB(self), GFB(self) )
		else
			--assert(false, format("Not found free bar for %s,%s,%s,%s,%s,%s,%s", tostring(name), tostring(destUnit), tostring(sourceUnit), tostring(aura_type),tostring(sourceFunc), tostring(targetName), tostring(sourceName)))
			return
		end
	end
	
	self.active[f] = true
		
	if duration == 0 and endTime == 0 then
		f:ToInfinite(duration,endTime,name,spellID,icon,aura_type,count,destGUID,sourceGUID,destUnit,sourceUnit,targetName,sourceName,raidIndex,sourceFunc,showticks,aura_number,realindex)
	else
		f:SetTimer(duration,endTime,name,spellID,icon,aura_type,count,destGUID,sourceGUID,destUnit,sourceUnit,targetName,sourceName,raidIndex,sourceFunc,showticks,aura_number,realindex)
	end
	self:SortTimers("ActivateTimer2 - new tiemr")

	return f
end

function SPTimers.SwapTimers(self, timer)

	if not options.bar_module_enabled then return end
	if timer.on_fading then return end
	if timer.isdead then return end
	
	local f

	if timer.aura_number and timer.aura_number > 0 then
		f = self:GetTimer(timer.targetGUID, timer.spellID, timer.sourceGUID, timer.aura_type, timer.aura_number, timer.duration, timer.endTime)
	else
		f = self:GetTimer(timer.targetGUID, timer.spellID, timer.sourceGUID, timer.aura_type)
	end
	
	--print("SwapTimers", name, aura_type, f, sourceFunc)
	
	if f then
		
		if timer.duration ~= 0 and timer.endTime ~= 0 then
			f:SetTimer(timer.duration,timer.endTime,timer.spellName,timer.spellID,timer.icon_file,timer.aura_type,timer.count,timer.targetGUID,
					   timer.sourceGUID,timer.destUnit,timer.sourceUnit,timer.targetName,timer.sourceName,timer.raidIndex,timer.sourceFunc,timer.showticks,timer.aura_number, timer.realindex)

			f.value_cache = deepcopy(timer.value_cache)
		else
		--	count = SPTimers:GetCheckStacks(spellID) or count
			f.value_cache = deepcopy(timer.value_cache)
			f:ResetBarSize()
			f:SetCount(timer.count)
			f:SetMark(timer.raidIndex)
			f.castTime	= SPTimers:GetCastTime(timer.spellID)
			f.updated = true
		end
	else
		f = next(self.free)
		
		if not f then
			
			if timer.targetGUID == SPTimers.myTarget or 
			   timer.targetGUID == SPTimers.myGUID or
			   timer.destUnit == "target" or
			   timer.destUnit == "player" then			

				f = SPTimers.GetQueueBars(self)
				
				if not f then return end				
			--	print("QUEUE BAR MAKE FREE", GQB(self), GAB(self), GFB(self) )
			else
				--[[
				if timer.sourceFunc ~= "CLEU" and timer.destUnit ~= "mouseover" then
					assert(false, format("Not found free bar for %s,%s,%s,%s,%s,%s,%s", tostring(timer.spellName), tostring(timer.destUnit), tostring(timer.sourceUnit), tostring(timer.aura_type),tostring(timer.sourceFunc), tostring(timer.targetName), tostring(timer.sourceName)))
				end
				]]
				return
			end
		end
		
		self.active[f] = true

		if timer.duration == 0 and timer.endTime == 0 then
			f:ToInfinite(timer.duration,timer.endTime,timer.spellName,timer.spellID,timer.icon_file,timer.aura_type,timer.count,timer.targetGUID,
						timer.sourceGUID,timer.destUnit,timer.sourceUnit,timer.targetName,timer.sourceName,timer.raidIndex,timer.sourceFunc,timer.showticks,timer.aura_number,timer.realindex)
		else
			f:SetTimer(timer.duration,timer.endTime,timer.spellName,timer.spellID,timer.icon_file,timer.aura_type,timer.count,timer.targetGUID,
						timer.sourceGUID,timer.destUnit,timer.sourceUnit,timer.targetName,timer.sourceName,timer.raidIndex,timer.sourceFunc,timer.showticks,timer.aura_number,timer.realindex)
		end
		
		f.value_cache = deepcopy(timer.value_cache)
		
		f:ClearAllPoints()
		timer:ClearAllPoints()
		
		timer:EndTimer()
		
		self:SortTimers("SwapTimers - new timer")
	end

	return f
end

function SPTimers.AppliedDose(self,duration,endTime,name,spellID,icon,aura_type,count,destGUID,sourceGUID,destUnit,sourceUnit,targetName,sourceName,raidIndex,sourceFunc,showticks,aura_number,realindex)
	if not options.bar_module_enabled then return end
	if not duration then return end
	local f = self:GetTimer(destGUID, spellID, sourceGUID, aura_type)
	
	if f then f:SetCount(count) end
	return f
end

do
	local fadetimer = 0.5
	local fadeframes = {}
	local trottle = 0
	
	local function OnFader(self, elapsed)
		
		for i,timer in ipairs(fadeframes) do
			timer.fading = timer.fading+elapsed
	
			if timer.fading > fadetimer then			
				if not timer.anim:IsPlaying() and timer.alreadyfade then				
					timer.alreadyfade = nil
					timer.fading = 0
					timer:FadeOut()
					tremove(fadeframes, i)	
				end
			end
		end
		
		if #fadeframes == 0 then 
			self:Hide() 
			return 
		end
	end
	
	local fader = CreateFrame("Frame")
	fader:SetScript("OnUpdate", OnFader)
	fader:Hide()
	function SPTimers.Fader(self)

		if not self.alreadyfade then
			self.alreadyfade = true
		--	print("1 SPTimers.Fader")
			self.fading = 0
			tinsert(fadeframes, self)			
			fader:Show()
			OnFader(fader, 0)
		end
	end
	
	function SPTimers.RemoveFader(self)
		
		if self.alreadyfade then	

			self.alreadyfade = nil
			self.fading = 0
			--[[
			if self.endTime < GetTime() then return end
		
			fader:Hide()
			for i, frame in ipairs(fadeframes) do			
				if frame == self then	
					tremove(fadeframes, i)
					
					print("REMOVE FROM FADER 2")
					break
				end
			end
			fader:Show()
			]]
		end
	end
end

--[[

	 self.startTime 		= curtime
    self.endTime		= self.startTime + 60
	self.elapsed 		= 0	
	self.targetGUID		= destGUID
	self.targetName 	= targetName
	self.sourceGUID 	= sourceGUID
	self.sourceName 	= sourceName
	self.destUnit		= destUnit
	self.sourceUnit 	= sourceUnit
	self.spellID 		= spellID
	self.spellName  	= name
	self.aura_type		= aura_type
	self.sourceFunc		= sourceFunc
	self.raidIndex  	= raidIndex
	self.priority		= SPTimers:GetPriority(spellID)
	self.group 			= SPTimers:GetGroup(spellID)
	self.castTime		= SPTimers:GetCastTime(spellID)
	self.customText		= SPTimers:GetCustomText(spellID)
	self.aura_number 	= aura_number
	self.targetType  	= SPTimers:GetTargetType(spellID)
	self.updated		= true
	self.stacks_string  = ""
	self.realindex		= realindex	
	self.tickOverlap	= SPTimers:TickOverlap(spellID) 
	self.alreadyfade	= false
	self.icon_file		= icon
	self.count			= count
	self.icon:SetTexture(icon)
	self.icon2:SetTexture(icon)
	self.isdead			= nil
	self.on_fading 		= nil
]]
function SPTimers_CheckNewTimer(timer, isend)

end


function SPTimers.EndTimer(self, fader, dored)
	
	
	if skipfader == "fade" then
	
	elseif fader == "play" and options.delayfading then
		if dored then


			if abs(self.endTime-GetTime()) > 0.2 then
			--	print(self.endTime, GetTime(), self.endTime-GetTime())			
				self.bar:SetStatusBarColor(1, 0, 64/255, 1)
			end
		end
		return SPTimers.Fader(self)
	end

	self.parent.free[self] = true
	
	
	if self.parent.queue[self] then 
		return self.parent.queue[self]:RestoreQueue() 
	end
	
	if options.adapttoonemax then
		self.parent:GetHighestStartTime()
	end



	self.duration 		= 0
	self.startTime		= 0
	self.endTime 		= 0
	self.targetGUID 	= nil
	self.targetName 	= nil
	self.sourceGUID 	= nil
	self.sourceName 	= nil
	self.destUnit		= nil
	self.sourceUnit 	= nil
	self.spellID 		= nil
	self.spellName  	= nil
	self.aura_type		= nil
	self.sourceFunc		= nil
	self.raidIndex  	= nil
	self.priority		= nil
	self.group 			= nil
	self.castTime		= nil
	self.customText		= nil
	self.aura_number 	= nil
	self.realindex		= 0
	self.targetType 	= 3
	self.updated		= true
	self.stacks_string  = ""
	self.value_cache 	= nil
	self.tickOverlap	= nil
	self.on_fading		= nil


	





	self.do_onshow = nil
	self.do_onhide = nil
	
	for i=1, #self.tiks do
		self.tiks[i]:Hide()
	end
end


function SPTimers:RemoveTimers(destGUID, auratype, isdead)
	for k,v in pairs(anchors) do
		local f = v:GetTimerByGUID(destGUID, auratype)
		
		if f then
			for t in pairs(f) do
			--	SPTimers.Fader(f, "RemoveTimers by DEST_GUID")
				t:IsDead(isdead)
				t:EndTimer("play", "dored")
			end
			
			v:SortTimers("RemoveTimers by DEST_GUID")
		end
	end
end

function SPTimers:OnCombatEndReset()
	for k,v in pairs(anchors) do
		for k in pairs(v.active) do
			if k.aura_type == "DEBUFF" then			   
				k:EndTimer()
			end
		end	
		
		v:SortTimers("OnCombatEndReset")
	end
end

function SPTimers:RemoveTimersGUID(destGUID, spellID, sourceGUID, aura_type, skipplay)
	for k,v in pairs(anchors) do
		local f = v:GetTimer(destGUID, spellID, sourceGUID, aura_type)
		if f then 
		
		--	SPTimers.Fader(f, "RemoveTimersGUID by SPELLID")
			
			if skipplay then
				f:EndTimer() 
			else
				f:EndTimer("play", "dored") 
			end
			if not options.delayfading then v:SortTimers("RemoveTimersGUID by SPELLID") end
		end
	end
end

function SPTimers:RemoveTimersBySourceGUID(spellID, sourceGUID, skipplay)
	for k,v in pairs(anchors) do
		local f = v:GetTimerSourceGUID(spellID, sourceGUID)
			
		--	print("RemoveTimersBySourceGUID", spellID, sourceGUID, skipplay, f)
		if f then 
		
		--	SPTimers.Fader(f, "RemoveTimersGUID by SPELLID")
			
			if skipplay then
				f:EndTimer() 
			else
				f:EndTimer("play", "dored") 
			end
			if not options.delayfading then v:SortTimers("RemoveTimersGUID by SPELLID") end
		end
	end
end


do
	-- UTF-8 Reference:
	-- 0xxxxxxx - 1 byte UTF-8 codepoint (ASCII character)
	-- 110yyyxx - First byte of a 2 byte UTF-8 codepoint
	-- 1110yyyy - First byte of a 3 byte UTF-8 codepoint
	-- 11110zzz - First byte of a 4 byte UTF-8 codepoint
	-- 10xxxxxx - Inner byte of a multi-byte UTF-8 codepoint
	
	local char
	local string_byte = string.byte
	local sub = sub
	
	local function chsize(char)
		if not char then
			return 0
		elseif char > 240 then
			return 4
		elseif char > 225 then
			return 3
		elseif char > 192 then
			return 2
		else
			return 1
		end
	end
	 
	-- This function can return a substring of a UTF-8 string, properly handling
	-- UTF-8 codepoints.  Rather than taking a start index and optionally an end
	-- index, it takes the string, the starting character, and the number of
	-- characters to select from the string.
	 
	local function utf8sub(str, startChar, numChars)
	  local startIndex = 1
	  while startChar > 1 do
		  local char = string_byte(str, startIndex)
		  startIndex = startIndex + chsize(char)
		  startChar = startChar - 1
	  end
	 
	  local currentIndex = startIndex
	 
	  while numChars > 0 and currentIndex <= #str do
		local char = string_byte(str, currentIndex)
		currentIndex = currentIndex + chsize(char)
		numChars = numChars -1
	  end
	  return str:sub(startIndex, currentIndex - 1)
	end


	local shortCache = {}
	local tinsert = table.insert
	local gmatch = string.gmatch
	function SPTimers:getShort(text)
		
		if not shortCache[text] then
			local msg = ""
			local tbl = {}
			
			local tbl = {}
			for v in gmatch(text, "[^ :\"-]+") do
			  tinsert(tbl, v)
			end
			
			
			if #tbl > 1 then	
				for k,v in ipairs(tbl) do
					msg = msg..utf8sub(v, 1, 1)
				end
			else
				for k,v in ipairs(tbl) do
					msg = msg..v
				end
			end
			
			shortCache[text] = msg
			
			return shortCache[text]
		else
			return shortCache[text]
		end
	end	
end



function SPTimers.UpdateBarColor(self)

	local cColor = SPTimers:GetColor(self.spellID)
	
	if cColor then	
		self.bar:SetStatusBarColor(cColor[1], cColor[2], cColor[3], 1)
	else
		self.bar:SetStatusBarColor(self.opts.bar.color[1], self.opts.bar.color[2], self.opts.bar.color[3], self.opts.bar.color[4])
	end
	
	local r,g,b,a = self.bar:GetStatusBarColor()
	self.bar.overlay2:SetTexture(r+0.4,g+0.4,b+0.4,0.8)		
	self.bar.bg3:SetTexture(r,g,b,0.7)
	
	if ( options.back_bar_color ) then
		self.bar.bg2:SetVertexColor(r*0.8,g*0.8,b*0.8,self.opts.bar.bgcolor[4])
	else
		self.bar.bg2:SetVertexColor(self.opts.bar.bgcolor[1], self.opts.bar.bgcolor[2], self.opts.bar.bgcolor[3], self.opts.bar.bgcolor[4])
	end
	
	for i=1, #self.tiks do		
		self.tiks[i]:SetVertexColor(1,1,1,1)	
	end
	
	r,g,b,a = self.bar.bg2:GetVertexColor()
		
	self.bar.overlay1:SetTexture(r+0.4,g+0.4,b+0.4,0.8)
end

function SPTimers.UpdateBarSize(self)
	
	self:SetSize(self.opts.w or 100 , self.opts.h or 20)
	self.parent:SetSize( self.opts.w or 100 , self.opts.h or 20)
	self.parent.mover:SetSize( self.opts.w or 100 , self.opts.h or 20)
	
	self.ic:SetSize(self.opts.h or 20, self.opts.h or 20)
	self.ic2:SetSize(self.opts.h or 20, self.opts.h or 20)

end

function SPTimers.BarTextUpdate(self)
	if self.customText then
		self.spellText:SetText(SPTimers.CustomTextCreate(self))
	elseif self.parent.opts.target_name and self.aura_type ~= "BUFF" then
		if self.parent.opts.short then	
			if self.parent.opts.debug_info then
				self.spellText:SetText(SPTimers:getShort(self.targetName).." #"..self.spellID .."@"..  self.sourceFunc .." @"..( self.destUnit or "unknown"))
			else
				self.spellText:SetText(SPTimers:getShort(self.targetName))
			end
		else
			if self.parent.opts.debug_info then
				self.spellText:SetText(self.targetName.." #"..self.spellID .."@"..  self.sourceFunc .." @"..( self.destUnit or "unknown"))
			else
				self.spellText:SetText(self.targetName)
			end
		end
	else
		if self.parent.opts.short then
			if self.parent.opts.debug_info then
				self.spellText:SetText(SPTimers:getShort(self.spellName).." #"..self.spellID .."@"..  self.sourceFunc .." @"..( self.destUnit or "unknown"))
			else
				self.spellText:SetText(SPTimers:getShort(self.spellName))
			end
		else
			if self.parent.opts.debug_info then
				self.spellText:SetText(self.spellName.." #"..self.spellID .."@"..  self.sourceFunc)
			else
				self.spellText:SetText(self.spellName)
			end
		end
	end
end

function SPTimers.SetTimer(self,duration,endTime,name,spellID,icon,aura_type,count,destGUID,sourceGUID,destUnit,sourceUnit,targetName,sourceName,raidIndex,sourceFunc,showticks,aura_number,realindex)
	
	--print("SetTimer",args.duration,args.name,args.spellID,args.destGUID,args.sourceGUID,args.destUnit,args.sourceUnit,args.expirationTime,args.icon,args.count)
	if not options.bar_module_enabled then return end
	
	
	if ( self.endTime and self.endTime > 0 and self.endTime ~= endTime ) or endTime == 0 then 
		self.value_cache = nil
		-- print(name, spellID, "Remove Value Cache")
	end

	if endTime then
		self.startTime = endTime - duration
		self.endTime = endTime
	else
		self.startTime = GetTime()
		self.endTime = self.startTime + duration
	end
	


	self.duration 		= duration	
	self.elapsed 		= 0
	self.targetGUID 	= destGUID
	self.targetName 	= targetName
	self.sourceGUID 	= sourceGUID
	self.sourceName 	= sourceName
	self.destUnit		= destUnit
	self.sourceUnit 	= sourceUnit
	self.spellID 		= spellID
	self.spellName  	= name
	self.aura_type		= aura_type
	self.sourceFunc		= sourceFunc
	self.raidIndex  	= raidIndex
	self.priority		= SPTimers:GetPriority(spellID)
	self.group 			= SPTimers:GetGroup(spellID)
	self.castTime		= SPTimers:GetCastTime(spellID)
	self.customText		= SPTimers:GetCustomText(spellID)
	self.aura_number 	= aura_number








	self.targetType  	= SPTimers:GetTargetType(spellID)
	self.updated		= true
	self.stacks_string  = ""
	self.realindex		= realindex or 0
	self.tickOverlap	= SPTimers:TickOverlap(spellID) 
	self.icon_file		= icon
	self.count			= count
	self.isdead			= nil
	self.on_fading 		= nil

	
	self:ResetBarSize()


	self.icon:SetTexture(icon)
	self.icon2:SetTexture(icon)

	self:SetMark(raidIndex)

	
	if SPTimers.onUpdateHandler.active and destGUID and not ( aura_type == "SPELL_CAST" or aura_type == "SPELL_SUMMON" or aura_type == "SPELL_ENERGIZE" ) then
		SPTimers.targetEngaged[destGUID] = GetTime()+options.engageThrottle
	end


	self.showticks  = showticks
	self:SetCount(count)

	
	self:UpdateBarColor()
	self:BarTextUpdate()

	
	if options.adapttoonemax then
		self.parent:GetHighestStartTime()
	else
		if options.maximumtime then
			self.bar:SetMinMaxValues(0, options.maximumtime_value)
		else
			self.bar:SetMinMaxValues(0, duration)
		end
		
		if showticks then
			self:SetTicks()




		end
	end
	








	self:Show()	
end


function SPTimers.ToInfinite(self,duration,endTime,name,spellID,icon,aura_type,count,destGUID,sourceGUID,destUnit,sourceUnit,targetName,sourceName,raidIndex,sourceFunc,showticks,aura_number,realindex)
	if not options.bar_module_enabled then return end
	
    self.bar:SetMinMaxValues(0,100)
    self.bar:SetValue(100)
	self.duration = 0
	
	local curtime = GetTime()
	
	if ( self.startTime and self.startTime > 0 and self.startTime ~= curtime ) or startTime == 0 then 
		self.value_cache = nil
		-- print(name, spellID, "Inf Remove Value Cache")
	end

    self.startTime 		= curtime
    self.endTime		= self.startTime + 60
	self.elapsed 		= 0	
	self.targetGUID		= destGUID
	self.targetName 	= targetName
	self.sourceGUID 	= sourceGUID
	self.sourceName 	= sourceName
	self.destUnit		= destUnit
	self.sourceUnit 	= sourceUnit
	self.spellID 		= spellID
	self.spellName  	= name
	self.aura_type		= aura_type
	self.sourceFunc		= sourceFunc
	self.raidIndex  	= raidIndex
	self.priority		= SPTimers:GetPriority(spellID)
	self.group 			= SPTimers:GetGroup(spellID)
	self.castTime		= SPTimers:GetCastTime(spellID)
	self.customText		= SPTimers:GetCustomText(spellID)
	self.aura_number 	= aura_number
	self.targetType  	= SPTimers:GetTargetType(spellID)
	self.updated		= true
	self.stacks_string  = ""
	self.realindex		= realindex	
	self.tickOverlap	= SPTimers:TickOverlap(spellID) 

	self.icon_file		= icon
	self.count			= count
	self.icon:SetTexture(icon)
	self.icon2:SetTexture(icon)
	self.isdead			= nil
	self.on_fading 		= nil



	self:ResetBarSize(true)










    self.timeText:SetText("")

	self:SetMark(raidIndex)

	if SPTimers.onUpdateHandler.active and destGUID then
		SPTimers.targetEngaged[destGUID] = curtime+options.engageThrottle
	end
	
	self.showticks  = showticks
	
	self:SetCount(count)
	
	self:BarTextUpdate()

	
	if options.adapttoonemax then
		self.parent:GetHighestStartTime()
	else
		if showticks then
			self:SetTicks()




		end
	end
	
	self:UpdateBarColor()
	self:Show()
end

function SPTimers.SetCount(self, count)
	
	if count ~= "tick" then
		count = SPTimers:GetCheckStacks(self.spellID) or ( count and count or 0 )
			
		self.stacks_string = count > 1 and count or ""
		
		self.count = count
	end
	
	if self.showticks and ( count == "tick" or options.tick_count_on_stacks ) then
	
		self.stacktext:SetText(self._ticks_left)
		self.stacktext2:SetText(self._ticks_left)
	
	elseif count ~= "tick" then

		self.stacktext:SetText(self.stacks_string)
		self.stacktext2:SetText(self.stacks_string)
	
	end
end

do
	local raidIndexCoord = {
		[1] = { 0, .25, 0, .25 }, --"STAR"
		[2] = { .25, .5, 0, .25}, --MOON
		[3] = { .5, .75, 0, .25}, -- CIRCLE
		[4] = { .75, 1, 0, .25}, -- SQUARE
		[5] = { 0, .25, .25, .5}, -- DIAMOND
		[6] = { .25, .5, .25, .5}, -- CROSS
		[7] = { .5, .75, .25, .5}, -- TRIANGLE
		[8] = { .75, 1, .25, .5}, --  SKULL
	}
	
	
	--[[
	{ text = RAID_TARGET_1, tCoordLeft = 0, tCoordRight = 0.25, tCoordTop = 0, tCoordBottom = 0.25 };
	{ text = RAID_TARGET_2, tCoordLeft = 0.25, tCoordRight = 0.5, tCoordTop = 0, tCoordBottom = 0.25 };
	{ text = RAID_TARGET_3, tCoordLeft = 0.5, tCoordRight = 0.75, tCoordTop = 0, tCoordBottom = 0.25 };
	{ text = RAID_TARGET_4, tCoordLeft = 0.75, tCoordRight = 1, tCoordTop = 0, tCoordBottom = 0.25 };
	{ text = RAID_TARGET_5, tCoordLeft = 0, tCoordRight = 0.25, tCoordTop = 0.25, tCoordBottom = 0.5 };
	{ text = RAID_TARGET_6, tCoordLeft = 0.25, tCoordRight = 0.5, tCoordTop = 0.25, tCoordBottom = 0.5 };
	{ text = RAID_TARGET_7, tCoordLeft = 0.5, tCoordRight = 0.75, tCoordTop = 0.25, tCoordBottom = 0.5 };
	{ text = RAID_TARGET_8, tCoordLeft = 0.75, tCoordRight = 1, tCoordTop = 0.25, tCoordBottom = 0.5 };
	
	]]

	function SPTimers.SetMark(self, mark)
		self.raidIndex	= mark
		if ( mark and mark > 0 and mark < 9 ) and options.show_mark then
			if self.raidMark:IsShown() then
				self.raidMark:SetTexCoord(raidIndexCoord[mark][1],raidIndexCoord[mark][2],raidIndexCoord[mark][3],raidIndexCoord[mark][4])
			else
				self.raidMark:Show()
				self.raidMark:SetTexCoord(raidIndexCoord[mark][1],raidIndexCoord[mark][2],raidIndexCoord[mark][3],raidIndexCoord[mark][4])
			end
		else
			self.raidMark:Hide()
		end
	end
end	
		
do
	local xOffset = 0
	local yOffset
	local point
	local to, to2
	local ySign
	
	local tsort = table.sort 
	local pairs = pairs
	local ipairs = ipairs
	
	local sorting_functions = {
		function(x,y)		-- 1 priority from lower to upper
			if x.priority == y.priority then
				if x.duration == y.duration then
					return x.endTime > y.endTime
				else
					return x.duration > y.duration 
				end
			else
				return x.priority < y.priority
			end
		end,
		function(x,y)		-- 2 endtime from upper to lower
			if x.endTime == y.endTime then
				return x.duration < y.duration
			else
				return x.endTime < y.endTime 
			end		
		end,
		function(x,y)		-- 3 endtime from lower to upper
			if x.endTime == y.endTime then
				return x.duration > y.duration
			else
				return x.endTime > y.endTime 
			end		
		end,
	}

	function SPTimers:updateSortings()

		for k,v in pairs(anchors) do
			v:ResetBarPositions()
			v:SortTimers("updateSortings")
		end	
	end
	
	function SPTimers:UpdateFormatTexts(anchor)
		for k,v in pairs(anchors) do
			
			if v.id == anchor then
				
				v.opts = self.db.profile.bars_anchors[anchor]
				
				for timer in pairs(v.active) do	
					timer:BarTextUpdate()
				end
			
			end
		end
	end
	
	--SPTimers:GetOffAnchor(spellID)
	
	function SPTimers:swaptimers()
	
		if not options.doswap then return end
		
		for i, anchor in ipairs(anchors) do
			for timer in pairs(anchor.active) do
			
				local main_anchor = SPTimers:GetAnchor(timer.spellID, timer.targetGUID)
				local offtarget_a = SPTimers:GetOffAnchor(timer.spellID, timer.targetGUID) 
				
				if offtarget_a and offtarget_a ~= main_anchor then
					if  self.myTarget ~= timer.targetGUID then
						anchors[offtarget_a]:SwapTimers(timer)
					elseif self.myTarget == timer.targetGUID then
						anchors[main_anchor]:SwapTimers(timer)
					end
				end
			end
		end
	end
	
	
	
--	local lastUpdate = 0
	
	--local tranTimer()
	function SPTimers.SortTimers(self, event)
		if not options.bar_module_enabled then return end
		-- self.targetGUID
		-- self.duration
		
	--	print("SPTimers.SortTimers in ", GetTime()-lastUpdate, " by ", event)
		
	--	lastUpdate = GetTime()
		
		point = ( not self.opts.add_up and "TOPLEFT" ) or "BOTTOMLEFT"
		to = ( not self.opts.add_up and "BOTTOMLEFT" ) or "TOPLEFT"
		to2 = ( not self.opts.add_up and "BOTTOMLEFT" ) or "TOPLEFT"
		
		ySign = ( not self.opts.add_up and -1 ) or 1
		yOffset = ( 0 )+self.opts.gap --self.opts.h

		for g,tbl in pairs(self.groups) do
		   wipe(tbl)
		end
		wipe(self.guid_groups)
		
		local playerTimers = self.groups.player
		local targetTimers = self.groups.target
		local targetGUID = UnitGUID("target")

		for timer in pairs(self.active) do
			
			timer:ClearAllPoints()
			
			timer.checkTarget = targetGUID
			local group = timer.group
			
			local main_anchor = SPTimers:GetAnchor(timer.spellID, timer.targetGUID)
			local offtarget_a = SPTimers:GetOffAnchor(timer.spellID, timer.targetGUID)
			
			if timer.sourceFunc == "TEST_BAR" then				
				if timer.targetGUID == "group1" then
					playerTimers[#playerTimers+1] = timer
				elseif timer.targetGUID == "group2" then
					targetTimers[#targetTimers+1] = timer
				else
					self.guid_groups[timer.targetGUID] = self.guid_groups[timer.targetGUID] or {}
					
					self.guid_groups[timer.targetGUID][#self.guid_groups[timer.targetGUID]+1] = timer
				end
			elseif group then		
				if timer.targetType == 1 and timer.targetGUID ~= targetGUID then
					timer:EndTimer()
				else
					if not SPTimers:UnitFilter_TIMER(timer) then
						timer:EndTimer()
					else
						self.groups[group] = self.groups[group] or {}
						self.groups[group][#self.groups[group]+1] = timer
					end
				--	print("1", group, targetGUID, timer.targetGUID, myGUID, timer.isbuff)
				end
			elseif timer.targetGUID == SPTimers.myGUID then
				playerTimers[#playerTimers+1] = timer
			--	table.insert(playerTimers,timer)
			--	print("5", group, targetGUID, timer.targetGUID, myGUID, timer.isbuff)
			elseif options.doswap and ( timer.targetGUID == targetGUID or timer.targetGUID == "channel:spell" )  then
			
			--	if SPTimers:GetOffAnchor(timer.spellID)

				if main_anchor == self.id then
					if not SPTimers:UnitFilter_TIMER(timer) then
						timer:EndTimer()
					else
				--		table.insert(targetTimers,timer)
						targetTimers[#targetTimers+1] = timer
					end
				elseif offtarget_a and offtarget_a == self.id then
					timer:EndTimer("swapend")
				end
					-- table.insert(targetTimers,timer)
			--	print("doswap1", group, targetGUID, timer.targetGUID, myGUID, timer.isbuff)
			--	print("doswap2", offtarget_a, self.id, main_anchor)
				
			elseif not options.doswap and SPTimers:DoUnitAnchor(timer.spellID) and main_anchor and main_anchor ~= self.id and timer.targetGUID ~= "channel:spell" then
				
			--	print("nodoswap", group, targetGUID, timer.targetGUID, myGUID, timer.isbuff)
				if not SPTimers:UnitFilter_TIMER(timer) then
					timer:EndTimer()
				elseif main_anchor and main_anchor ~= self.id  then
					timer:EndTimer()
				end
				
			elseif timer.targetGUID == nil then
				if self.aura_type == "BUFF" then
			--		print("3", group, targetGUID, timer.targetGUID, myGUID, timer.isbuff)
			--		table.insert(playerTimers,timer)
			
					playerTimers[#playerTimers+1] = timer
				else
				--	print("4", group, targetGUID, timer.targetGUID, SPTimers.myGUID, timer.isbuff)
					if not SPTimers:UnitFilter_TIMER(timer) then
						timer:EndTimer()
					else
			--			table.insert(targetTimers,timer)
						targetTimers[#targetTimers+1] = timer
					end
				end
			else
			--	print("4", group, targetGUID, timer.targetGUID, SPTimers.myGUID, timer.isbuff, timer.targetType)
				if timer.targetType == 1 then
					timer:EndTimer()
				elseif not SPTimers:UnitFilter_TIMER(timer) then
					timer:EndTimer()
				else
					if options.doswap and offtarget_a and offtarget_a ~= self.id then
						anchors[offtarget_a]:SwapTimers(timer)
					elseif not options.doswap and SPTimers:DoUnitAnchor(timer.spellID) and main_anchor and main_anchor ~= self.id then
						anchors[main_anchor]:SwapTimers(timer)
					else
						self.guid_groups[timer.targetGUID] = self.guid_groups[timer.targetGUID] or {}
				--		table.insert(self.guid_groups[timer.targetGUID],timer)
				
						self.guid_groups[timer.targetGUID][#self.guid_groups[timer.targetGUID]+1] = timer
					end
				end
			end
			
		end


	--	local sort_func = self.opts.sort_func or 1
		local sort_func = sorting_functions[self.opts.sort_func or 1]
		
		for g,tbl in pairs(self.groups) do

			tsort(tbl,sort_func)
		end
		for g,tbl in pairs(self.guid_groups) do

			tsort(tbl,sort_func)
		end
			
		local anchor = self
        local aopts = self.opts
        local prev
        local gap = 0
			
		for _, gopts in pairs(aopts.sorting) do
            local gname = gopts.name
            local alpha = gopts.alpha

            if gname == "offtargets" then
                for guid, group_timers in pairs(self.guid_groups) do
                    for i,timer in ipairs(group_timers) do
                        local noswap_alpha = guid == targetGUID and 1 or alpha
                        timer:SetAlpha(noswap_alpha)
						
					--	if not prev then print("No prev", self.opts.add_up, gname, self.id, point, to2, xOffset, (yOffset+gap)*ySign) end
					--	if prev then print("Prev", self.opts.add_up, gname, self.id, point, to) end
						
						
                        timer:SetPoint(point, prev or anchor, prev and to or to2, xOffset,(yOffset+gap)*ySign)

                        prev = timer
						
                        gap = 0
                    end
                    gap = gopts.gap  -- 4 gap option
                end
				break
            else
                local group_timers = self.groups[gname]
                if group_timers then
					for i,timer in ipairs(group_timers) do
						timer:SetAlpha(alpha)
						
					--	if not prev then print("No prev", self.opts.add_up, gname, self.id, point, to2, xOffset, (yOffset+gap)*ySign) end
					--	if prev then print("Prev", self.opts.add_up, gname, self.id, point, to) end
						
						timer:SetPoint(point, prev or anchor, prev and to or to2, xOffset, (yOffset+gap)*ySign)

						prev = timer
						gap = 0
					end
                end
                gap = prev and gopts.gap or 0 -- 4 gap option
            end
		end
	end
end

function SPTimers.Update_Bar(f)
	local opt = f:GetOpts() -- local opt = f.parent.opts or f.opts
--	local opt = f.opts
	
--	print("Update_Bar", opt, opt2, f:GetParent():GetName())
	local height = opt.h
	local width = opt.w

	f.ic:SetWidth(height)
    f.ic:SetHeight(height)
    
	f.ic2:SetWidth(height)
    f.ic2:SetHeight(height)
	
	--f:SetWidth(width)
	--f:SetHeight(height)

	f:UpdateBarSize()
	
	f.stacktext:SetTextColor(unpack(opt.stack.textcolor))
	f.stacktext:SetFont(SPTimers.SharedMedia:Fetch("font", opt.stack.font),
						opt.stack.size,
						opt.stack.flags)
	f.stacktext:SetJustifyH(opt.stack.justify)
	f.stacktext:SetAlpha(opt.stack.alpha or 1)
	
	
	f.stacktext:SetShadowColor(unpack(opt.stack.shadow or { 0, 0, 0, 1 }))
	f.stacktext:SetShadowOffset(opt.stack.offset and opt.stack.offset[1] or 0,opt.stack.offset and opt.stack.offset[2] or 0)
	
	f.stacktext2:SetTextColor(unpack(opt.stack.textcolor))
	f.stacktext2:SetFont(SPTimers.SharedMedia:Fetch("font", opt.stack.font),
						opt.stack.size,
						opt.stack.flags)
	f.stacktext2:SetJustifyH(opt.stack.justify)
	f.stacktext2:SetAlpha(opt.stack.alpha or 1)
	
	f.stacktext2:SetShadowColor(unpack(opt.stack.shadow or { 0, 0, 0, 1 }))
	f.stacktext2:SetShadowOffset(opt.stack.offset and opt.stack.offset[1] or 0,opt.stack.offset and opt.stack.offset[2] or 0)
		
--	f.stacktext:SetVertexColor(unpack(opt.stack.vertexcolor))
	
	f.bar:SetStatusBarTexture(GetDefaultTexture(opt.bar.texture))
	f.bar:SetStatusBarColor(unpack(opt.bar.color))
	
	f.bar:SetReverseFill(opt.reverse_fill)
	
	f.bar:SetHeight(height)
	f.bar:SetWidth(width - height - 1)
	
	--f.bar:SetBackdropColor(unpack(main_bg_color2)) --цвет фона
	--f.bar:SetBackdropBorderColor(unpack(main_border_color2)) --цвет краев	
	
	f.bar.bg2:SetTexture(GetDefaultTexture(opt.bar.bgtexture))
	f.bar.bg2:SetVertexColor(unpack(opt.bar.bgcolor))

	--f.bar.bg2:SetVertexColor(unpack(opt.bar.bgcolor))
	
	f.bar.bg:SetPoint("TOPLEFT", -opt.borderinset, opt.borderinset) -- Implemented 'insets'
	f.bar.bg:SetPoint("BOTTOMRIGHT", opt.borderinset, -opt.borderinset) -- Implemented 'insets'

	f.bar.bg:SetBackdrop({
		edgeFile = SPTimers.SharedMedia:Fetch("border", opt.border),
		edgeSize = opt.bordersize,
	})
	f.bar.bg:SetBackdropBorderColor(opt.bordercolor[1], opt.bordercolor[2], opt.bordercolor[3], opt.bordercolor[4])
	
	f.ic.bg:SetPoint("LEFT", -opt.borderinset, 0)		
	f.ic.bg:SetPoint("RIGHT", opt.borderinset, 0)
	f.ic.bg:SetPoint("TOP", 0, opt.borderinset)		
	f.ic.bg:SetPoint("BOTTOM", 0, -opt.borderinset)

	f.ic.bg:SetBackdrop({
		edgeFile = SPTimers.SharedMedia:Fetch("border", opt.border),
		edgeSize = opt.bordersize,
	})
	f.ic.bg:SetBackdropBorderColor(opt.bordercolor[1], opt.bordercolor[2], opt.bordercolor[3], opt.bordercolor[4])
	
	f.ic2.bg:SetPoint("LEFT", -opt.borderinset, 0)		
	f.ic2.bg:SetPoint("RIGHT", opt.borderinset, 0)
	f.ic2.bg:SetPoint("TOP", 0, opt.borderinset)		
	f.ic2.bg:SetPoint("BOTTOM", 0, -opt.borderinset)

	f.ic2.bg:SetBackdrop({
		edgeFile = SPTimers.SharedMedia:Fetch("border", opt.border),
		edgeSize = opt.bordersize,
	})
	f.ic2.bg:SetBackdropBorderColor(opt.bordercolor[1], opt.bordercolor[2], opt.bordercolor[3], opt.bordercolor[4])
	
    f.timeText:SetTextColor(unpack(opt.timer.textcolor))
    f.timeText:SetFont(SPTimers.SharedMedia:Fetch("font",opt.timer.font),
					   opt.timer.size,
					   opt.timer.flags)
    f.timeText:SetJustifyH(opt.timer.justify)
    f.timeText:SetAlpha(opt.timer.alpha or 1)
 --   f.timeText:SetVertexColor(unpack(opt.timer.vertexcolor))

 
	f.timeText:SetShadowColor(unpack(opt.timer.shadow or { 0, 0, 0, 1 }))
	f.timeText:SetShadowOffset(opt.timer.offset and opt.timer.offset[1] or 0,opt.timer.offset and opt.timer.offset[2] or 0)
	
    f.spellText:SetTextColor(unpack(opt.spell.textcolor))
    f.spellText:SetFont(SPTimers.SharedMedia:Fetch("font",opt.spell.font),
						opt.spell.size,
						opt.spell.flags)



	f.spellText:SetShadowColor(unpack(opt.spell.shadow or { 0, 0, 0, 1 }))
	f.spellText:SetShadowOffset(opt.spell.offset and opt.spell.offset[1] or 0,opt.spell.offset and opt.spell.offset[2] or 0)
	
					
    f.spellText:SetWidth(f.bar:GetWidth()*0.8)
    f.spellText:SetHeight(height/2+1)
    f.spellText:SetJustifyH(opt.spell.justify)
    f.spellText:SetAlpha(opt.spell.alpha or 1)
	
	f.spellText:SetPoint("LEFT", f.bar, "LEFT",f.bar:GetWidth()*0.1+opt.spell.offsetx,0)
	f.spellText:SetPoint("RIGHT", f.bar, "RIGHT",-f.bar:GetWidth()*0.1+opt.spell.offsetx,0)
	
	f.bar.overlay1:SetHeight(f.bar:GetHeight())
	f.bar.overlay2:SetHeight(f.bar:GetHeight())
	
	f.bar.spark:SetHeight(f.bar:GetHeight()*3)
	
	f:UpdateRaidIcon()
	
	
	if opt.reverse_fill then
	
	else
	
	end
	
	
	if opt.left_icon then
		f.ic:Show()
	else
		f.ic:Hide()
	end
	
	if opt.right_icon then
		f.ic2:Show()
	else
		f.ic2:Hide()
	end
	
	--f:UpdateTick_Color()
	--f:UpdateSpark_Color()
end

function SPTimers.UpdateStackText(f)
	local opt = f:GetOpts() --local opt = f:GetParent().opts
	local opt2 = f.opts
	f.stacktext:SetTextColor(unpack(opt.stack.textcolor))
	f.stacktext:SetFont(SPTimers.SharedMedia:Fetch("font",opt.stack.font),
						opt.stack.size,
						opt.stack.flags)
	f.stacktext:SetJustifyH(opt.stack.justify)
	f.stacktext:SetAlpha(opt.stack.alpha or 1)

	
	f.stacktext:SetShadowColor(unpack(opt.stack.shadow or { 0, 0, 0, 1 }))
	f.stacktext:SetShadowOffset(opt.stack.offset and opt.stack.offset[1] or 0,opt.stack.offset and opt.stack.offset[2] or 0)
	
	f.stacktext2:SetTextColor(unpack(opt.stack.textcolor))
	f.stacktext2:SetFont(SPTimers.SharedMedia:Fetch("font",opt.stack.font),
						opt.stack.size,
						opt.stack.flags)
	f.stacktext2:SetJustifyH(opt.stack.justify)
	f.stacktext2:SetAlpha(opt.stack.alpha or 1)
	
	f.stacktext2:SetShadowColor(unpack(opt.stack.shadow or { 0, 0, 0, 1 }))
	f.stacktext2:SetShadowOffset(opt.stack.offset and opt.stack.offset[1] or 0,opt.stack.offset and opt.stack.offset[2] or 0)
	
--	f.stacktext:SetVertexColor(unpack(opt.stack.vertexcolor))
end

function SPTimers.UpdateTimeText(f)
	local opt = f:GetOpts() -- local opt = f:GetParent().opts
	local opt2 = f.opts
	f.timeText:SetTextColor(unpack(opt.timer.textcolor))
    f.timeText:SetFont(SPTimers.SharedMedia:Fetch("font",opt.timer.font),
					   opt.timer.size,
					   opt.timer.flags)
    f.timeText:SetJustifyH(opt.timer.justify)
    f.timeText:SetAlpha(opt.timer.alpha or 1)
	
	f.timeText:SetShadowColor(unpack(opt.timer.shadow or { 0, 0, 0, 1 }))
	f.timeText:SetShadowOffset(opt.timer.offset and opt.timer.offset[1] or 0,opt.timer.offset and opt.timer.offset[2] or 0)
	
	if not opt.lefttext then
		f.timeText:ClearAllPoints()
		f.timeText:SetPoint("TOP", f.bar, "TOP")
		f.timeText:SetPoint("BOTTOM", f.bar, "BOTTOM")
		f.timeText:SetPoint("RIGHT", f.bar, "RIGHT",0,0)
	else
		f.timeText:ClearAllPoints()
		f.timeText:SetPoint("TOP", f.bar, "TOP")
		f.timeText:SetPoint("BOTTOM", f.bar, "BOTTOM")
		f.timeText:SetPoint("LEFT", f.bar, "LEFT", 0,0)
	end
	
  --  f.timeText:SetVertexColor(unpack(opt.timer.vertexcolor))
end
function SPTimers.UpdateSpellText(f)
	local opt = f:GetOpts() -- local opt = f:GetParent().opts
	local opt2 = f.opts
	local height = opt.h
	local width = opt.w
	f.spellText:SetTextColor(unpack(opt.spell.textcolor))
    f.spellText:SetFont(SPTimers.SharedMedia:Fetch("font",opt.spell.font),
						opt.spell.size,
						opt.spell.flags)
    f.spellText:SetWidth(f.bar:GetWidth()*0.9)
    f.spellText:SetHeight(height/2+1)
    f.spellText:SetJustifyH(opt.spell.justify)
    f.spellText:SetAlpha(opt.spell.alpha or 1)
	
	f.spellText:SetShadowColor(unpack(opt.spell.shadow or { 0, 0, 0, 1 }))
	f.spellText:SetShadowOffset(opt.spell.offset and opt.spell.offset[1] or 0,opt.spell.offset and opt.spell.offset[2] or 0)
	
	f.spellText:SetPoint("LEFT", f.bar, "LEFT",f.bar:GetWidth()*0.1+opt.spell.offsetx,0)
	f.spellText:SetPoint("RIGHT", f.bar, "RIGHT",-f.bar:GetWidth()*0.1+opt.spell.offsetx,0)
	
	--f.spellText:SetPoint("CENTER", f.bar, "CENTER", 0, 0)
	
	--[[
	if not opt.lefttext then		
		f.spellText:SetPoint("LEFT", f.bar, "LEFT",6,0)
	else
		f.spellText:SetPoint("LEFT", f.bar, "LEFT",40,0)
	end
	]]
end

function SPTimers.UpdateTick_Color(f)
	local opt = f:GetOpts() -- local opt = f:GetParent().opts
	
	if opt.dotticks and opt.dotticks.color and f.tiks then
		for k,v in ipairs(f.tiks) do
			v:SetVertexColor(unpack(opt.dotticks.color))
		end
	end
end

function SPTimers.UpdateSpark_Color(f)
	local opt = f:GetOpts() -- local opt = f:GetParent().opts
		
		
		f.bar.spark:SetAlpha(opt.castspark and opt.castspark.alpha or 1)
		f.bar.spark:SetWidth(opt.castspark and opt.castspark.w or 10)		
		f.bar.spark:SetHeight(opt.castspark and opt.castspark.h or f.bar:GetHeight()*1.5)
		if opt.castspark and opt.castspark.color then
			f.bar.spark:SetVertexColor(unpack(opt.castspark.color))
		end
		
		
		f.bar.shine:SetWidth(opt.castspark and opt.castspark.w and opt.castspark.w*1.6 or 16)
		f.bar.shine:SetHeight(opt.castspark and opt.castspark.h and opt.castspark.h*2 or f.bar:GetHeight()*3)
		
end

function SPTimers.UpdateRaidIcon(f)
	local opt = f:GetOpts()

	f.raidMark:SetPoint("TOP", f.bar, "TOP", opt.raidicon_x or 0, opt.raidicon_y or 5)
	f.raidMark:SetSize(opt.raidiconsize or 10, opt.raidiconsize or 10)
	f.raidMark:SetAlpha(opt.raidicon_alpha or 1)

end

function SPTimers.UpdateBorder(f)
	local opt = f:GetOpts() -- f:GetParent().opts
	
	f.bar.bg:SetPoint("TOPLEFT", -opt.borderinset, opt.borderinset) -- Implemented 'insets'
	f.bar.bg:SetPoint("BOTTOMRIGHT", opt.borderinset, -opt.borderinset) -- Implemented 'insets'
	
	
--	f.ic:SetPoint("TOPRIGHT",f.bar,"TOPLEFT",-opt.icon_gap, 0)
	
	--f.ic2:SetPoint("TOPLEFT",f.bar,"TOPRIGHT",opt.icon_gap, 0)
	
	f.ic:SetPoint("TOPRIGHT",f.bar,"TOPLEFT",-opt.icon_gap, 0)
	f.ic:SetPoint("BOTTOMRIGHT",f.bar,"BOTTOMLEFT",-opt.icon_gap, 0)
	
	f.ic2:SetPoint("TOPLEFT",f.bar,"TOPRIGHT",opt.icon_gap, 0)
	f.ic2:SetPoint("BOTTOMLEFT",f.bar,"BOTTOMRIGHT",opt.icon_gap, 0)
		
	f.bar.bg:SetBackdrop({
			edgeFile = SPTimers.SharedMedia:Fetch("border", opt.border),
			edgeSize = opt.bordersize,
		})
	f.bar.bg:SetBackdropBorderColor(opt.bordercolor[1], opt.bordercolor[2], opt.bordercolor[3], opt.bordercolor[4])
	
	f.ic.bg:SetPoint("LEFT", -opt.borderinset, 0)		
	f.ic.bg:SetPoint("RIGHT", opt.borderinset, 0)
	f.ic.bg:SetPoint("TOP", 0, opt.borderinset)		
	f.ic.bg:SetPoint("BOTTOM", 0, -opt.borderinset)

	f.ic.bg:SetBackdrop({
		edgeFile = SPTimers.SharedMedia:Fetch("border", opt.border),
		edgeSize = opt.bordersize,
	})
	f.ic.bg:SetBackdropBorderColor(opt.bordercolor[1], opt.bordercolor[2], opt.bordercolor[3], opt.bordercolor[4])
	
	f.ic2.bg:SetPoint("LEFT", -opt.borderinset, 0)		
	f.ic2.bg:SetPoint("RIGHT", opt.borderinset, 0)
	f.ic2.bg:SetPoint("TOP", 0, opt.borderinset)		
	f.ic2.bg:SetPoint("BOTTOM", 0, -opt.borderinset)

	f.ic2.bg:SetBackdrop({
		edgeFile = SPTimers.SharedMedia:Fetch("border", opt.border),
		edgeSize = opt.bordersize,
	})
	f.ic2.bg:SetBackdropBorderColor(opt.bordercolor[1], opt.bordercolor[2], opt.bordercolor[3], opt.bordercolor[4])
	
end

do 
	local function Round(num) return math.floor(num+.5) end --.5
	
	local function getbarpos(timer, tik)
		local duration = timer.duration
		--timer.duration or
		local minValue, maxValue = timer.bar:GetMinMaxValues()
		if tik >= 0 then
			return tik / maxValue * timer.bar:GetWidth()
		else
			return (maxValue+tik) / maxValue * timer.bar:GetWidth()
		end
	end
	 
	local function getbarcurrentpos(timer, value)
		local duration = timer.duration		
	--	local current_time = timer.bar:GetValue()
		local minValue, maxValue = timer.bar:GetMinMaxValues()
	--	print(duration, current_time, timer.bar:GetValue())
		
		return value/maxValue * timer.bar:GetWidth()
	end

	local function getoverlay2point(timer, _time, value)
	--	local duration = timer.duration
		
		local minValue, maxValue = timer.bar:GetMinMaxValues()
		local current_time = value - _time
		
		
		if current_time <= 0 then current_time = 0 end
		return current_time/maxValue * timer.bar:GetWidth()
	end
	
	local function overlayWidth(timer, _time)
		local duration = timer.duration
		local minValue, maxValue = timer.bar:GetMinMaxValues()
		
		return _time/maxValue * timer.bar:GetWidth()
	end

	function SPTimers.OnValueChanged(self, value)
		
		local parent = self.parent
		local current_position = getbarcurrentpos(parent, value)

		if parent.duration == 0 then
			self.spark:Hide()
		else		
		--	self.sp1:SetPoint("CENTER",self,"LEFT", current_position, 0)
			
			if parent.opts.reverse_fill then
				self.sp1:SetPoint("TOPLEFT",self,"TOPLEFT", -current_position+parent.opts.w, 0)
				self.sp1:SetPoint("BOTTOMLEFT",self,"BOTTOMLEFT", -current_position+parent.opts.w, 0)
			else
				self.sp1:SetPoint("TOPLEFT",self,"TOPLEFT", current_position, 0)
				self.sp1:SetPoint("BOTTOMLEFT",self,"BOTTOMLEFT", current_position, 0)
			end
			
			self.spark:Show()
		end
		
		local castTime = parent.castTime or 0
		local overlaytime = castTime or 0

		if parent.tickOverlap and parent.ticks_evert_s then
		
			if overlaytime > 0 then
				overlaytime = overlaytime+parent.ticks_evert_s
			else
				overlaytime = parent.ticks_evert_s
			end
		end

		
		if overlaytime and overlaytime > 0 then

		--	self.sp2:SetPoint("CENTER",self,"LEFT", getoverlay2point(parent, overlaytime, value), 0)
			
			
			self.overlay1:Hide()
			local over_widht = 0
			
		--	print(castoverlay, (castoverlay > 0))
			if castTime > 0 then
				over_widht = overlayWidth(parent, castTime)
				
				self.overlay2:Show() -- cast overlay
				self.overlay2:SetWidth(over_widht)
			elseif castTime < 0 then
				over_widht = overlayWidth(parent, abs(castTime))
				
				self.overlay2:Show() -- cast overlay
				self.overlay2:SetWidth(over_widht)
			else
				self.overlay2:Hide()
			end
			
			local ov_2point = getoverlay2point(parent, overlaytime, value)
			
			if parent.opts.reverse_fill then
				self.sp2:SetPoint("TOPLEFT",self,"TOPLEFT", -ov_2point-over_widht+parent.opts.w, 0)
				self.sp2:SetPoint("BOTTOMLEFT",self,"BOTTOMLEFT", -ov_2point-over_widht+parent.opts.w, 0)
			else
				self.sp2:SetPoint("TOPLEFT",self,"TOPLEFT", ov_2point, 0)
				self.sp2:SetPoint("BOTTOMLEFT",self,"BOTTOMLEFT", ov_2point, 0)
			end
			
		else
			self.overlay1:Hide()
			self.overlay2:Hide()
		end

		
		
		if ( parent.showticks and parent.amounttoshow) then
			for i=1, #parent.tiks do 
				parent.tiks[i]:Hide()
			end
			
		--	print("1 SPTimers.OnValueChanged")
			if not parent.endTime or not parent.startTime then return end	
		--	print("2 SPTimers.OnValueChanged")
			local lowertick = 99999999
			local num = parent.amounttoshow


			
			if value == 0 then
				parent._ticks_left = 0
			else
				parent._ticks_left = floor(value/parent.ticks_evert_s)+1
			end


			
			if options.hide_dot_ticks then return end
			
			for i=1, num, 1 do
		--		print("4 SPTimers.OnValueChanged")
				if options.showonlynext then
					if current_position <= parent.tiks[i].tick_pos then
						
						if lowertick > i then --frame.myticks.tiks[i].mynumber
							lowertick = i		--frame.myticks.tiks[i].mynumber
						end

						if parent.tiks[i].shine  then
							parent.bar.spark.shine:Play()
							parent.tiks[i].shine = false
						end
						parent.tiks[i]:Hide()
						
						if parent.tiks[lowertick-1] then
							parent.tiks[lowertick-1]:Show()
						end
					end
				elseif options.ticksfade then
					if current_position <= parent.tiks[i].tick_pos then		
						if parent.tiks[i].shine  then
							parent.bar.spark.shine:Play()
							parent.tiks[i].shine = false
						end
						parent.tiks[i]:Hide()
					else 
						parent.tiks[i]:Show()
					end
				else

					if current_position <= parent.tiks[i].tick_pos then		
						if parent.tiks[i].shine  then
							parent.bar.spark.shine:Play()
							parent.tiks[i].shine = false
						end
					end
					
					if parent.tiks[i].tick_pos < self:GetWidth() then
						parent.tiks[i]:Show()
					end
				end
			end
		else
		--	print("3 SPTimers.OnValueChanged", parent.showticks, parent.amounttoshow, parent.spellName )
			for i=1, #parent.tiks do 
				parent.tiks[i]:Hide()
			end
		end
	end




































	







	function SPTimers.SetTicks(self)
		










































































		if not self.showticks then return end
		
















		local isChanneling = SPTimers:IsChanneling(self.spellID)
		local tiks_every = nil
		


		if isChanneling then
			tiks_every = SPTimers.dots[self.spellID.."channel:spell"]
		else
			tiks_every = SPTimers.dots[self.spellID..self.targetGUID] or SPTimers.dots[self.spellID..SPTimers.myGUID]
		end

		if not tiks_every then		


			if SPTimers:GetCLEUSpellInfo(self.spellID) and not isChanneling then				
				SPTimers:GetTickInfo(self.spellID, SPTimers.myGUID)
				
				tiks_every = SPTimers.dots[self.spellID..self.targetGUID]
			end
		end
		
	--	print("SPTimers.SetTicks", tiks_every)
		if not tiks_every then return end

		local duration = self.duration or ( self.endTime - self.startTime )

		local tickcount = Round(duration/tiks_every)-1


		--print("SPTimers.SetTicks", tickcount, duration/tiks_every, isChanneling)
		
		if ( self.amounttoshow ~= tickcount ) 
		or ( self.ticks_evert_s ~= tiks_every ) 


		or isChanneling 
		or ( self._lastupdate ~= self.endTime ) then






			for i=1, tickcount, 1 do
		
				if not self.tiks[i] then		
					self.tiks[i] = self.bar.bg:CreateTexture(nil, "BACKGROUND", nil, -8)
					self.tiks[i]:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
					self.tiks[i]:SetAlpha(0.9)
					self.tiks[i]:SetWidth(5)
					self.tiks[i]:SetHeight(self.opts.h*1.8)
					self.tiks[i]:SetBlendMode('ADD')		
				end




				local tick_position = floor(getbarpos(self, tiks_every*i))
				self.tiks[i].mynumber = i
				self.tiks[i].shine = true
				self.tiks[i].tick_pos = getbarpos(self, tiks_every*i)
			--	self.tiks[i]:SetPoint("CENTER",self.bar,"LEFT", floor(tick_position), 0)
				



































				if self.opts.reverse_fill then
				--	self.sp1:SetPoint("TOPLEFT",self,"TOPLEFT", -current_position+parent.opts.w, 0)
				--	self.sp1:SetPoint("BOTTOMLEFT",self,"BOTTOMLEFT", -current_position+parent.opts.w, 0)










					
					self.tiks[i]:SetPoint("TOPLEFT",self.bar,"TOPLEFT", -(tick_position)+self.opts.w, self.opts.h*0.4)


					self.tiks[i]:SetPoint("BOTTOMLEFT",self.bar,"BOTTOMLEFT", -(tick_position)+self.opts.w, -self.opts.h*0.4)
				






				else
					self.tiks[i]:SetPoint("TOPLEFT",self.bar,"TOPLEFT", (tick_position), self.opts.h*0.4)
					self.tiks[i]:SetPoint("BOTTOMLEFT",self.bar,"BOTTOMLEFT", (tick_position), -self.opts.h*0.4)
				end

		--		print(tick_position, tiks_every, tiks_every*i)
			end




















			self.ticks_evert_s = tiks_every
			self.amounttoshow = tickcount
			self._lastupdate = self.endTime
		--	print("Update Ticks count", tickcount, self.spellName)
	--	else
	--		print("SPTimers.SetTicks SKIP", tickcount, duration/tiks_every, isChanneling)
		end
	end
end

do
	local smoothing = {}
	local function Smooth(self, value)
		if value ~= self:GetValue() or value == 0 then
	--		print(self, value)
			smoothing[self] = value
		else
	--		print(self, value, "nil")
			smoothing[self] = nil
		end
	end
	local function SmoothBar(bar)
	--	print("SmoothBar", bar)
		if not bar.SetValue_ then
			bar.Smooth = true 
			bar.SetValue_ = bar.SetValue;
			bar.SetValue = Smooth;
		end
	end

	local function ResetBar(bar)
	--	print("ResetBar", bar)
		if bar.SetValue_ then
			bar.Smooth = false
			bar.SetValue = bar.SetValue_;
			bar.SetValue_ = nil;
		end
	end
	
	local smooth_fps = 1
	
	function SPTimers:UpdateSmothFPS()
	
		smooth_fps = options.bar_smooth_value/100
	end
	
	local f, min, max = CreateFrame('Frame'), math.min, math.max
	f:SetScript('OnUpdate', nil)
	
	local function SmothOnUpdate()
		local rate = GetFramerate()
		local limit = 30/rate

		for bar, value in pairs(smoothing) do
			local cur = bar:GetValue()
			local new = cur + min((value-cur)/3, max(value-cur, limit))*smooth_fps
			if new ~= new then
				-- Mad hax to prevent QNAN.
				new = value
			end
			bar:SetValue_(new)
			if (cur == value or abs(new - value) < 2) and bar.Smooth then
				bar:SetValue_(value)
				smoothing[bar] = nil
			elseif not bar.Smooth then
				bar:SetValue_(value)
				smoothing[bar] = nil		
			end
		end
	end

	function SPTimers.SmoothBars(self)
	--	print("SmoothBars", self)
		if options.bar_smooth then
			SmoothBar(self.bar)
			f:SetScript('OnUpdate', SmothOnUpdate)
		else
			ResetBar(self.bar)
			f:SetScript('OnUpdate', nil)
		end
		SPTimers:UpdateSmothFPS()
	end

end








































































































local f_time = 0.4
local function OnFadeUpdate(self, elapsed)
	self.elapsed = self.elapsed - elapsed
	if self.elapsed >= 0 then
		local a = (self.elapsed/f_time)
		























		self.f.timeText:SetAlpha((self.f.opts.timer.alpha or 1)*a*0.4)
		self.f.stacktext:SetAlpha((self.f.opts.stack.alpha or 1)*a*0.4)
		self.f.stacktext2:SetAlpha((self.f.opts.stack.alpha or 1)*a*0.4)
		self.f.spellText:SetAlpha((self.f.opts.spell.alpha or 1)*a*0.4)
		self.f:SetHeight(self.f.opts.h*a)
		self.f.bar:SetHeight(self.f.opts.h*a)










































































	else
		self:OnFinished()
	end
end

local function IsActiveBar(self)
	return self.parent.active[self] or false
end
				
function SPTimers.Create_Bar(self, number)
	local opt = self.opts
	local bars = self.bars
	local f, ic, ict, ri, rit, ict2
	local height = opt.h
	local width = opt.w

	if not bars[number] then
		f = CreateFrame("Frame","SPTimers-Bar:"..number.."A:"..self.id ,self)
		f.parent = self
		f.opts = self.opts
		f:SetParent(self)
		f.Update = SPTimers.Update_Bar
		f.UpdateStackText = SPTimers.UpdateStackText
		f.UpdateTimeText = SPTimers.UpdateTimeText
		f.UpdateSpellText = SPTimers.UpdateSpellText
		f.UpdateBorder = SPTimers.UpdateBorder
		f.UpdateTick_Color = SPTimers.UpdateTick_Color
		f.UpdateSpark_Color = SPTimers.UpdateSpark_Color
		f.UpdateBarSize		= SPTimers.UpdateBarSize
		f.UpdateRaidIcon		= SPTimers.UpdateRaidIcon
		f.IsActiveBar = IsActiveBar
		
		f.ResetBarSize = function(self, spark)




			--if self.anim:IsPlaying() then 
			SPTimers.RemoveFader(self)


			self.anim:Stop() 


			--end
			if not spark then
				self.bar.spark:Show()
			end


			self.timeText:SetAlpha(self.opts.timer.alpha or 1)
			self.stacktext:SetAlpha(self.opts.stack.alpha or 1)
			self.stacktext2:SetAlpha(self.opts.stack.alpha or 1)
			self.spellText:SetAlpha(self.opts.spell.alpha or 1)
			self.bar:SetHeight(self.opts.h)
			self.bar:SetWidth(self.opts.w-self.opts.h-1)
			self:SetHeight(self.opts.h)
		end
		
		f.anim = CreateFrame("Frame", nil, f)
		f.anim.f = f
		f.anim.elapsed = 0
		f.anim.isplaying = false
		f.anim.duration = f_time
		f.anim:SetScript("OnUpdate", nil)
		f.anim.IsPlaying = function(self)
			return self.isplaying 
		end
		f.anim.Stop = function(self)
			self:SetScript("OnUpdate", nil)
			self.isplaying = false
		end
		f.anim.Play = function(self)
			self.isplaying = true
			self.elapsed = f_time
			self:SetScript("OnUpdate", OnFadeUpdate)
		end
		f.anim.OnFinished = function(self)
			self:Stop()
			self.f:EndTimer("fade")
			self.f.parent:SortTimers("FadeOUT Animation")	
		end
		--[[
		local aag = f.anim:CreateAnimationGroup()
		aag.f = f
		aag.aa1 = aag:CreateAnimation("Scale")
		






		aag.aa1:SetOrigin("TOP",0,0)		
		if ( not f.opts.add_up ) then aag.aa1:SetOrigin("TOP",0,0) 
		else aag.aa1:SetOrigin("BOTTOM",0,0) end		
		aag.aa1:SetScale(1, 1)
		
		aag.aa1:SetDuration(f_time)
		aag.aa1:SetOrder(1)
		
		
		f.fadeoutanum = aag
		]]
		
		f.FadeOut = function(self)	
			if not self.anim:IsPlaying() then
			--	print("2 SPTimers.Fader OUT")
				self.on_fading = true
				self.bar.spark:Hide()				
				self.anim:Play()
			end
		end
		--[[
		aag:SetScript("OnFinished", function(self, force)
			self.f:EndTimer("fade")
			self.f.parent:SortTimers("FadeOUT Animation")	
			self:SetScript("OnUpdate", nil)
		end)
		]]
		tinsert(bars_list, f)

		f.duration = 0
		f.startTime = 0
		f.endTime = 0
		
		f:SetSize(width, height)

		
		f.ToInfinite = SPTimers.ToInfinite
		f.SetTimer = SPTimers.SetTimer
		f.EndTimer = SPTimers.EndTimer
		f.SetCount = SPTimers.SetCount
		f.SetMark = SPTimers.SetMark
		f.SetTicks = SPTimers.SetTicks
		f.RestoreQueue = SPTimers.RestoreQueue
		f.SmoothBars = SPTimers.SmoothBars
		f.IsDead = function(self, isdead)
			f.isdead = isdead
		end
		
		f.GetOpts = function(self) 
			return self.parent.opts 
		end
		
		f.UpdateBarColor = SPTimers.UpdateBarColor
		f.BarTextUpdate = SPTimers.BarTextUpdate
		
		
		f.tiks = {}

		f.bar = CreateFrame("StatusBar",nil,f)
		f.bar:SetScript("OnValueChanged", SPTimers.OnValueChanged)
		f.bar.parent = f
		
		f.bar:SetParent(f)
		
		f.bar:SetReverseFill(opt.reverse_fill)
		f.bar:SetStatusBarTexture("Interface\\ChatFrame\\ChatFrameBackground")
		f.bar:GetStatusBarTexture():SetDrawLayer("ARTWORK")
		
		f.bar:SetPoint("TOPLEFT",f,"TOPLEFT",0,0)
		f.bar:SetPoint("BOTTOMRIGHT",f,"BOTTOMRIGHT",0,0)
		
		f.ic = CreateFrame("Frame",nil,f.bar)
		f.ic:SetPoint("TOPRIGHT",f.bar,"TOPLEFT",-opt.icon_gap, 0)
		f.ic:SetPoint("BOTTOMRIGHT",f.bar,"BOTTOMLEFT",-opt.icon_gap, 0)
		
		
		f.ic.bg = CreateFrame("Frame", nil, f.ic)
		f.ic.bg:SetFrameStrata("LOW")
		f.ic.bg:SetPoint("LEFT", -opt.borderinset, 0)		
		f.ic.bg:SetPoint("RIGHT", opt.borderinset, 0)
		f.ic.bg:SetPoint("TOP", 0, opt.borderinset)		
		f.ic.bg:SetPoint("RIGHT", 0, -opt.borderinset)
		f.ic.bg:SetBackdrop({
			edgeFile = SPTimers.SharedMedia:Fetch("border", opt.border),
			edgeSize = opt.bordersize,
		})
		f.ic.bg:SetBackdropBorderColor(opt.bordercolor[1], opt.bordercolor[2], opt.bordercolor[3], opt.bordercolor[4])
		
		ict = f.ic:CreateTexture(nil,"ARTWORK",1)
		ict:SetTexCoord(.1, .9, .1, .9)
		ict:SetAllPoints(f.ic)
		
		f.icon = ict
		
		f.stacktext = f.ic:CreateFontString(nil, "OVERLAY");
		f.stacktext:SetPoint("RIGHT", f.ic, "RIGHT",4,0)
		f.stacktext:SetPoint("LEFT", f.ic, "LEFT",-8,0)
		f.stacktext:SetPoint("BOTTOM", f.ic, "BOTTOM",0,0)
	
		f.stacktext:SetShadowColor(0,0,0,1)
		f.stacktext:SetShadowOffset(0,0)
		
		f.ic2 = CreateFrame("Frame",nil,f.bar)
		f.ic2:SetPoint("TOPLEFT",f.bar,"TOPRIGHT", opt.icon_gap, 0)
		f.ic2:SetPoint("BOTTOMLEFT",f.bar,"BOTTOMRIGHT", opt.icon_gap, 0)
		
		f.ic2.bg = CreateFrame("Frame", nil, f.ic2)
		f.ic2.bg:SetFrameStrata("LOW")

		f.ic2.bg:SetPoint("LEFT", -opt.borderinset, 0)		
		f.ic2.bg:SetPoint("RIGHT", opt.borderinset, 0)
		f.ic2.bg:SetPoint("TOP", 0, opt.borderinset)		
		f.ic2.bg:SetPoint("RIGHT", 0, -opt.borderinset)
		
		f.ic2.bg:SetBackdrop({
			edgeFile = SPTimers.SharedMedia:Fetch("border", opt.border),
			edgeSize = opt.bordersize,
		})
		f.ic2.bg:SetBackdropBorderColor(opt.bordercolor[1], opt.bordercolor[2], opt.bordercolor[3], opt.bordercolor[4])
		
		ict2 = f.ic2:CreateTexture(nil,"ARTWORK",1)
		ict2:SetTexCoord(.1, .9, .1, .9)
		ict2:SetAllPoints(f.ic2)
		
		f.icon2 = ict2
		
		f.stacktext2 = f.ic2:CreateFontString(nil, "OVERLAY")
		f.stacktext2:SetPoint("RIGHT", f.ic2, "RIGHT",8,0)
		f.stacktext2:SetPoint("LEFT", f.ic2, "LEFT",-4,0)
		f.stacktext2:SetPoint("BOTTOM", f.ic2, "BOTTOM",0,0)

		f.stacktext2:SetShadowColor(0,0,0,1)
		f.stacktext2:SetShadowOffset(0,0)
		
		f:SetScript("OnUpdate", SPTimers.UpdateStatusBar)

		f.bar.bg2 = f.bar:CreateTexture(nil,"BORDER", 0)
		f.bar.bg2:SetTexture("Interface\\ChatFrame\\ChatFrameBackground")
		f.bar.bg2:SetAllPoints(f.bar)
		
		f.bar.bg3 = f.bar:CreateTexture(nil,"BORDER", 5)
	--	f.bar.bg3:SetTexture("Interface\\ChatFrame\\ChatFrameBackground")
		f.bar.bg3:SetAllPoints(f.bar)
		f.bar.bg3:Hide()
		f.bar.bg3:SetAlpha(0)
		
		f.bg3_fade = f.bar.bg3:CreateAnimationGroup()
		f.bg3_fade.a1 = f.bg3_fade:CreateAnimation("Alpha")
		f.bg3_fade.a1:SetChange(0.8)
		f.bg3_fade.a1:SetDuration(0.6)
		f.bg3_fade.a1:SetOrder(1)
		f.bg3_fade.a2 = f.bg3_fade:CreateAnimation("Alpha")
		f.bg3_fade.a2:SetChange(-0.8)
		f.bg3_fade.a2:SetDuration(0.6)
		f.bg3_fade.a2:SetOrder(2)

		f.bgFade = function(self, dur, cur)
			if not self.bg3_fade:IsPlaying() then				
				local m = dur/cur
				
				if m < .35 then m = .35 end
				
				f.bg3_fade.a1:SetDuration(0.6*m)
				f.bg3_fade.a2:SetDuration(0.6*m)
				
				self.bar.bg3:Show()
				self.bg3_fade:Play()
			end
		end
		
		f.bar.bg = CreateFrame("Frame", nil, f.bar)
		f.bar.bg:SetParent(f.bar)
		f.bar.bg:SetPoint("TOPLEFT", -opt.borderinset, opt.borderinset)
		f.bar.bg:SetPoint("BOTTOMRIGHT", opt.borderinset, -opt.borderinset)

		f.bar.bg:SetBackdrop({
			edgeFile = SPTimers.SharedMedia:Fetch("border", opt.border),
			edgeSize = opt.bordersize,
		})
		f.bar.bg:SetBackdropBorderColor(opt.bordercolor[1], opt.bordercolor[2], opt.bordercolor[3], opt.bordercolor[4])
		
		rit = f.bar.bg:CreateTexture(nil,"OVERLAY", nil, 5)
		rit:SetTexture("Interface\\TargetingFrame\\UI-RaidTargetingIcons")
		rit:SetPoint("TOP", f.bar, "TOP", opt.raidicon_x or 0, opt.raidicon_y or 5)
		rit:SetSize(opt.raidiconsize or 10, opt.raidiconsize or 10)
		rit:SetAlpha(opt.raidicon_alpha or 1)
		
		rit:Hide()
		
		f.raidMark = rit
		
		f.timeText = f.bar.bg:CreateFontString(nil, "OVERLAY")


		f.timeText:SetJustifyV("CENTER")
		f.timeText:SetPoint("TOP", f.bar, "TOP")
		f.timeText:SetPoint("BOTTOM", f.bar, "BOTTOM")
		
		f.timeText:SetShadowColor(0,0,0,1)
		f.timeText:SetShadowOffset(0,0)
		
		if not opt.lefttext then
			f.timeText:SetPoint("RIGHT", f.bar, "RIGHT", 0,0)
		else	
			f.timeText:SetPoint("LEFT", f.bar, "LEFT", 0,0)
		end
		
		--[[
		if (true) then
			f.spellTextMover = CreateFrame("Frame",nil, f.bar)
			f.spellTextMover:SetFrameStrata("HIGH")



			f.spellText = f.spellTextMover:CreateFontString(nil, "OVERLAY")
			f.spellText:SetParent(f.spellTextMover)
		else
			f.spellText = f.bar:CreateFontString(nil, "OVERLAY")
			f.spellText:SetParent(f.bar)
		end


		]]
		f.spellText = f.bar.bg:CreateFontString(nil, "OVERLAY", nil, 5)
		f.spellText:SetParent(f.bar.bg)

		f.spellText:SetJustifyV("CENTER")


		--[[
		if not opt.lefttext then
			f.spellText:SetPoint("LEFT", f.bar, "LEFT",6,0)
		else
			f.spellText:SetPoint("LEFT", f.bar, "LEFT",40,0)
		end
		]]
		
		if not opt.spell.offsetx then opt.spell.offsetx = 0 end

		
		f.spellText:SetShadowColor(0,0,0,1)
		f.spellText:SetShadowOffset(0,0)
		
		f.spellText:SetPoint("LEFT", f.bar, "LEFT",f.bar:GetWidth()*0.1+opt.spell.offsetx,0)
		f.spellText:SetPoint("RIGHT", f.bar, "RIGHT",-f.bar:GetWidth()*0.1+opt.spell.offsetx,0)
		f.spellText:SetPoint("TOP", f.bar, "TOP")
		f.spellText:SetPoint("BOTTOM", f.bar, "BOTTOM")
		
		
		local sp1 = CreateFrame("Frame",nil,f)
		sp1:SetParent(f)
		sp1:SetWidth(1)
		sp1:SetHeight(f:GetHeight()*0.7)
		sp1:SetFrameLevel(4)
		sp1:SetAlpha(1)
	--	sp1:SetPoint("CENTER",f.bar,"LEFT",10,0)
		
		sp1:SetPoint("TOPLEFT",f.bar,"TOPLEFT", 10, 0)
		sp1:SetPoint("BOTTOMLEFT",f.bar,"BOTTOMLEFT", 10, 0)
			
		local sp2 = CreateFrame("Frame",nil,f)
		sp2:SetParent(f)
		sp2:SetWidth(1)
		sp2:SetHeight(f:GetHeight()*0.9)
		sp2:SetFrameLevel(4)
		sp2:SetAlpha(1)
	--	sp2:SetPoint("CENTER",f.bar,"LEFT",10,0)
		
		sp2:SetPoint("TOPLEFT",f.bar,"TOPLEFT", 10, 0)
		sp2:SetPoint("BOTTOMLEFT",f.bar,"BOTTOMLEFT", 10, 0)
		
		f.bar.sp1 = sp1
		f.bar.sp2 = sp2
		--[[
			opt.castspark.alpha
			opt.castspark.color
			opt.castspark.w
			opt.castspark.h
		]]
		local spark_t = f.bar.bg:CreateTexture(nil, "ARTWORK", nil, 3)
		spark_t:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
		spark_t:SetAlpha(1)
		spark_t:SetWidth(10)		
		spark_t:SetHeight(f.bar:GetHeight()*1.5)			
		
		spark_t:SetBlendMode('ADD')
		spark_t.parent = sp1
		spark_t:SetPoint("CENTER",sp1,"LEFT",0,0)
		spark_t:SetPoint("TOP", sp1, "TOP",0,10)
		spark_t:SetPoint("BOTTOM", sp1, "BOTTOM",0,-10)
		
		f.bar.spark = spark_t
		
		local shine = f.bar.bg:CreateTexture(nil, "ARTWORK", nil, 4)
		shine:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
		shine:SetAlpha(0)
		shine:SetWidth(10)
		shine:SetHeight(50)		
		shine:SetBlendMode('ADD')
		shine.parent = sp1
		shine:SetPoint("CENTER",sp1,"LEFT",0,0)
		
		f.bar.shine = shine
		
		local ag = shine:CreateAnimationGroup()
		local a1 = ag:CreateAnimation("Alpha")
		a1:SetChange(1)
		a1:SetDuration(0.1)
		a1:SetOrder(1)
		local a2 = ag:CreateAnimation("Alpha")
		a2:SetChange(-1)
		a2:SetDuration(0.1)
		a2:SetOrder(2)

		f.bar.spark.shine = ag
		
		local overlay1 = f.bar:CreateTexture(nil, "ARTWORK", nil, 2)
		overlay1:SetTexture(1,1,1,1)	
		overlay1:SetAlpha(0.8)
		overlay1:SetWidth(20)
		overlay1:Hide()
		--overlay1:SetBlendMode('ADD')
		
		f.bar.overlay1 = overlay1
		
		overlay1:SetPoint("TOPRIGHT",sp1,"TOPLEFT",0,0)
		overlay1:SetPoint("BOTTOMRIGHT",sp1,"BOTTOMLEFT",0,0)
		
		
	--	overlay1:SetPoint("LEFT",sp2,"LEFT",0,0)
		
		overlay1:SetPoint("TOPLEFT",sp2,"TOPLEFT",0,0)
		overlay1:SetPoint("BOTTOMLEFT",sp2,"BOTTOMLEFT",0,0)
		
		local overlay2 = f.bar:CreateTexture(nil, "ARTWORK", nil, 2)
		overlay2:SetTexture(1,1,1,1)	
		overlay2:SetWidth(20)
		--overlay2:SetBlendMode('ADD')	
		overlay2:Hide()		
		f.bar.overlay2 = overlay2
		
	--	overlay2:SetPoint("LEFT",sp2,"LEFT",0,0)
		
		overlay2:SetPoint("TOPLEFT",sp2,"TOPLEFT",0,0)
		overlay2:SetPoint("BOTTOMLEFT",sp2,"BOTTOMLEFT",0,0)
		
		
	else
		f = bars[number]
		f:SetParent(self)
		f.parent = self
		f.opts = self.opts
	end
	
	f:Update()
	f:Hide()
	f:SmoothBars()


	print("UPDATE BAR")
	
    return f
end