local engine = AleaUI

local CB = engine:Module("CastBar")
local UF = engine:Module("UnitFrames")
local w, h

local events = {
	"UNIT_SPELLCAST_CHANNEL_START",
	"UNIT_SPELLCAST_CHANNEL_STOP",
	"UNIT_SPELLCAST_CHANNEL_UPDATE",
	
	--"UNIT_SPELLCAST_SENT",
	"UNIT_SPELLCAST_START",
	"UNIT_SPELLCAST_DELAYED",
	
	"UNIT_SPELLCAST_INTERRUPTED",
	"UNIT_SPELLCAST_STOP",
--	"UNIT_SPELLCAST_SUCCEEDED",
	
	"UNIT_SPELLCAST_INTERRUPTIBLE",
	"UNIT_SPELLCAST_NOT_INTERRUPTIBLE"
}	

local colors = {
	["notinterruptible"] = { 0,0,0, 1 },
	["interruptible"] = { .6, .2, .2, 1},
	
	["ping"] = { 1, 1, 1, 0.7 },
	["ticks"] = {1, 1, 1, 1},
}

local channeling_info = {
	[GetSpellInfo(15407)]  = { amount = 3, every = 1, haste = true },
	[GetSpellInfo(48045)]  = { amount = 5, every = 1, haste = true },
	[GetSpellInfo(129197)] = { amount = 3, every = 1, haste = true },
}



local function Round(num) return math.floor(num+.5) end --.5

local function getbarpos(bar, tik)
	local minValue, maxValue = bar:GetMinMaxValues()
	if tik >= 0 then
		return tik / maxValue * bar:GetWidth()
	else
		return (maxValue+tik) / maxValue * bar:GetWidth()
	end
end

local bordersize = 1

local function SameUnit(u1,u2)
	if u1 and u2 and UnitIsUnit(u1, u2) then
		return false
	end
	return true
end

local SetPing = function(self, value)
	--self:Hide()
	--if SameUnit(self.parent.unit, "player") then return end
	
	if not value then return end
	
	if value >= 0.4 then
		value = 0.4
	end

	local minv, maxv = self.parent:GetMinMaxValues()
	local width, height = self.parent:GetWidth(), self.parent:GetHeight()
	local mywidth = width/maxv*value

	self:SetSize(mywidth, height-bordersize-bordersize)
	
	local left1 = self.parent:GetRight()
	local left2 = self:GetRight() or 0
	
	if left2 >= left1 then
		self:SetSize(mywidth-abs(left2-left1), height-bordersize-bordersize)
	end
	
	--print(left2-self:GetRight(), mywidth)
	
	self:Show()
end

local function DrawLatency(f, name)	
	if f.parent.unit ~= "player" then return end
	
		if channeling_info[name] then
			for i=1, #f.tiks do
				if f.tiks[i]:IsShown() then
					f.tiks[i].latensy:SetPing(f.ping)
				else
					f.tiks[i].latensy:Hide()
				end
			end		
			f.channelLatency:SetPing(f.ping)
			f.castLatency:Hide()
		else
			for i=1, #f.tiks do		
				f.tiks[i].latensy:Hide()
			end		
			f.channelLatency:Hide()
			f.castLatency:SetPing(f.ping)
		end
end

local function DrawTicks(f, name)	

	for i=1, #f.tiks do
		f.tiks[i]:Hide()
	end

	if f.parent.unit ~= "player" then return end
	
	if channeling_info[name] then
	
	--	print(f.parent.unit)
		
		local haste = f.haste
		local tick_every, amount_to_show
		local tick = channeling_info[name].every
		local duration = f.duration
		
		if channeling_info[name].haste then
			tick_every	= tick/(1+(haste/100))
		else
			tick_every	= tick
		end
		
		amount_to_show = Round(duration/tick_every)-1
			
		if f.duration2 > 0 then			
			amount_to_show = amount_to_show + 1
		end
		
	--	print("T2", format("%.2f", amount_to_show), format("%.2f", f.duration2), format("%.2f", f.duration), format("%.2f", duration), format("%.2f", tick_every), haste)

		
		for i=1, amount_to_show do
			w,h = f:GetWidth(), f:GetHeight()
			
			if not f.tiks[i] then		
				f.tiks[i] = f.ticksparent:CreateTexture(nil, "ARTWORK")
				f.tiks[i]:SetTexture(unpack(colors.ticks))
				f.tiks[i]:SetAlpha(1)
				f.tiks[i]:SetWidth(1)
				f.tiks[i]:SetHeight(h)
				
				local lat3 = CreateFrame("StatusBar", nil, f)
				lat3:SetFrameLevel(f:GetFrameLevel()+1)
				lat3.parent = f
				lat3:SetStatusBarTexture("Interface\\ChatFrame\\ChatFrameBackground")
				lat3:GetStatusBarTexture():SetDrawLayer("OVERLAY")
				lat3:SetFrameStrata("LOW")
				lat3:SetStatusBarColor(unpack(colors.ping))
				lat3:SetPoint("TOPLEFT", f.tiks[i], "TOPLEFT", 0, -0)
				lat3:SetPoint("BOTTOMLEFT", f.tiks[i], "BOTTOMLEFT", 0, 0)
				lat3:SetSize(40, h)
				lat3.SetPing = SetPing
				lat3:Hide()
				
				f.tiks[i].latensy = lat3
			end

			local tick_position = floor(getbarpos(f, tick_every*i))
			
			if false then
				f.tiks[i]:SetPoint("LEFT",f,"LEFT", -(tick_position)+w, 0)			
			else
				f.tiks[i]:SetPoint("LEFT",f,"LEFT", (tick_position), 0)
			end
			
			f.tiks[i]:Show()
		end
	end
end

local function CastBarOnUpdate(f, elapsed)
	local curtime = GetTime()
	if curtime > f.endTime then
		f:Hide()
		return  
	end
	
	local curdur
	
	if f.nap == 1 then -- cast
		curdur = curtime - f.startTime
	else -- channel
		curdur = f.endTime - curtime
	end
	local v = f.duration2-f.duration
	
		local i = 0	
	
		for k,v in pairs(f.tiks) do
			if v:IsShown() then
				i = i + 1
			end
		end
		
	if v > 0 then
		f.rightText:SetFormattedText(" %d - %.1f | %.1f + %.1f", i, curdur, f.duration, v)
	else
		f.rightText:SetFormattedText(" %d - %.1f | %.1f", i, curdur, f.duration)
	end
	
	f:SetValue(curdur)
end

-- name, subText, text, texture, startTime / 1000, endTime / 1000, isTradeSkill, notInterruptible = UnitChannelInfo("unit")
-- name, subText, text, texture, startTime / 1000, endTime / 1000, isTradeSkill, castID, notInterruptible = UnitCastingInfo("unit")

local function OnCast(f, unit)
	
	local name, subText, text, texture, startTime, endTime, isTradeSkill, castID, notInterruptible = UnitCastingInfo(unit)	
	if not startTime or not endTime then return end
	
	f.startTime		= startTime/1000
	f.endTime		= endTime/1000
	f.duration		= f.endTime - f.startTime
	f.duration2		= 0
	f.nap			= 1

	if f.castTime_Start then
		f.ping			= f.startTime - f.castTime_Start
	end
	
	--print("OnCast")
	
	f.icon:SetTexture(texture)
	f.leftText:SetText(name)
	f:SetMinMaxValues(0, f.duration)
	f:DrawTicks(name)
	f:UpdateIntrerruptState(notInterruptible)
	f:DrawLatency(name)
	--f:SetReverseFill(false)
	
	f.castTime_Start = nil
	f:Show()
end

local function OnCastUpdate(f, unit)
	
	local name, subText, text, texture, startTime, endTime, isTradeSkill, castID, notInterruptible = UnitCastingInfo(unit)
	if not startTime or not endTime then return end
	
	f.startTime		= startTime/1000
	f.endTime		= endTime/1000
	f.nap			= 1
	
	f.duration2		= f.endTime - f.startTime
	
	--print("OnCastUpdate")
	
	f.icon:SetTexture(texture)
	f.leftText:SetText(name)
	f:DrawTicks(name)
	f:SetMinMaxValues(0, f.duration2)
	f:UpdateIntrerruptState(notInterruptible)
	f:DrawLatency(name)

	f:Show()
end

local function OnChannel(f, unit)
	
	local name, subText, text, texture, startTime, endTime, isTradeSkill, notInterruptible = UnitChannelInfo(unit)
	if not startTime or not endTime then return end
	
--	print("OnChannel")
	
	f.startTime		= startTime/1000
	f.endTime		= endTime/1000
	f.duration		= f.endTime - f.startTime
	f.duration2		= 0
	f.nap			= -1
	f.haste			= UnitSpellHaste("player")

	if f.castTime_Start then
		f.ping	= f.startTime - f.castTime_Start
	end
	
	f.icon:SetTexture(texture)
	f.leftText:SetText(name)
	f:SetMinMaxValues(0, f.duration)
	f:DrawTicks(name)
	f:UpdateIntrerruptState(notInterruptible)
	f:DrawLatency(name)

	f.castTime_Start = nil
	
	f:Show()
end

local function OnChannelUpdate(f, unit)
	
	local name, subText, text, texture, startTime, endTime, isTradeSkill, notInterruptible = UnitChannelInfo(unit)
	if not startTime or not endTime then return end
	
--	print("OnChannelUpdate", f.endTime, endTime/1000, (endTime/1000)-f.endTime, f.parent.unit)

	f.duration2 	= f.duration + abs((endTime/1000)-f.endTime)
	
	f.startTime		= startTime/1000
	f.endTime		= endTime/1000

	f.nap			= -1
	
	f.icon:SetTexture(texture)
	f.leftText:SetText(name)
	f:SetMinMaxValues(0, f.duration2)
	f:UpdateIntrerruptState(notInterruptible)

	f:DrawTicks(name)
	f:DrawLatency(name)

	f:Show()
end

	
function UF:CreateCastBar(frame, w, h, drawticks)
	w, h = w or 200, h or 20
	
	
	local mover = CreateFrame("StatusBar", nil, frame)
	mover:SetSize(w,h)
	mover:SetPoint("TOP", frame, "BOTTOM", 0, -h)
	mover:SetFrameStrata("LOW")
	
	local f = CreateFrame("StatusBar", nil, mover)
	f:SetFrameLevel(mover:GetFrameLevel()+1)
	f.parent = frame
	f:SetStatusBarTexture("Interface\\ChatFrame\\ChatFrameBackground")
	f:GetStatusBarTexture():SetDrawLayer("OVERLAY")
	f:SetFrameStrata("LOW")
	f:SetStatusBarColor(unpack(colors["interruptible"]))
	f:SetPoint("TOPRIGHT", mover, "TOPRIGHT", -bordersize, -bordersize)
	f:SetPoint("BOTTOMRIGHT", mover, "BOTTOMRIGHT", -bordersize, bordersize)
	f:SetSize(w-h-bordersize-bordersize-bordersize, h-bordersize-bordersize)
	
	f.unit = frame.unit
	
	local lat1 = CreateFrame("StatusBar", nil, f)
	lat1:SetFrameLevel(f:GetFrameLevel()+1)
	lat1.parent = f
	lat1:SetStatusBarTexture("Interface\\ChatFrame\\ChatFrameBackground")
	lat1:GetStatusBarTexture():SetDrawLayer("OVERLAY")
	lat1:SetFrameStrata("LOW")
	lat1:SetStatusBarColor(unpack(colors.ping))
	lat1:SetPoint("TOPLEFT", f, "TOPLEFT", 0, -0)
	lat1:SetPoint("BOTTOMLEFT", f, "BOTTOMLEFT", 0, 0)
	lat1:SetSize(40, h)
	lat1.SetPing = SetPing
	lat1:Hide()
	
	f.channelLatency = lat1
	
	local lat2 = CreateFrame("StatusBar", nil, f)
	lat2:SetFrameLevel(f:GetFrameLevel()+1)
	lat2.parent = f
	lat2:SetStatusBarTexture("Interface\\ChatFrame\\ChatFrameBackground")
	lat2:GetStatusBarTexture():SetDrawLayer("OVERLAY")
	lat2:SetFrameStrata("LOW")
	lat2:SetStatusBarColor(unpack(colors.ping))
	lat2:SetPoint("TOPRIGHT", f, "TOPRIGHT", 0, 0)
	lat2:SetPoint("BOTTOMRIGHT", f, "BOTTOMRIGHT", 0, 0)
	lat2:SetSize(40, h)
	lat2.SetPing = SetPing
	lat2:Hide()
	
	f.castLatency = lat2
	
	f.ticksparent = CreateFrame("Frame", nil, f)
	f.ticksparent:SetFrameLevel(f:GetFrameLevel()+2)
	f.tiks = {}
	
	f.border = CreateFrame("Frame", nil, f)
	
	f.border:SetBackdrop({
		edgeFile = "Interface\\ChatFrame\\ChatFrameBackground", 
		edgeSize = bordersize, 
	})
	f.border:SetBackdropBorderColor(0,0,0,1)
	f.border:SetPoint("LEFT", -bordersize, 0)		
	f.border:SetPoint("RIGHT", bordersize, 0)
	f.border:SetPoint("TOP", 0, bordersize)		
	f.border:SetPoint("BOTTOM", 0, -bordersize)
	
	f.UpdateIntrerruptState = function(self, state)
		if state then
			self:SetStatusBarColor(unpack(colors["interruptible"]))
		else
			self:SetStatusBarColor(unpack(colors["notinterruptible"]))
		end
	end
	
	
	f.DrawTicks = DrawTicks
	f.DrawLatency = DrawLatency
	f.OnChannelUpdate = OnChannelUpdate
	f.OnCast = OnCast
	f.OnCastUpdate = OnCastUpdate
	f.OnChannel = OnChannel
	f.CastBarOnUpdate = CastBarOnUpdate
	
	local bg = f:CreateTexture(nil,"BACKGROUND")
	bg:SetTexture("Interface\\ChatFrame\\ChatFrameBackground")
	bg:SetAllPoints(f)
	bg:SetTexture(0.3,0.3,0.3,1)
	
	f.bg = bg
	
	local icon = f:CreateTexture(nil,"OVERLAY")
	icon:SetTexture("Interface\\ChatFrame\\ChatFrameBackground")
	icon:SetSize(h-bordersize-bordersize,h-bordersize-bordersize)
	icon:SetPoint("TOPRIGHT", f, "TOPLEFT", -bordersize-bordersize-bordersize, 0)
	icon:SetPoint("BOTTOMRIGHT", f, "BOTTOMLEFT", -bordersize-bordersize-bordersize, 0)
	icon:SetTexCoord(0.07, 0.93, 0.07, 0.93)
	f.icon = icon
	
	bg = f:CreateTexture(nil,"BACKGROUND")
	bg:SetTexture("Interface\\ChatFrame\\ChatFrameBackground")
	bg:SetAllPoints(f.icon)
	bg:SetTexture(0,0,0,1)
	
	f.icon.bg = bg
	
	f.icon.border = CreateFrame("Frame", nil, f)
	f.icon.border:SetBackdrop({
		edgeFile = "Interface\\ChatFrame\\ChatFrameBackground", 
		edgeSize = bordersize, 
	})
	f.icon.border:SetBackdropBorderColor(0,0,0,1)
	f.icon.border:SetPoint("LEFT", f.icon,"LEFT",-bordersize, 0)		
	f.icon.border:SetPoint("RIGHT", f.icon,"RIGHT", bordersize, 0)
	f.icon.border:SetPoint("TOP", f.icon,"TOP",0, bordersize)		
	f.icon.border:SetPoint("BOTTOM", f.icon,"BOTTOM",0, -bordersize)

	frame.UpdateStatusBar = UpdateStatusBar
	
	
	for i, event in ipairs(events) do
		frame:RegisterEvent(event)
	end
	
	local leftText = f.ticksparent:CreateFontString(nil, "OVERLAY", "GameFontNormal");
	leftText.parent = f
	leftText:SetPoint("LEFT", f, "LEFT")	
	leftText:SetTextColor(1,1,1)
	leftText:SetFont(ALEAUI_FONT, 12, "OUTLINE")
	leftText:SetAlpha(1)
	leftText:SetJustifyH("LEFT")
	leftText:SetText("Left")
	f.leftText = leftText
	
	local rightText = f.ticksparent:CreateFontString(nil, "OVERLAY", "GameFontNormal");
	rightText.parent = f
	rightText:SetPoint("RIGHT", f, "RIGHT")	
	rightText:SetTextColor(1,1,1)
	rightText:SetFont(ALEAUI_FONT, 12, "OUTLINE")
	rightText:SetAlpha(1)
	rightText:SetJustifyH("RIGHT")
	rightText:SetText("rightText")
	f.rightText = rightText
	
	
	f:Hide()
	
	frame.castBar = f
	
	f:SetScript("OnUpdate", f.CastBarOnUpdate)

	
	function frame:UNIT_SPELLCAST_CHANNEL_START(event, unit)
		-- if SameUnit(unit, self.unit) then return end
		
		 if self.unit ~= unit then return end
		self.castBar:OnChannel(unit)	
	end
	
	if f.unit == "player" then		
		frame:RegisterEvent("UNIT_SPELLCAST_SENT")
		function frame:UNIT_SPELLCAST_SENT(event, unit)
			-- if SameUnit(unit, self.unit) then return end
		
			if self.unit ~= unit then return end
			self.castBar.castTime_Start = GetTime()
		end
	end
	
	function frame:UNIT_SPELLCAST_CHANNEL_UPDATE(event, unit)
		-- if SameUnit(unit, self.unit) then return end
		
		 if self.unit ~= unit then return end
		self.castBar:OnChannelUpdate(unit)	
	end
	
	function frame:UNIT_SPELLCAST_START(event, unit)
		-- if SameUnit(unit, self.unit) then return end
		
		 if self.unit ~= unit then return end
		self.castBar:OnCast(unit)
	end
	
	function frame:UNIT_SPELLCAST_DELAYED(event, unit)
		-- if SameUnit(unit, self.unit) then return end
		
		 if self.unit ~= unit then return end
		self.castBar:OnCastUpdate(unit)
	end

	function frame:UNIT_SPELLCAST_INTERRUPTED(event, unit)
		-- if SameUnit(unit, self.unit) then return end
		
		 if self.unit ~= unit then return end
		self.castBar:Hide()
	end
	
	frame.UNIT_SPELLCAST_STOP = frame.UNIT_SPELLCAST_INTERRUPTED
	frame.UNIT_SPELLCAST_CHANNEL_STOP = frame.UNIT_SPELLCAST_INTERRUPTED
	
	function frame:UNIT_SPELLCAST_INTERRUPTIBLE(event, unit)
		-- if SameUnit(unit, self.unit) then return end
		
		 if self.unit ~= unit then return end
		self.castBar:UpdateIntrerruptState(false)
	end
	
	function frame:UNIT_SPELLCAST_NOT_INTERRUPTIBLE(event, unit)
		-- if SameUnit(unit, self.unit) then return end
		
		 if self.unit ~= unit then return end
		self.castBar:UpdateIntrerruptState(true)
	end

	frame.CastBarUpdate = function(self)
		self:UNIT_SPELLCAST_INTERRUPTED()
		self:UNIT_SPELLCAST_CHANNEL_START(_, self.unit)
		self:UNIT_SPELLCAST_START(_, self.unit)
	end
	
	return mover
end
