local L = LibStub("AceLocale-3.0"):GetLocale("SPTimers")
local ACD = LibStub("AceConfigDialog-3.0")

local o
local anchor_value = nil
local class_select = nil
local proc_select = nil
local others_select = nil
local cooldown_select = nil
local ICD_select = nil
local blocklist_select = nil
local unit_filter_set = nil
local unit_per_achor_choose = nil

local old_print = print
local print = function(...)
	if SPTimers.dodebugging then	
		old_print(GetTime(), "SPTimers_BARS, ", ...)
	end
end

local message = SPTimers.message

local justifu = {
	["RIGHT"] = "RIGHT",
	["CENTER"] = "CENTER",
	["LEFT"] = "LEFT",
}

local text_flaggs = {
	[""] = NO,
	["OUTLINE"] = "OUTLINE",
	["THICKOUTLINE"] = "THICKOUTLINE",
	["MONOCHROMEOUTLINE"] = "MONOCHROMEOUTLINE",
}

local filters = {
	L["None"],
	L["Only buff"],
	L["Only debuff"],
	L["All"],
}

local filters_cleu = {
	L["None"],
	L["Only buff"],
	L["Only debuff"],
	L["All"],
	L["SpellCast"],
	L["Summon"],
	L["Energize"],
}

local affiliation = {
	L["Any"],
	L["player"],
	L["raid"],
	L["party"],
	L["pet"],
}

local targetType = {
	L["single"],
	L["multi"],
	L["auto"],
}

local groupSorting = {
	["player"] = "player",
	["procs"]  = "procs",
	["auto"]   = "auto",
	["target"] = "target",
}
					
local bar_timer_format = {
	"1h,2m,119s,29.9",
	"1h,2m,119s/300,29.99/300",
	"1:11m,59s,10s,1s",
	"1:11m,59.1s,10.2s,1.1s",
	"1,2,119,29.9",
	"1,2,119/300,29.99/300",
	"1:11,59,10,1",
	"1:11,59.1,10.2,1.1",
}

local cooldown_timer_format = {
	"1h,2m,119s,29.9",
	"1:11m,59s,10s,1s",
	"1:11m,59.1s,10.2s,1.1s",
	"1 ,2 ,119,29.9",
	"1:11,59,10,1",
	"1:11,59.1,10.2,1.1",
}

local sorting_timer_table = {
	"priority from lower to upper",
	"endtime from upper to lower",
	"endtime from lower to upper",
}


local SpellString
do

	local _spelliconcache = {}

	function SPTimers.SpellString(spellID,size, hideid)
		if not size then size = 12 end
		if not _spelliconcache[spellID..size] then
			if spellID then
			
				local name, rank, icon = GetSpellInfo(spellID)

				_spelliconcache[spellID..size] = "\124T"..icon..":"..size.."\124t #"..spellID.." "..name
				
				return _spelliconcache[spellID..size]
			end
		else
			return _spelliconcache[spellID..size]
		end
	end

end

local placeholder = { [114207] = 114206, }

function SPTimers:SearchDBSpell(spellid, dbtype)
	
	local spellid = placeholder[spellid] or spellid
	
--	print("Do searth")
	if self.db.profile.classSpells[self.myCLASS][spellid] then
		
		message(SPTimers.SpellString(spellid)..L[" already exists in "]..L["Class Spells"]..L[". Redirect..."])
		
		ACD:SelectGroup("SPTimers", "bars", "ClassSpells")
		self:ClassSpell(spellid)
		return
	end
	
	if self.db.profile.procSpells[spellid] then
		
		message(SPTimers.SpellString(spellid)..L[" already exists in "]..L["Procs"]..L[". Redirect..."])
		
		ACD:SelectGroup("SPTimers", "bars", "spellList2")
		self:ProcsSpell(spellid)
		return
	end
	
	if self.db.profile.othersSpells[spellid] then
		
		message(SPTimers.SpellString(spellid)..L[" already exists in "]..L["Other"]..L[". Redirect..."])
		
		ACD:SelectGroup("SPTimers", "bars", "spellList3")
		self:OthersSpell(spellid)
		return
	end
	
	if dbtype == "class" then
		self.db.profile.classSpells[self.myCLASS][spellid] = {}
		self:ClassSpell(spellid)
		return
	elseif dbtype == "procs" then
		self.db.profile.procSpells[spellid] = {}
		self:ProcsSpell(spellid)
		return
	elseif dbtype == "others" then
		self.db.profile.othersSpells[spellid] = {}
		self:OthersSpell(spellid)
		return
	end
end

local function deepcopy(t)
		if type(t) ~= 'table' then return t end
			local mt = getmetatable(t)
			local res = {}
			for k,v in pairs(t) do
				if type(v) == 'table' then
					v = deepcopy(v)
				end
			res[k] = v
			end
			setmetatable(res,mt)
		return res
	end
	
function SPTimers:GlobalStyleUpdate()
	
	--statusbar
	local global_statusbar			= self.db.profile.global_statusbar
	local global_statusbrcolor		= self.db.profile.global_statusbrcolor
	local global_statusbar_bg		= self.db.profile.global_statusbar_bg
	local global_statusbrcolor_bg	= self.db.profile.global_statusbrcolor_bg
	--font
	local global_font 				= self.db.profile.global_font	
	local global_textcolor 			= self.db.profile.global_textcolor	
	local global_size 				= self.db.profile.global_size	
	local global_alpha 				= self.db.profile.global_alpha
	local global_flags 				= self.db.profile.global_flags
	-- border
	local global_border 			= self.db.profile.global_border
	local global_bordercolor 		= self.db.profile.global_bordercolor
	local global_bordersize 		= self.db.profile.global_bordersize
	local global_borderinset 		= self.db.profile.global_borderinset
	
	local global_font_shadow 		= self.db.profile.global_font_shadow
	local global_font_shadow_offset = self.db.profile.global_font_shadow_offset
				
	for i, opts in ipairs(self.db.profile.bars_anchors) do
	
		self.db.profile.bars_anchors[i].border 		= global_border or opts.border
		self.db.profile.bars_anchors[i].bordersize		= global_bordersize or opts.bordersize
		self.db.profile.bars_anchors[i].borderinset	= global_borderinset or opts.borderinset
		self.db.profile.bars_anchors[i].bordercolor	= deepcopy(global_bordercolor or opts.bordercolor)
		
		self.db.profile.bars_anchors[i].bar.color		= deepcopy(global_statusbrcolor or opts.bar.color)
		self.db.profile.bars_anchors[i].bar.texture 	= global_statusbar or opts.bar.texture
		self.db.profile.bars_anchors[i].bar.bgcolor		= deepcopy(global_statusbrcolor_bg or opts.bar.bgcolor)
		self.db.profile.bars_anchors[i].bar.bgtexture	= global_statusbar_bg or opts.bar.bgtexture
	
		self.db.profile.bars_anchors[i].stack.textcolor 	= deepcopy(global_textcolor or opts.stack.textcolor)
		self.db.profile.bars_anchors[i].stack.font			= global_font or opts.stack.font
		self.db.profile.bars_anchors[i].stack.size			= global_size or opts.stack.size
		self.db.profile.bars_anchors[i].stack.flags		= global_flags or opts.stack.flags
		self.db.profile.bars_anchors[i].stack.alpha		= global_alpha or opts.stack.alpha		
		self.db.profile.bars_anchors[i].stack.shadow		= global_font_shadow or opts.stack.shadow
		self.db.profile.bars_anchors[i].stack.offset		= global_font_shadow_offset or opts.stack.offset
	
		self.db.profile.bars_anchors[i].timer.textcolor	= deepcopy(global_textcolor or opts.timer.textcolor)
		self.db.profile.bars_anchors[i].timer.font			= global_font or opts.timer.font
		self.db.profile.bars_anchors[i].timer.size			= global_size or opts.timer.size
		self.db.profile.bars_anchors[i].timer.flags		= global_flags or opts.timer.flags
		self.db.profile.bars_anchors[i].timer.alpha		= global_alpha or opts.timer.alpha		
		self.db.profile.bars_anchors[i].timer.shadow		= global_font_shadow or opts.timer.shadow
		self.db.profile.bars_anchors[i].timer.offset		= global_font_shadow_offset or opts.timer.offset
		
		self.db.profile.bars_anchors[i].spell.textcolor	= deepcopy(global_textcolor or opts.spell.textcolor)
		self.db.profile.bars_anchors[i].spell.font			= global_font or opts.spell.font
		self.db.profile.bars_anchors[i].spell.size			= global_size or opts.spell.size
		self.db.profile.bars_anchors[i].spell.flags		= global_flags or opts.spell.flags
		self.db.profile.bars_anchors[i].spell.alpha		= global_alpha or opts.spell.alpha
		self.db.profile.bars_anchors[i].spell.shadow		= global_font_shadow or opts.spell.shadow
		self.db.profile.bars_anchors[i].spell.offset		= global_font_shadow_offset or opts.spell.offset
	end
	
	self:Visibility()
	
	-- 1-r 2-g 3-b 4-a
	
	self.db.profile.cooldownline.statusbar 				= global_statusbar or self.db.profile.cooldownline.statusbar 
	self.db.profile.cooldownline.bgcolor				= global_statusbrcolor and { r = global_statusbrcolor[1], g = global_statusbrcolor[2], b = global_statusbrcolor[3], a = global_statusbrcolor[4], } or self.db.profile.cooldownline.bgcolor
	self.db.profile.cooldownline.border					= global_border or self.db.profile.cooldownline.border
	self.db.profile.cooldownline.bordersize				= global_bordersize or self.db.profile.cooldownline.bordersize
	self.db.profile.cooldownline.borderinset			= global_borderinset or self.db.profile.cooldownline.borderinset
	self.db.profile.cooldownline.bordercolor			= global_bordercolor and { r = global_bordercolor[1], g = global_bordercolor[2], b = global_bordercolor[3], a = global_bordercolor[4], } or self.db.profile.cooldownline.bordercolor

	self.db.profile.cooldownline.icon_border			= global_border or self.db.profile.cooldownline.icon_border
	self.db.profile.cooldownline.icon_bordersize		= global_bordersize or self.db.profile.cooldownline.icon_bordersize
	self.db.profile.cooldownline.icon_borderinset		= global_borderinset or self.db.profile.cooldownline.icon_borderinset
	self.db.profile.cooldownline.icon_background_inset	= 0 or self.db.profile.cooldownline.icon_background_inset
	self.db.profile.cooldownline.icon_bordercolor		= global_bordercolor and { r = global_bordercolor[1], g = global_bordercolor[2], b = global_bordercolor[3], a = global_bordercolor[4], } or self.db.profile.cooldownline.icon_bordercolor
	
	self.db.profile.cooldownline.font					= global_font and self.db.profile.cooldownline.font	
	self.db.profile.cooldownline.fontflags				= global_flags and self.db.profile.cooldownline.fontflags
	self.db.profile.cooldownline.fontsize				= global_size and self.db.profile.cooldownline.fontsize
	self.db.profile.cooldownline.fontcolor				= global_textcolor and { r = global_textcolor[1], g = global_textcolor[2], b = global_textcolor[3], a = global_textcolor[4], } or self.db.profile.cooldownline.fontcolor	
	self.db.profile.cooldownline.fontshadowcolor		= global_textcolor and { r = global_textcolor[1], g = global_textcolor[2], b = global_textcolor[3], a = global_textcolor[4], } or self.db.profile.cooldownline.fontshadowcolor	
	self.db.profile.cooldownline.fontshadowoffset		= global_font_shadow_offset or self.db.profile.cooldownline.fontshadowoffset

	self.db.profile.cooldownline.icon_font				= global_font and self.db.profile.cooldownline.icon_font
	self.db.profile.cooldownline.icon_fontflaggs		= global_flags and self.db.profile.cooldownline.icon_fontflaggs
	self.db.profile.cooldownline.icon_fontsize			= global_size and self.db.profile.cooldownline.icon_fontsize
	self.db.profile.cooldownline.icon_fontcolor			= global_textcolor and { r = global_textcolor[1], g = global_textcolor[2], b = global_textcolor[3], a = global_textcolor[4], } or self.db.profile.cooldownline.icon_fontcolor
	self.db.profile.cooldownline.icon_fontshadowcolor	= global_textcolor and { r = global_textcolor[1], g = global_textcolor[2], b = global_textcolor[3], a = global_textcolor[4], } or self.db.profile.cooldownline.icon_fontshadowcolor
	self.db.profile.cooldownline.icon_fontshadowoffset	= global_font_shadow_offset or self.db.profile.cooldownline.icon_fontshadowoffset
	
	self.UpdateSettings()
end

SpellString = SPTimers.SpellString

function SPTimers:DefaultOptions()

	local default = {
			profile =
			{
				locked = false,
				totalanchor = 1,
				engageThrottle = 30,
				unitThrottleValue = 0.3,
				throttleOutCombat = 30,
				playerThrottleValue = 0,
				show_mark = true,
				bar_module_enabled = true,
				bar_smooth_value = 100,
				doswap = true,
				onlyMineDebuff = true,
				othersFiltersoff = false,
				procsFiltersoff = false,
				classlistFiltersoff = false,
				maximumtime_value = 30,
				hide_during_petbattle = true,
				hide_dot_ticks = false,
				
				sound_channel = "Master",
				
				minimap = {
					hide = true,
				},			
				global_statusbar = "Flat",
				global_statusbrcolor = {80/255,80/255,80/255,1},
				global_statusbar_bg = "Flat",
				global_statusbrcolor_bg = {80/255,80/255,80/255,0.5},
				global_font = "Friz Quadrata TT",
				global_textcolor = { 1, 1, 1, 1},
				global_size = 12,
				global_alpha = 1,
				global_flags = "OUTLINE",
				global_border = "Flat",
				global_bordercolor = {0,0,0,1},
				global_bordersize = 1,
				global_borderinset = 1,
	
				global_font_shadow = { 0, 0, 0, 1},
				global_font_shadow_offset = { 0, 0},
				
				unit_filters = {
					target = true,
					focus  = true,
					boss1  = true,
					boss2  = true,
					boss3  = true,
					boss4  = true,
					boss5  = true,
					arena1  = true,
					arena2  = true,
					arena3  = true,
					arena4  = true,
					arena5  = true,
					unknown = true,
				},
				totems = {
					["totem1"] = {
						show = true,
						color = {1,80/255,0},
						priority = -19,
						group = "procs",
					},
					["totem2"] = {
						show = true,
						color = {74/255, 142/255, 42/255},
						priority = -20,
						group = "procs",
					},
					["totem3"] = {
						show = true,
						color = { 65/255, 110/255, 1},
						priority = -18,
						group = "procs",
					},
					["totem4"] = {
						show = true,
						color = {0.6, 0, 1},
						priority = -17,
						group = "procs",
					},
				},
				enchants = {},
				cacheSpells = {},
				classSpells = {
					["PRIEST"] = {},
					["WARLOCK"] = {},
					["WARRIOR"] = {},
					["PALADIN"] = {},
					["MAGE"] = {},
					["DEATHKNIGHT"] = {},
					["DRUID"] = {},
					["SHAMAN"] = {},
					["MONK"] = {},
					["HUNTER"] = {},
					["ROGUE"] = {},
				},
				classCooldowns = {
					["PRIEST"] = {},
					["WARLOCK"] = {},
					["WARRIOR"] = {},
					["PALADIN"] = {},
					["MAGE"] = {},
					["DEATHKNIGHT"] = {},
					["DRUID"] = {},
					["SHAMAN"] = {},
					["MONK"] = {},
					["HUNTER"] = {},
					["ROGUE"] = {},
				},
				procSpells = {},
				othersSpells = {},
				internal_cooldowns = {},
				cooldownline = {
					w = 360, 
					h = 18, 
					x = 0, 
					y = -240,
					hide_cooldown_line = false,
					tooltip_anchor_gap = 0,
					tooltip_anchor_to_frame = 2,
					tooltip_anchor_to = 1,
					slash_x = 0,
					slash_y = 0,
					slash_size = 200,
					slash_alpha = 0.5,
					slash_show = true,
					slash_show_small = true,
					slash_small_alpha = 0.7,
					
					custom_text_timer = "0 1 10 30 60 120 300",
					
					splash_big = {
						time_in = 0.5,
						time_out = 0.6,
						step_in = 0.6,
						step_out = -1.5,
						alpha_in = 1,
						alpha_out = -1.5,
					},
					
					splash_small = {
						time_in = 0.5,
						time_out = 0,
						step_in = 3,
						step_out = -2,
						alpha_in = 0,
						alpha_out = 0,
					},
					
					fortam_s = 1,
					
					enabled = true,
					
					hidepet = false,
					hidebag = false,
					hideinv = false,
					hidefail = false,
					hideplay = false,
					hidevehi = false,
					hideinternal = false,
					
					pla_color = {1,0.5,0,1},
					veh_color = {1,0.5,0,1},
					pet_color = {1,0,.95,1},
					bag_color = {1,1,0,1},
					inv_color = {1,1,0,1},
					inter_color = {0, 0.6, 0.85, 1},
					
					blood_runes = true,
					frost_runes = true,
					unholy_runes = true,
					
					blood_runes_color = {1, 0, 0, 1},
					frost_runes_color = {0, 0.43, 1, 1},
					unholy_runes_color = {0.2, 0.8, 0, 1},
					
					showenchants = false,
					
					iconplus = 4,
					inactivealpha = 0.5,
					activealpha = 1.0,
					statusbar = "Flat",
					bgcolor = { 
						r = 0, 
						g = 0, 
						b = 0, 
						a = 0.5, 
					},
					border = "Flat",
					bordersize = 1, -- Added defaults
					borderinset = 1, -- Added defaults
					bordercolor = { 
						r = 80/255, 
						g = 80/255, 
						b = 80/255, 
						a = 1, 
					},
					icon_backgroundcolor = {
						r = 0, 
						g = 0, 
						b = 0, 
						a = 0.8, 				
					},
					icon_border = "Flat",
					icon_bordersize = 1, -- Added defaults
					icon_borderinset = 0, -- Added defaults
					icon_background_inset = 0,
					icon_bordercolor = { 
						r = 80/255, 
						g = 80/255, 
						b = 80/255, 
						a = 1, 
					},
					splash_background_color	= {
						r = 0, 
						g = 0, 
						b = 0, 
						a = 0.3, 
					},
					splashsmall_background_color = {
						r = 0, 
						g = 0, 
						b = 0, 
						a = 0.3, 
					},
					font = "Friz Quadrata TT",
					fontflags = "OUTLINE",
					fontsize = 10,
					fontcolor = { 
						r = 1,
						g = 1,
						b = 1, 
						a = 1, 
					},
					
					fontshadowcolor = {
						r = 0,
						g = 0,
						b = 0,
						a = 1,					
					},
					
					fontshadowoffset = { 0, 0},
					
					icon_font = "Friz Quadrata TT",
					icon_fontflaggs = "OUTLINE",
					icon_fontsize = 10,
					icon_fontcolor = { 
						r = 1,
						g = 1,
						b = 1, 
						a = 1, 
					},
					
					icon_fontshadowcolor = {
						r = 0,
						g = 0,
						b = 0,
						a = 1,					
					},
					
					icon_fontshadowoffset = { 0, 0},
					
					spellcolor = { 
						r = 204/255, 
						g = 0, 
						b = 0, 
						a = 1,
						},
					nospellcolor = { 
						r = 0, 
						g = 0, 
						b = 0, 
						a = 1, 
					},
					block = {  -- [spell or item name] = true,
						[GetItemInfo(6948) or "Hearthstone"] = { itemid = 6948, hide = true },  -- Hearthstone
						[GetSpellInfo(125439) or "Revive Battle Pets"] = { spellid = 125439, hide = true },
					},	
				},
				bars_anchors = {
					{
						name = 1,
						bar_number = 20,
						left_icon = true,
						right_icon = false,
						icon_gap = 5,
						reverse = false,
						vertical = false,
						add_up = true,
						point = { 0, 0},
						w = 250,
						h = 14,
						target_name =true,
						fortam_s = 1,
						gap = 5,
						border = "Flat",
						bordersize = 1, -- Added defaults
						borderinset = 1, -- Added defaults
						bordercolor = {80/255,80/255,80/255,1},
						bar = { 
							color = {118/255, 0, 0, 1},
							texture = "Flat",
							bgcolor = {0, 0, 0, 0.5},
							bgtexture = "Flat",
						},
						stack = {
							textcolor = {1, 1, 1,1},
							font = STANDARD_TEXT_FONT,
							size = 14,
							flags = "OUTLINE",
							justify = "RIGHT",
							
							shadow =  { 0, 0, 0, 1},
							offset = { 0, 0 },
						},
						timer ={
							textcolor = {1, 1, 1,1},
							font = STANDARD_TEXT_FONT,
							size = 14,
							flags = "OUTLINE",
							justify = "RIGHT",
							
							shadow =  { 0, 0, 0, 1},
							offset = { 0, 0 },
						},
						spell ={
							textcolor = {1, 1, 1, 1},
							font = STANDARD_TEXT_FONT,
							size = 14,
							flags = "OUTLINE",
							justify = "LEFT",
							offsetx = 0,
							
							shadow =  { 0, 0, 0, 1},
							offset = { 0, 0 },
						},
						
						raidicon_x = 0,
						raidicon_y = 5, 
						raidiconsize = 10,
						raidicon_alpha = 1,

						castspark = {
							color = {1, 1, 1, 1},	
							alpha = 1,
						},
						dotticks = {
							color = {1, 1, 1, 1},	
							alpha = 1,
						},
						sorting = {
							{name = "target", 		gap = 10, alpha = 1,  sort = 1 },
							{name = "player", 		gap = 10, alpha = 1,  sort = 2 },
							{name = "procs",		gap = 15, alpha = .7, sort = 3 },
							{name = "offtargets",	gap = 6,  alpha = .7, sort = 4 },
						},	
					},
				},
			}
		}
		
	self:SetupAuras(default)
	self.db = LibStub("AceDB-3.0"):New("SPTimersDB", default)	
end

function SPTimers:OptionsTable()
	o = {
		type = "group",name = L["SPTimers (drag here to move options frame)"],
		args = {
			lock = {
				order = 1,name = L["Lock frames"],type = "toggle",
				set = function(info,val) self.db.profile.locked = not self.db.profile.locked; self:UpdateMovers(); self:UnlockCooldownLine(); end,
				get = function() return self.db.profile.locked end
			},
			test_bars = {
				type = 'execute',
				order = 2,
				name = L['Test bars'],
				func = function(info, value)
					self:TestBars();
				end,
			},
			show_more_buttons = {
				order = 3,name = L["Move Buttons"],type = "toggle",
				set = function(info,val) self.db.profile.show_more_buttons = not self.db.profile.show_more_buttons; self:UpdateMoverPosition() end,
				get = function() return self.db.profile.show_more_buttons end
			},
			about ={
				order = -1,name = L["About"],type = "group",
				args={					
				},
			},
			general={
				order = 1,name = L["General"],type = "group",
				args={
					minimap = {
						order = 1,name = L["Show minimap"],type = "toggle",
						set = function(info,val) self.db.profile.minimap.hide = not self.db.profile.minimap.hide; self:UpdateMinimap() end,
						get = function(info) return self.db.profile.minimap.hide end
					},
					show_marks = {
						order = 1.1,name = L["Show raid marks"],type = "toggle",
						set = function(info,val) self.db.profile.show_mark = not self.db.profile.show_mark; end,
						get = function(info) return self.db.profile.show_mark end
					},
					doswap = {
						order = 1.2,name = L["Swap bars when change target"],type = "toggle",
						set = function(info,val) self.db.profile.doswap = not self.db.profile.doswap; end,
						get = function(info) return self.db.profile.doswap end
					},
					
					showonlynext = {
						order = 1.3,name = L["Show only next tick"],type = "toggle",
						set = function(info,val) self.db.profile.showonlynext = not self.db.profile.showonlynext; end,
						get = function(info) return self.db.profile.showonlynext end
					},
					hide_dot_ticks = {
						order = 1.4,name = L["Hide dot ticks"],type = "toggle",
						set = function(info,val) self.db.profile.hide_dot_ticks = not self.db.profile.hide_dot_ticks; end,
						get = function(info) return self.db.profile.hide_dot_ticks end						
					},
					ticksfade = {
						order = 1.5,name = L["Hide passed tick"],type = "toggle",
						set = function(info,val) self.db.profile.ticksfade = not self.db.profile.ticksfade; end,
						get = function(info) return self.db.profile.ticksfade end
					},
					engageThrottle = {
						name = L["Engage Throttle"],
						desc = L["Engage Throttle Desc"],
						type = "range",
						order	= 2,
						min		= 1,
						max		= 60,
						step	= 1,
						set = function(info,val) 
							self.db.profile.engageThrottle = val
						end,
						get =function(info)
							return self.db.profile.engageThrottle
						end,
					},
					unitThrottleValue = {
						name = L["Unit Check Throttle"],
						desc = L["Unit Check Throttle Desc"],
						type = "range",
						order	= 3,
						min		= 0,
						max		= 3,
						step	= 0.1,
						set = function(info,val) 
							self.db.profile.unitThrottleValue = val
						end,
						get =function(info)
							return self.db.profile.unitThrottleValue
						end,
					},
					throttleOutCombat = {
						name = L["Out Of Combat Throttle"],
						desc = L["Out Of Combat Throttle Desc"],
						type = "range",
						order	= 4,
						min		= 5,
						max		= 30,
						step	= 1,
						set = function(info,val) 
							self.db.profile.throttleOutCombat = val
						end,
						get =function(info)
							return self.db.profile.throttleOutCombat
						end,
					},
					playerThrottleValue = {
						name = L["Player Check Throttle"],
						desc = L["Player Check Throttle Desc"],
						type = "range",
						order	= 5,
						min		= 0,
						max		= 1,
						step	= 0.1,
						set = function(info,val) 
							self.db.profile.playerThrottleValue = val
						end,
						get =function(info)
							return self.db.profile.playerThrottleValue
						end,
					},
					hide_during_petbattle = {
						order = 1,name = L["Hide during PetBattle"],type = "toggle", width = "full",
						set = function(info,val) self.db.profile.hide_during_petbattle = not self.db.profile.hide_during_petbattle; end,
						get = function(info) return self.db.profile.hide_during_petbattle end
					},
					sound_channel = {
						order = 7,type = 'select',name = L["Sound Channel"],
						values = {
							["Master"] = "Master",
							["SFX"] = "SFX",
							["Ambience"] = "Ambience",
							["Music"] = "Music",
						},
						set = function(info,value) self.db.profile.sound_channel = value; end,
						get = function(info) return self.db.profile.sound_channel end,
					},
					
					global_style_settings = {
					
						type = "group",	order	= 10,
						guiInline = true,
						name	= L["Global Style Settings"],
						args = {						
							statusbr = {
								order = 1,type = 'select',name = L["Texture"],
								dialogControl = 'LSM30_Statusbar',
								values = SPTimers.SharedMedia:HashTable("statusbar"),
								set = function(info,value) self.db.profile.global_statusbar = value; end,
								get = function(info) return self.db.profile.global_statusbar end,
							},
							statusbrcolor = {
								order = 2,name = L["Texture Color"],type = "color", hasAlpha = true,
								set = function(info,r,g,b,a) self.db.profile.global_statusbrcolor={r,g,b,a}; end,
								get = function(info)
									if self.db.profile.global_statusbrcolor then
										return self.db.profile.global_statusbrcolor[1],self.db.profile.global_statusbrcolor[2],self.db.profile.global_statusbrcolor[3],self.db.profile.global_statusbrcolor[4] 
									end
								end
							},
							statusbar_bg = {
								order = 2.1,type = 'select',name = L["Background Texture"],
								dialogControl = 'LSM30_Statusbar',
								values = SPTimers.SharedMedia:HashTable("statusbar"),
								set = function(info,value) self.db.profile.global_statusbar_bg = value; end,
								get = function(info) return self.db.profile.global_statusbar_bg end,
							},
							statusbrcolor_bg = {
								order = 2.2,name = L["Background Color"],type = "color", hasAlpha = true,
								set = function(info,r,g,b,a) self.db.profile.global_statusbrcolor_bg={r,g,b,a}; end,
								get = function(info)
									if self.db.profile.global_statusbrcolor_bg then
										return self.db.profile.global_statusbrcolor_bg[1],self.db.profile.global_statusbrcolor_bg[2],self.db.profile.global_statusbrcolor_bg[3],self.db.profile.global_statusbrcolor_bg[4] 
									end
								end
							},
							font = {
								order = 3,name = L["Font"],type = 'select',
								dialogControl = 'LSM30_Font',
								values = SPTimers.SharedMedia:HashTable("font"),
								set = function(info,key) self.db.profile.global_font = key; end,
								get = function(info) return self.db.profile.global_font end,
							},
							font_color = {
								order = 4,name = L["Text color"],type = "color",
								set = function(info,r,g,b) self.db.profile.global_textcolor={r,g,b}; end,
								get = function(info)
									if self.db.profile.global_textcolor then
										return self.db.profile.global_textcolor[1],self.db.profile.global_textcolor[2],self.db.profile.global_textcolor[3],1 
									end
								end
							},
							font_size = {
								name = L["Size"],
								type = "range",
								order	= 5,
								min		= 1,
								max		= 32,
								step	= 1,
								set = function(info,val) 
									self.db.profile.global_size = val
								end,
								get = function(info)
									return	self.db.profile.global_size
								end,
							},
							font_alpha = {
								name = L["Alpha"],
								type = "range",
								order	= 6,
								min		= 0,
								max		= 1,
								step	= 0.1,
								set = function(info,val) 
									self.db.profile.global_alpha = val
								end,
								get = function(info)
									return	self.db.profile.global_alpha
								end,
							},
							fontflaggs = {
								type = "select",	order = 6.1,
								name = L["Flags"],
								values = text_flaggs,
								set = function(info,val) 
									self.db.profile.global_flags = val
								end,
								get = function(info) return self.db.profile.global_flags end
							},
							
							font_shadow_color = {
								order = 6.2,name = "Shadow color",type = "color", hasAlpha = true,
								set = function(info,r,g,b,a) self.db.profile.global_font_shadow={r,g,b,a} end,
								get = function(info) 
									local color = self.db.profile.global_font_shadow or { 0, 0, 0, 1}
									
								return color[1],color[2],color[3],color[4] end
							},

							font_shadow_offset_x = {
								name = "Shadow Offset X",
								type = "range",
								order	= 6.3,
								min		= -10,
								max		= 10,
								step	= 0.1,
								set = function(info,val)
									if not self.db.profile.global_font_shadow_offset then self.db.profile.global_font_shadow_offset = {} end
									self.db.profile.global_font_shadow_offset[1] = val
								end,
								get = function(info)
									return self.db.profile.global_font_shadow_offset and self.db.profile.global_font_shadow_offset[1] or 0
								end,
							},

							font_shadow_offset_y = {
								name = "Shadow Offset Y",
								type = "range",
								order	= 6.4,
								min		= -10,
								max		= 10,
								step	= 0.1,
								set = function(info,val) 
									self.db.profile.global_font_shadow_offset[2] = val
								end,
								get = function(info)
									return self.db.profile.global_font_shadow_offset[2]
								end,
							},
		
							border = {
								order = 7,type = 'select',name = L["Border Texture"],
								dialogControl = 'LSM30_Border',
								values = SPTimers.SharedMedia:HashTable("border"),
								set = function(info,value) self.db.profile.global_border = value; end,
								get = function(info) return self.db.profile.global_border end,
							},
							bordercolor = {
								order = 8,name = L["Border Color"],type = "color", hasAlpha = true,
								set = function(info,r,g,b,a) self.db.profile.global_bordercolor={r,g,b,a}; end,
								get = function(info)
									if self.db.profile.global_bordercolor then
										return self.db.profile.global_bordercolor[1],self.db.profile.global_bordercolor[2],self.db.profile.global_bordercolor[3],self.db.profile.global_bordercolor[4] 
									end
								end
							},
							bordersize = {
								name = L["Border Size"],
								desc = L["Set Border Size"],
								type = "range",
								order	= 9,
								min		= 0,
								max		= 32,
								step	= 1,
								set = function(info,val) 
									self.db.profile.global_bordersize = val
								end,
								get =function(info)
									return self.db.profile.global_bordersize
								end,
							},							
							borderinset = {
								name = L["Border Inset"],
								desc = L["Set Border Inset"],
								type = "range",
								order	= 10,
								min		= 0,
								max		= 32,
								step	= 1,
								set = function(info,val) 
									self.db.profile.global_borderinset = val
								end,
								get =function(info)
									return self.db.profile.global_borderinset
								end,
							},
							apply = {
								type = 'execute',
								order = -1,
								name = APPLY,
								func = function(info, value)
									self:GlobalStyleUpdate()
								end,
							},
						},
					},
					
				}
			},
			bars={
				order = 2,name = L["Bars"],type = "group",
				args={
					bar_module_enabled = {
								order = 1,name = L["Enabled"],type = "toggle", width = "full",
								set = function(info,val) self.db.profile.bar_module_enabled = not self.db.profile.bar_module_enabled; self:UpdateStatusBars(); self:CoreBarsStatusUpdate() end,
								get = function(info) return self.db.profile.bar_module_enabled end
							},
							
							bar_smooth = {
								order = 2,name = L["Smooth bar"],type = "toggle",
								set = function(info,val) self.db.profile.bar_smooth = not self.db.profile.bar_smooth; self:UpdateSmothBars() end,
								get = function(info) return self.db.profile.bar_smooth end
							},
							bar_smooth_value = {
								name = L["Smooth bar speed"],
								type = "range",
								width = "halh",
								order	= 3,
								min		= 1,
								max		= 100,
								step	= 1,
								set = function(info,val) 
									self.db.profile.bar_smooth_value = val
									self:UpdateSmothFPS()
								end,
								get =function(info)
									return self.db.profile.bar_smooth_value
								end,
							},
							casterTargetAura = {
								order = 4,name = L["Show caster target raid debuffs"],type = "toggle", width = "full",
								set = function(info,val) self.db.profile.casterTargetAura = not self.db.profile.casterTargetAura; end,
								get = function(info) return self.db.profile.casterTargetAura end
							},
							
							meleeTargetAura = {
								order = 5,name = L["Show melee target raid debuffs"],type = "toggle", width = "full",
								set = function(info,val) self.db.profile.meleeTargetAura = not self.db.profile.meleeTargetAura; end,
								get = function(info) return self.db.profile.meleeTargetAura end
							},
							
							tankTargetAura = {
								order = 6,name = L["Show tank target raid debuffs"],type = "toggle", width = "full",
								set = function(info,val) self.db.profile.tankTargetAura = not self.db.profile.tankTargetAura; end,
								get = function(info) return self.db.profile.tankTargetAura end
							},
							procsFilters = {
								order = 7,name = L["Hide auras from"].." '"..L["Procs"].."'",type = "toggle", width = "full",
								set = function(info,val) self.db.profile.procsFiltersoff = not self.db.profile.procsFiltersoff; end,
								get = function(info) return self.db.profile.procsFiltersoff end
							},
							othersFilters = {
								order = 8,name = L["Hide auras from"].." '"..L["Other List"].."'",type = "toggle", width = "full",
								set = function(info,val) self.db.profile.othersFiltersoff = not self.db.profile.othersFiltersoff; end,
								get = function(info) return self.db.profile.othersFiltersoff end
							},
							classlistFiltersoff = {
								order = 9,name = L["Hide auras from"].." '"..L["Class Spells"].."'",type = "toggle", width = "full",
								set = function(info,val) self.db.profile.classlistFiltersoff = not self.db.profile.classlistFiltersoff; end,
								get = function(info) return self.db.profile.classlistFiltersoff end
							},
							adapttoonemax = {
								order = 10,name = L["Adapt to one maximum"],type = "toggle", width = "full",
								set = function(info,val) self.db.profile.adapttoonemax = not self.db.profile.adapttoonemax; end,
								get = function(info) return self.db.profile.adapttoonemax end
							},
							maximumtime = {
								order = 11,name = L["Maximum time"],type = "toggle",
								set = function(info,val) self.db.profile.maximumtime = not self.db.profile.maximumtime; end,
								get = function(info) return self.db.profile.maximumtime end
							},
							maximumtime_value = {
								type = "input",	order	= 12,
								name = L["Maximum time"],
								set = function(info,val)
									local num = tonumber(val)
									if num then
										self.db.profile.maximumtime_value = num
									end	
								end,
								get = function(info) return self.db.profile.maximumtime_value and tostring(self.db.profile.maximumtime_value) or "30" end
							},

							background_fading = {
								order = 13,name = L["Background fading"],type = "toggle",
								set = function(info,val) self.db.profile.background_fading = not self.db.profile.background_fading; end,
								get = function(info) return self.db.profile.background_fading end
							},
							back_bar_color = {
								order = 14,name = L["Bar color for background"],type = "toggle",
								set = function(info,val) self.db.profile.back_bar_color = not self.db.profile.back_bar_color; self:UpdateBackgroundBarColor(); end,
								get = function(info) return self.db.profile.back_bar_color end
							},
							
							tick_count_on_stacks = {
								order = 15,name = L["Tick count as stack text"],type = "toggle", width = "full",
								set = function(info,val) self.db.profile.tick_count_on_stacks = not self.db.profile.tick_count_on_stacks; end,
								get = function(info) return self.db.profile.tick_count_on_stacks end
							},
							delayfading = {
								order = 16,name = L["Delay timer fading"],type = "toggle", width = "full",
								set = function(info,val) self.db.profile.delayfading = not self.db.profile.delayfading; end,
								get = function(info) return self.db.profile.delayfading end
							},
							UnitFilter = {		
								type = "group",	order	= 17,
								guiInline = true,
								name	= L["Unit Filter"],
								args = {
									enabled = {
										order = 1,name = L["Enabled"],type = "toggle", width = "full", desc = "|cFFff5c5c\n>>>>EXPERIMENTAL!<<<<|r\n\n\n"..L["Target Type"].." "..L["should be "]..L["multi"].." "..L["or"].." "..L["auto"],
										set = function(info,val) self.db.profile.unit_filter_enabled = not self.db.profile.unit_filter_enabled; end,
										get = function(info) return self.db.profile.unit_filter_enabled end
									},
									unit = {
										name = L["Unit"],
										order = 2,
									--	desc = L["Select Anchor offtarget Desc"].." ->'"..L["General"].."'->'"..L["Swap bars when change target"].."'",
										type = "select",
										values = function()
											local t = {}
											for k,v in pairs(self.db.profile.unit_filters) do						
												if v then
													t[k] = "|cFF0eed28"..k.."|r"
												else
													t[k] = "|cFFff5c5c"..k.."|r"
												end									
											end				
											return t
										end,
										set = function(info,val)
											unit_filter_set = val
										end,
										get = function(info, val)
											return unit_filter_set
										end
									},					
									enabled_unit = { -- unit_filter_set
										order = 3,name = L["Show"],type = "toggle",
										set = function(info,val) 
											if unit_filter_set then 
												self.db.profile.unit_filters[unit_filter_set] = not self.db.profile.unit_filters[unit_filter_set]; 
											end
										end,
										get = function(info)
											return self.db.profile.unit_filters[unit_filter_set]
										end
									},
								}
							},
					style = {
						type = "group",	order	= 1,
						name	= L["Style"],						
						args={
							CreateNew = {
								type = 'execute',
								--width = "half",
								order = 1,
								name = L['Add New Anchor'],
								func = function(info, value) 
									SPTimers:CreateNewAnhors()					
								end,
								},
							Anchor = {
								name = L["Select Anchor"],
								order = 1,
								desc = L["Select Anchor Desc"],
								type = "select",						
								values = function()
									local t = {}
									
									for k,v in ipairs(self.db.profile.bars_anchors) do						
										t[k] = v.name or k
									end
									
									return t
								end,
								set = function(info,val)
									self:SetAnchorTable(val)
								end,
								get = function(info, val) 
									return anchor_value
								end
							},
						},
					},
					ClassSpells = {
						type = "group",	order	= 2,
						name	= L["Class Spells"],						
						args={
							Anchor = {
								name = L["Select Spell"],
								order = 2,
								desc = L["Select Spell"],
							--	width = "half",
								width = "full",
								type = "select",
								values = function()
									local t = {}												
									for k,v in pairs(self.db.profile.classSpells[self.myCLASS]) do						
										t[k] = SpellString(k, 10)
									end								
									return t
								end,
								set = function(info,val)
									SPTimers:ClassSpell(val)
								end,
								get = function(info, val)
									return class_select
								end
							},
							AddNew = {
								type = "input",	order	= 1,
								name = L["Spell ID"],
								desc = L["Change spellID"],
								set = function(info,val)
									local num = tonumber(val)
									if num then
										local spellname = GetSpellInfo(num)										
										if spellname then										
											SPTimers:SearchDBSpell(num, "class")
										end
									end	
								end,
								get = function(info) end
							},						
						}, --done later
					},
					spellList2 = {
						type = "group",	order	= 3,
						name	= L["Procs"],						
						args={
							
							Anchor = {
								name = L["Select Spell"],
								order = 2,
								desc = L["Select Spell"],
							--	width = "half",
								width = "full",
								type = "select",
								values = function()
									local t = {}												
									for k,v in pairs(self.db.profile.procSpells) do						
										t[k] = SpellString(k, 10)
									end									
									return t
								end,
								set = function(info,val)
									SPTimers:ProcsSpell(val)
								end,
								get = function(info, val)
									return proc_select
								end
							},
							AddNew = {
								type = "input",	order	= 1,
								name = L["Spell ID"],
								desc = L["Change spellID"],
								set = function(info,val)
									local num = tonumber(val)
									if num then
										local spellname = GetSpellInfo(num)										
										if spellname then										
											SPTimers:SearchDBSpell(num, "procs")
										end
									end	
								end,
								get = function(info)end
							},
						}, --done later
					  },
					spellList3 = {
						type = "group",	order	= 4,
						name	= L["Other List"],					
						args={
							Anchor = {
								name = L["Select Spell"],
								order = 2,
								desc = L["Select Spell"],
							--	width = "half",
								width = "full",
								type = "select",
								values = function()
									local t = {}												
									for k,v in pairs(self.db.profile.othersSpells) do						
										t[k] = SpellString(k, 10)
									end								
									return t
								end,
								set = function(info,val)
									SPTimers:OthersSpell(val)
								end,
								get = function(info, val)
									return others_select
								end
							},
							AddNew = {
								type = "input",	order	= 1,
								name = L["Spell ID"],
								desc = L["Change spellID"],
								set = function(info,val)
									local num = tonumber(val)
									if num then
										local spellname = GetSpellInfo(num)										
										if spellname then										
											SPTimers:SearchDBSpell(num, "others")
										end
									end	
								end,
								get = function(info) end
							},
						}, --done later
					  },
					}
			},
			coolline = {
				order = 3,name = L["Coolline"],type = "group",
				args={
					ICD = {
						type = "group",	order	= 2,
						name	= L["ICD"],						
						args={						
							Anchor = {
								name = L["Select Spell"],
								order = 2,
								desc = L["Select Spell"],
								width = "full",
							--	width = "half",
								type = "select",
								values = function()
									local t = {}												
									for k,v in pairs(self.db.profile.internal_cooldowns) do						
										t[k] = SpellString(v.spellid, 10)
									end								
									return t
								end,
								set = function(info,val)
									SPTimers:ICooldown(val)
								end,
								get = function(info, val)
									return ICD_select
								end
							},
							AddNew = {
								type = "input",	order	= 1,
								name = L["Spell ID"],
								desc = L["Change spellID"],
								set = function(info,val)
									local num = tonumber(val)
									if num then
										local spellname = GetSpellInfo(num)										
										if spellname then
											if not self.db.profile.internal_cooldowns then self.db.profile.internal_cooldowns = {} end											
											if not self.db.profile.internal_cooldowns[spellname] then self.db.profile.internal_cooldowns[spellname] = {} end
											
											self.db.profile.internal_cooldowns[spellname].spellid = num
											SPTimers:ICooldown(spellname)
										end
									end	
								end,
								get = function(info) end
							},	
						
						},
					},
					ClassSpells = {
						type = "group",	order	= 3,
						name	= L["Class CD"],						
						args={						
							Anchor = {
								name = L["Select Spell"],
								order = 1,
								desc = L["Select Spell"],
								type = "select",
								width = "full",
								values = function()
									local t = {}												
									for k,v in pairs(self.db.profile.classCooldowns[self.myCLASS]) do						
										t[k] = SpellString(v.spellid, 10)
									end								
									return t
								end,
								set = function(info,val)
									SPTimers:Cooldown(val)
								end,
								get = function(info, val)
									return cooldown_select
								end
							},
							AddNew = {
								type = "input",	order	= 1,
								name = L["Spell ID"],
								desc = L["Change spellID"],
								set = function(info,val)
									local num = tonumber(val)
									if num then
										local spellname = GetSpellInfo(num)										
										if spellname then
											if not self.db.profile.classCooldowns[SPTimers.myCLASS] then self.db.profile.classCooldowns[SPTimers.myCLASS] = {} end											
											if not self.db.profile.classCooldowns[SPTimers.myCLASS][spellname] then self.db.profile.classCooldowns[SPTimers.myCLASS][spellname] = {} end
											
											self.db.profile.classCooldowns[self.myCLASS][spellname].spellid = num
											
											SPTimers:Cooldown(spellname)
										end
									end	
								end,
								get = function(info) end
							},						
						},
					},
					BanCooldown = {
						type = "group",	order	= 4,
						name	= L["Other List"],
						args = {
							Anchor = {
								name = L["Select CD"],
								order = 3,
								type = "select",
								width = "full",
								values = function()
									local t = {}												
									for k,v in pairs(self.db.profile.cooldownline.block) do
										if v.hide then
											t[k] = "|cFFff5c5c"..k.."|r"
										else
											t[k] = "|cFF0eed28"..k.."|r"
										end
									end								
									return t
								end,
								set = function(info,val)
									blocklist_select = val
								end,
								get = function(info, val)
									return blocklist_select
								end
							},
							AddNewSpellID = {
								type = "input",	order	= 1,
								name = L["Spell ID"],
								desc = L["Add new spellID"],
								set = function(info,val)
									local num = tonumber(val)
									if num then
										local spellname = GetSpellInfo(num)										
										if spellname then											
											self.db.profile.cooldownline.block[spellname] = { spellid = num, hide = true }
											
											blocklist_select = spellname
										end
									end	
								end,
								get = function(info) end
							},
							AddNewItem = {
								type = "input",	order	= 2,
								name = L["Item ID"],
								desc = L["Add new ItemID"],
								set = function(info,val)
									local num = tonumber(val)
									if num then
										local itemname = GetItemInfo(num)										
										if itemname then											
											self.db.profile.cooldownline.block[itemname] = { itemid = num, hide = true }
											
											blocklist_select = itemname
										end
									end	
								end,
								get = function(info) end
							},
							enabled = {
								order = 4,name = L["Hide"],type = "toggle" ,
								set = function(info,val) 
									if blocklist_select then
										self.db.profile.cooldownline.block[blocklist_select].hide = not self.db.profile.cooldownline.block[blocklist_select].hide
									end
								end,
								get = function(info)
									if not blocklist_select then return nil end
									
							--		print(blocklist_select)
									
									return self.db.profile.cooldownline.block[blocklist_select] and self.db.profile.cooldownline.block[blocklist_select].hide or false
								
								end
							},
							annonce = {
								order = 5,name = L["Reporting"], type = "toggle",
								desc = L["Turn on/off spell cooldown report"],
								set = function(info,val) 
									if blocklist_select then 
										self.db.profile.cooldownline.block[blocklist_select].reporting  = not self.db.profile.cooldownline.block[blocklist_select].reporting 
									end
								end,
								get = function(info) 
									if not blocklist_select then return nil end
									return self.db.profile.cooldownline.block[blocklist_select] and self.db.profile.cooldownline.block[blocklist_select].reporting or false 
									
								end
							},
							hide_splash = {
								order = 5.1,name = "Forced big splash", type = "toggle",
								set = function(info,val)
									if blocklist_select then 
										self.db.profile.cooldownline.block[blocklist_select].hide_splash = not self.db.profile.cooldownline.block[blocklist_select].hide_splash 
									end
								end,
								get = function(info)
									if not blocklist_select then return nil end
									return self.db.profile.cooldownline.block[blocklist_select] and self.db.profile.cooldownline.block[blocklist_select].hide_splash or false
								end
							},						
							SoundFile = {					
								type = "group",	order	= 6,
								guiInline = true,
								name	= L["Sound"],
								args = {						
									OnShow = {
										order = 1,type = 'select',name = L["On Show"],
										dialogControl = 'LSM30_Sound',
										values = SPTimers.SharedMedia:HashTable("sound"),
										set = function(info,value) 
											if blocklist_select then 
												self.db.profile.cooldownline.block[blocklist_select].sound_onshow = value
											end
										end,
										get = function(info) 
											if not blocklist_select then return "None" end
											return self.db.profile.cooldownline.block[blocklist_select] and self.db.profile.cooldownline.block[blocklist_select].sound_onshow or "None"; 
										end,
									},
									OnHide = {
										order = 1,type = 'select',name = L["On Hide"],
										dialogControl = 'LSM30_Sound',
										values = SPTimers.SharedMedia:HashTable("sound"),
										set = function(info,value) 
											if blocklist_select then
												self.db.profile.cooldownline.block[blocklist_select].sound_onhide = value 
											end
										end,
										get = function(info) 
											if not blocklist_select then return "None" end
											return self.db.profile.cooldownline.block[blocklist_select] and self.db.profile.cooldownline.block[blocklist_select].sound_onhide or "None";
										end,
									},
								}
							},
							delete = {
								type = 'execute',
								order = -1,
								name = L['Delete'],
								func = function(info, value)
									if self.db.profile.cooldownline.block[blocklist_select] then
										self.db.profile.cooldownline.block[blocklist_select] = nil
									end
									blocklist_select = nil								
								end,
							},
						},
					},
					desc1 = {
						type = "header",
						name = "",
						order = 1,
					},
					enabled = {
						order = 2,name = L["Enabled"],type = "toggle",
						set = function(info,val) self.db.profile.cooldownline.enabled = not self.db.profile.cooldownline.enabled; self.UpdateSettings() end,
						get = function(info) return self.db.profile.cooldownline.enabled end
					},
					hide_cooldown_line = {
						order = 2.1,name = L["Hide"],desc = L["Only Hide cooldown line, so big cooldown spash will continue to work"], type = "toggle",
						set = function(info,val) self.db.profile.cooldownline.hide_cooldown_line = not self.db.profile.cooldownline.hide_cooldown_line; self.UpdateSettings()end,
						get	= function(info) return self.db.profile.cooldownline.hide_cooldown_line end
					},
					enabled_mouse_events = {
						order = 2.11,name = L["Enable mouse events"],type = "toggle" ,
						set = function(info,val) self.db.profile.cooldownline.mouse_events = not self.db.profile.cooldownline.mouse_events; self.UpdateSettings() end,
						get = function(info) return self.db.profile.cooldownline.mouse_events end
					},
					
					hide_fail = {
						order = 2.2,name = L["Hide fail glow"],type = "toggle",
						set = function(info,val) self.db.profile.cooldownline.hidefail = not self.db.profile.cooldownline.hidefail; self.UpdateSettings() end,
						get = function(info) return self.db.profile.cooldownline.hidefail end
						},
						
					tooltip = {
					
						type = "group",	order	= 2.21,
						guiInline = true,
						name	= L["Tooltip"],
						args = {
							show_tooltip = {
								order = 1,name = L["Show"],type = "toggle",
								set = function(info,val) self.db.profile.cooldownline.show_tooltip = not self.db.profile.cooldownline.show_tooltip end,
								get	= function(info) return self.db.profile.cooldownline.show_tooltip end
							},
							tooltip_anchor_to = {
										name = L["Position"],
										order = 2.21,
										type = "select",
										values = {
											L["TOP"],
											L["BOTTOM"],
											L["LEFT"],
											L["RIGHT"],
										},
										set = function(info,val)
											self.db.profile.cooldownline.tooltip_anchor_to = val
											self:UpdateTooltip()
										end,
										get = function(info, val)
											return self.db.profile.cooldownline.tooltip_anchor_to
										end
									},
							tooltip_anchor_to_frame = {
										name = L["Select Anchor"],
										order = 2.22,
										type = "select",
										values = {
											L["ICON"],
											L["LINE"],
										},
										set = function(info,val)
											self.db.profile.cooldownline.tooltip_anchor_to_frame = val
											self:UpdateTooltip()
										end,
										get = function(info, val)
											return self.db.profile.cooldownline.tooltip_anchor_to_frame
										end
									},

							tooltip_anchor_gap = {
								name = L["Gap"],
								type = "range",
								order	= 3,
								min		= -400,
								max		= 400,
								step	= 1,
								set = function(info,val) 
									self.db.profile.cooldownline.tooltip_anchor_gap = val
									self:UpdateTooltip()
								end,
								get =function(info)
									return self.db.profile.cooldownline.tooltip_anchor_gap
								end,
							
							
							}
						}
					},
					player_color_group = {
					
						type = "group",	order	= 2.3,
						guiInline = true,
						name	= L["Player"],
						args = {
							player_color = {
								order = 1, name = L["Color"],type = "color", hasAlpha = false, width = "half",
								set = function(info,r,g,b) self.db.profile.cooldownline.pla_color={r,g,b}; self.UpdateSettings() end,
								get = function(info) return self.db.profile.cooldownline.pla_color[1],self.db.profile.cooldownline.pla_color[2],self.db.profile.cooldownline.pla_color[3],1 end
								},
							hide_player = {
								order = 2,name = L["Hide"],type = "toggle", width = "half",
								set = function(info,val) self.db.profile.cooldownline.hideplay = not self.db.profile.cooldownline.hideplay; self.UpdateSettings() end,
								get = function(info) return self.db.profile.cooldownline.hideplay end
								},
							annonce = {
								order = 3,name = L["Reporting"], type = "toggle", width = "half",
								desc = L["Turn on/off spell cooldown report"],
								set = function(info,val) self.db.profile.cooldownline.player_reporting = not self.db.profile.cooldownline.player_reporting end,
								get = function(info) return self.db.profile.cooldownline.player_reporting end
							}
						},
					},
					
					pet_color_group = {
					
						type = "group",	order	= 2.4,
						guiInline = true,
						name	= L["Pet"],
						args = {
							pet_color = {
								order = 5, name = L["Color"],type = "color", hasAlpha = false, width = "half",
								set = function(info,r,g,b) self.db.profile.cooldownline.pet_color={r,g,b}; self.UpdateSettings() end,
								get = function(info) return self.db.profile.cooldownline.pet_color[1],self.db.profile.cooldownline.pet_color[2],self.db.profile.cooldownline.pet_color[3],1 end
								},
							hide_pet = {
								order = 6,name = L["Hide"],type = "toggle", width = "half",
								set = function(info,val) self.db.profile.cooldownline.hidepet = not self.db.profile.cooldownline.hidepet; self.UpdateSettings() end,
								get = function(info) return self.db.profile.cooldownline.hidepet end
								},
							annonce = {
								order = 7,name = L["Reporting"], type = "toggle", width = "half",
								desc = L["Turn on/off spell cooldown report"],
								set = function(info,val) self.db.profile.cooldownline.pet_reporting = not self.db.profile.cooldownline.pet_reporting end,
								get = function(info) return self.db.profile.cooldownline.pet_reporting end
							}
						},
					},
					
					bag_color_group = {
					
						type = "group",	order	= 2.5,
						guiInline = true,
						name	= L["Bags"],
						args = {
							bag_color = {
								order = 7, name = L["Color"],type = "color", hasAlpha = false, width = "half",
								set = function(info,r,g,b) self.db.profile.cooldownline.bag_color={r,g,b}; self.UpdateSettings() end,
								get = function(info) return self.db.profile.cooldownline.bag_color[1],self.db.profile.cooldownline.bag_color[2],self.db.profile.cooldownline.bag_color[3],1 end
								},
							hide_bag = {
								order = 8,name = L["Hide"],type = "toggle", width = "half",
								set = function(info,val) self.db.profile.cooldownline.hidebag = not self.db.profile.cooldownline.hidebag; self.UpdateSettings() end,
								get = function(info) return self.db.profile.cooldownline.hidebag end
								},
							annonce = {
								order = 9,name = L["Reporting"], type = "toggle", width = "half",
								desc = L["Turn on/off spell cooldown report"],
								set = function(info,val) self.db.profile.cooldownline.bag_reporting = not self.db.profile.cooldownline.bag_reporting end,
								get = function(info) return self.db.profile.cooldownline.bag_reporting end
							}
						},
					},
					
					inv_color_group = {
					
						type = "group",	order	= 2.6,
						guiInline = true,
						name	= L["Equipped Items"],
						args = {
							inv_color = {
								order = 9, name = L["Color"],type = "color", hasAlpha = false, width = "half",
								set = function(info,r,g,b) self.db.profile.cooldownline.inv_color={r,g,b}; self.UpdateSettings() end,
								get = function(info) return self.db.profile.cooldownline.inv_color[1],self.db.profile.cooldownline.inv_color[2],self.db.profile.cooldownline.inv_color[3],1 end
								},
							hide_inv = {
								order = 10,name = L["Hide"],type = "toggle", width = "half",
								set = function(info,val) self.db.profile.cooldownline.hideinv = not self.db.profile.cooldownline.hideinv; self.UpdateSettings() end,
								get = function(info) return self.db.profile.cooldownline.hideinv end
								},
							annonce = {
								order = 11,name = L["Reporting"], type = "toggle", width = "half",
								desc = L["Turn on/off spell cooldown report"],
								set = function(info,val) self.db.profile.cooldownline.inv_reporting = not self.db.profile.cooldownline.inv_reporting end,
								get = function(info) return self.db.profile.cooldownline.inv_reporting end
							}
						},
					},
					
					veh_color_group = {
					
						type = "group",	order	= 2.7,
						guiInline = true,
						name	= L["Vehicle"],
						args = {
							veh_color = {
								order = 11, name = L["Color"],type = "color", hasAlpha = false, width = "half",
								set = function(info,r,g,b) self.db.profile.cooldownline.veh_color={r,g,b}; self.UpdateSettings() end,
								get = function(info) return self.db.profile.cooldownline.veh_color[1],self.db.profile.cooldownline.veh_color[2],self.db.profile.cooldownline.veh_color[3],1 end
								},
							hide_veh = {
								order = 12,name = L["Hide"],type = "toggle", width = "half",
								set = function(info,val) self.db.profile.cooldownline.hidevehi = not self.db.profile.cooldownline.hidevehi; self.UpdateSettings() end,
								get = function(info) return self.db.profile.cooldownline.hidevehi end
								},
							annonce = {
								order = 13,name = L["Reporting"], type = "toggle", width = "half",
								desc = L["Turn on/off spell cooldown report"],
								set = function(info,val) self.db.profile.cooldownline.veh_reporting = not self.db.profile.cooldownline.veh_reporting end,
								get = function(info) return self.db.profile.cooldownline.veh_reporting end
							}
						},
					},
					
					icd_color_group = {
					
						type = "group",	order	= 2.8,
						guiInline = true,
						name	= L["Internal cd"],
						args = {
							inter_color = {
								order = 13, name = L["Color"],type = "color", hasAlpha = false, width = "half",
								set = function(info,r,g,b) self.db.profile.cooldownline.inter_color={r,g,b}; self.UpdateSettings() end,
								get = function(info) return self.db.profile.cooldownline.inter_color[1],self.db.profile.cooldownline.inter_color[2],self.db.profile.cooldownline.inter_color[3],1 end
								},
							hideinternal = {
								order = 14,name = L["Hide"],type = "toggle", width = "half",
								set = function(info,val) self.db.profile.cooldownline.hideinternal = not self.db.profile.cooldownline.hideinternal; self.UpdateSettings() end,
								get = function(info) return self.db.profile.cooldownline.hideinternal end
								},
							annonce = {
								order = 15,name = L["Reporting"], type = "toggle", width = "half",
								desc = L["Turn on/off spell cooldown report"],
								set = function(info,val) self.db.profile.cooldownline.icd_reporting = not self.db.profile.cooldownline.icd_reporting end,
								get = function(info) return self.db.profile.cooldownline.icd_reporting end
							}
						},
					},

					cooldownline_style = {
						type = "group",	order	= 1,
						name	= L["Style"],
						args = {
						
							desc4 = {
							type = "header",
							name = L["Line config"],
							order = 15,
						},
						
						reverse = {
							order = 16,name = L["Reverse"],type = "toggle",
							set = function(info,val) self.db.profile.cooldownline.reverse = not self.db.profile.cooldownline.reverse; self.UpdateSettings() end,
							get = function(info) return self.db.profile.cooldownline.reverse end
						},

						custom_text_timer = {
							type = "input",	order	= 17,
							name = L["Custom Text"],
							desc = L["Coolline Custom Text Desc"],
							set = function(info,val) 
								local num = tostring(val)				
								if num then
									self.db.profile.cooldownline.custom_text_timer = num
									self.UpdateSettings()
								end
							end,
							get = function(info) return tostring(self.db.profile.cooldownline.custom_text_timer) end
						},
						vertical = {
							order = 16,name = L["Vertical"],type = "toggle",
							set = function(info,val) self.db.profile.cooldownline.vertical = not self.db.profile.cooldownline.vertical; self.UpdateSettings() end,
							get = function(info) return self.db.profile.cooldownline.vertical end
						},
						
						hidelinetext = {
							order = 17.1,name = L["Hide line text"],type = "toggle",
							set = function(info,val) self.db.profile.cooldownline.hidelinetext = not self.db.profile.cooldownline.hidelinetext; self.UpdateSettings() end,
							get = function(info) return self.db.profile.cooldownline.hidelinetext end
						},
						
						format_s = {
							name = L["Time format"],
							order = 18,
							desc = L["Select timer format Desc"],
							type = "select",						
							values = cooldown_timer_format,
							set = function(info,val)
								 self.db.profile.cooldownline.fortam_s = val
							end,
							get = function(info, val) 
								return  self.db.profile.cooldownline.fortam_s
							end
						},			
						bg = {
							order = 19,type = 'select',name = L["Main Texture"],
							dialogControl = 'LSM30_Statusbar',
							values = SPTimers.SharedMedia:HashTable("statusbar"),
							set = function(info,value) self.db.profile.cooldownline.statusbar = value; self.UpdateSettings() end,
							get = function(info) return self.db.profile.cooldownline.statusbar end,
						},
						bg_color = {
							order = 20,name = L["Texture Color"],type = "color", hasAlpha = true,
							set = function(info,r,g,b,a) self.db.profile.cooldownline.bgcolor={r = r,g = g,b = b, a = a}; self.UpdateSettings()  end,
							get = function(info) return self.db.profile.cooldownline.bgcolor.r,self.db.profile.cooldownline.bgcolor.g,self.db.profile.cooldownline.bgcolor.b,self.db.profile.cooldownline.bgcolor.a end
						},
						hidestatusbar = {
							order = 20.1,name = L["Hide cd texture"],type = "toggle",
							set = function(info,val) self.db.profile.cooldownline.hidestatusbar = not self.db.profile.cooldownline.hidestatusbar; self.UpdateSettings() end,
							get = function(info) return self.db.profile.cooldownline.hidestatusbar end
						},
						w = {
							name = L["Width"],
							desc = L["Set bar Width"],
							type = "range",
							order	= 21,
							min		= 1,
							max		= 1920,
							step	= 1,
							set = function(info,val) 
								self.db.profile.cooldownline.w = val
								self.UpdateSettings()
							end,
							get =function(info)
								return self.db.profile.cooldownline.w
							end,
						},
						h = {
							name = L["Height"],
							desc = L["Set bar Height"],
							type = "range",
							order	= 22,
							min		= 1,
							max		= 32,
							step	= 1,
							set = function(info,val) 
								self.db.profile.cooldownline.h = val
								self.UpdateSettings()
							end,
							get =function(info)
								return self.db.profile.cooldownline.h
							end,
						},
						activealpha = {
							name = L["Active Alpha"],
							desc = L["Set alpha when cooldown line is active"],
							type = "range",
							order	= 23,
							min		= 0,
							max		= 1,
							step	= 0.1,
							set = function(info,val) 
								self.db.profile.cooldownline.activealpha = val
								self.UpdateSettings()
							end,
							get =function(info)
								return self.db.profile.cooldownline.activealpha
							end,
						},
						inactivealpha = {
							name = L["Inactiv Alpha"],
							desc = L["Set alpha when cooldown line is inactive"],
							type = "range",
							order	= 24,
							min		= 0,
							max		= 1,
							step	= 0.1,
							set = function(info,val) 
								self.db.profile.cooldownline.inactivealpha = val
								self.UpdateSettings()
							end,
							get =function(info)
								return self.db.profile.cooldownline.inactivealpha
							end,
						},
						borderDsk = {
							type = "header",
							name = L["Borders"],
							order = 25,
						},
						border = {
							order = 26,type = 'select',name = L["Border Texture"],
							dialogControl = 'LSM30_Border',
							values = SPTimers.SharedMedia:HashTable("border"),
							set = function(info,value) self.db.profile.cooldownline.border = value; self.UpdateSettings() end,
							get = function(info) return self.db.profile.cooldownline.border end,
						},
						bordersize = {
							name = L["Border Size"],
							desc = L["Set Border Size"],
							type = "range",
							order	= 27,
							min		= 1,
							max		= 32,
							step	= 1,
							set = function(info,val) 
								self.db.profile.cooldownline.bordersize = val
								self.UpdateSettings()
							end,
							get =function(info)
								return self.db.profile.cooldownline.bordersize
							end,
						},
						bordercolor = {
							order = 28,name = L["Border Color"],type = "color", hasAlpha = true,
							set = function(info,r,g,b,a) self.db.profile.cooldownline.bordercolor={r = r,g = g,b = b, a = a}; self.UpdateSettings()  end,
							get = function(info) return self.db.profile.cooldownline.bordercolor.r,self.db.profile.cooldownline.bordercolor.g,self.db.profile.cooldownline.bordercolor.b,self.db.profile.cooldownline.bordercolor.a end
						},
						borderinset = {
							name = L["Border Inset"],
							desc = L["Set Border Inset"],
							type = "range",
							order	= 29,
							min		= -32,
							max		= 32,
							step	= 1,
							set = function(info,val) 
								self.db.profile.cooldownline.borderinset = val
								self.UpdateSettings()
							end,
							get =function(info)
								return self.db.profile.cooldownline.borderinset
							end,
						},

						icon_backdrop = {
							type = "header",
							name = L["Icon"],
							order = 29.1,
						},
						icon_border = {
							order = 29.2,type = 'select',name = L["Border Texture"],
							dialogControl = 'LSM30_Border',
							values = SPTimers.SharedMedia:HashTable("border"),
							set = function(info,value) self.db.profile.cooldownline.icon_border = value; self.UpdateSettings() end,
							get = function(info) return self.db.profile.cooldownline.icon_border end,
						},
						icon_bordersize = {
							name = L["Border Size"],
							desc = L["Set Border Size"],
							type = "range",
							order	= 29.3,
							min		= 1,
							max		= 32,
							step	= 1,
							set = function(info,val) 
								self.db.profile.cooldownline.icon_bordersize = val
								self.UpdateSettings()
							end,
							get =function(info)
								return self.db.profile.cooldownline.icon_bordersize
							end,
						},
						icon_bordercolor = {
							order = 29.4,name = L["Border Color"],type = "color", hasAlpha = true,
							set = function(info,r,g,b,a) self.db.profile.cooldownline.icon_bordercolor={r = r,g = g,b = b, a = a}; self.UpdateSettings()  end,
							get = function(info) return self.db.profile.cooldownline.icon_bordercolor.r,self.db.profile.cooldownline.icon_bordercolor.g,self.db.profile.cooldownline.icon_bordercolor.b,self.db.profile.cooldownline.icon_bordercolor.a end
						},
						icon_borderinset = {
							name = L["Border Inset"],
							desc = L["Set Border Inset"],
							type = "range",
							order	= 29.5,
							min		= -32,
							max		= 32,
							step	= 1,
							set = function(info,val) 
								self.db.profile.cooldownline.icon_borderinset = val
								self.UpdateSettings()
							end,
							get =function(info)
								return self.db.profile.cooldownline.icon_borderinset
							end,
						},
						
						icon_background_inset = {
							name = L["Background Inset"],
							desc = L["Set Background Inset"],
							type = "range",
							order	= 29.6,
							min		= -32,
							max		= 32,
							step	= 1,
							set = function(info,val) 
								self.db.profile.cooldownline.icon_background_inset = val
								self.UpdateSettings()
							end,
							get =function(info)
								return self.db.profile.cooldownline.icon_background_inset
							end,
						},
						icon_background_color = {
							order = 29.4,name = L["Background Color"],type = "color", hasAlpha = true,
							set = function(info,r,g,b,a) self.db.profile.cooldownline.icon_backgroundcolor={r = r,g = g,b = b, a = a}; self.UpdateSettings()  end,
							get = function(info) return self.db.profile.cooldownline.icon_backgroundcolor.r,self.db.profile.cooldownline.icon_backgroundcolor.g,self.db.profile.cooldownline.icon_backgroundcolor.b,self.db.profile.cooldownline.icon_backgroundcolor.a end
						},
						iconplus = {
							name = L["Icon plus"],
							type = "range",
							order	= 29.7,
							min		= 1,
							max		= 32,
							step	= 1,
							set = function(info,val) 
								self.db.profile.cooldownline.iconplus = val
								self.UpdateSettings()
							end,
							get = function(info)
								return	self.db.profile.cooldownline.iconplus
							end,
						},
						desc2 = {
							type = "header",
							name = L["Line fonts"],
							order = 30,
						},
						textcolor = {
							order = 31,name = L["Text color"],type = "color", hasAlpha = true,
							set = function(info,r,g,b,a) self.db.profile.cooldownline.fontcolor={r=r, g=g,b=b, a=a}; self.UpdateSettings() end,
							get = function(info) return self.db.profile.cooldownline.fontcolor.r,self.db.profile.cooldownline.fontcolor.g,self.db.profile.cooldownline.fontcolor.b,self.db.profile.cooldownline.fontcolor.a end
						},
						font = {
							order = 32,name = L["Font"],type = 'select',
							dialogControl = 'LSM30_Font',
							values = SPTimers.SharedMedia:HashTable("font"),
							set = function(info,key) self.db.profile.cooldownline.font = key; self.UpdateSettings() end,
							get = function(info) return self.db.profile.cooldownline.font end,
						},
						fontsize = {
							name = L["Size"],
							type = "range",
							order	= 33,
							min		= 1,
							max		= 32,
							step	= 1,
							set = function(info,val) 
								self.db.profile.cooldownline.fontsize = val
								self.UpdateSettings()
							end,
							get = function(info)
								return	self.db.profile.cooldownline.fontsize
							end,
						},
						fontflaggs = {
							type = "select",	order = 34,
							name = L["Flags"],
							values = text_flaggs,
							set = function(info,val) 
								self.db.profile.cooldownline.fontflags = val
								self.UpdateSettings()
							end,
							get = function(info) return self.db.profile.cooldownline.fontflags end
						},
						textshadowcolor = {
							order = 34.1,name = "Text Shadow color",type = "color", hasAlpha = true,
							set = function(info,r,g,b,a) self.db.profile.cooldownline.fontshadowcolor={r=r, g=g,b=b, a=a}; self.UpdateSettings() end,
							get = function(info) 
								local color = self.db.profile.cooldownline.fontshadowcolor or { r=0,g=0,b=0,a=1}
								
								return color.r,color.g,color.b,color.a 
							end
						},
						fontshadowoffset_x = {
							name = "Shadow offset X",
							type = "range",
							order	= 34.2,
							min		= -10,
							max		= 10,
							step	= 0.1,
							set = function(info,val)
								if not self.db.profile.cooldownline.fontshadowoffset then self.db.profile.cooldownline.fontshadowoffset = {} end
								self.db.profile.cooldownline.fontshadowoffset[1] = val
								self.UpdateSettings()
							end,
							get = function(info)
								return self.db.profile.cooldownline.fontshadowoffset and self.db.profile.cooldownline.fontshadowoffset[1] or 0
							end,
						},
						fontshadowoffset_y = {
							name = "Shadow offset Y",
							type = "range",
							order	= 34.3,
							min		= -10,
							max		= 10,
							step	= 0.1,
							set = function(info,val)
								if not self.db.profile.cooldownline.fontshadowoffset then self.db.profile.cooldownline.fontshadowoffset = {} end
								self.db.profile.cooldownline.fontshadowoffset[2] = val
								self.UpdateSettings()
							end,
							get = function(info)
								return self.db.profile.cooldownline.fontshadowoffset and self.db.profile.cooldownline.fontshadowoffset[2] or 0
							end,
						},
						desc3 = {
							type = "header",
							name = L["Icon fonts"],
							order = 35,
						},
						icon_fontcolor = {
							order = 36,name = L["Text color"],type = "color", hasAlpha = true,
							set = function(info,r,g,b,a) self.db.profile.cooldownline.icon_fontcolor={r=r, g=g,b=b, a=a}; self.UpdateSettings() end,
							get = function(info) return self.db.profile.cooldownline.icon_fontcolor.r,self.db.profile.cooldownline.icon_fontcolor.g,self.db.profile.cooldownline.icon_fontcolor.b,self.db.profile.cooldownline.icon_fontcolor.a end
						},
						icon_font= {
							order = 37,name = L["Font"],type = 'select',
							dialogControl = 'LSM30_Font',
							values = SPTimers.SharedMedia:HashTable("font"),
							set = function(info,key) self.db.profile.cooldownline.icon_font = key; self.UpdateSettings() end,
							get = function(info) return self.db.profile.cooldownline.icon_font end,
						},
						icon_fontsize = {
							name = L["Size"],
							type = "range",
							order	= 38,
							min		= 1,
							max		= 32,
							step	= 1,
							set = function(info,val) 
								self.db.profile.cooldownline.icon_fontsize = val
								self.UpdateSettings()
							end,
							get = function(info)
								return	self.db.profile.cooldownline.icon_fontsize
							end,
						},
						icon_fontflaggs = {
							type = "select",	order = 39,
							name = L["Flags"],
							values = text_flaggs,
							set = function(info,val) 
								self.db.profile.cooldownline.icon_fontflaggs = val
								self.UpdateSettings()
							end,
							get = function(info) return self.db.profile.cooldownline.icon_fontflaggs end
						},
						
						icon_textshadowcolor = {
							order = 39.1,name = "Text Shadow color",type = "color", hasAlpha = true,
							set = function(info,r,g,b,a) self.db.profile.cooldownline.icon_fontshadowcolor={r=r, g=g,b=b, a=a}; self.UpdateSettings() end,
							get = function(info) 
								local color = self.db.profile.cooldownline.icon_fontshadowcolor or {r=0, g=0, b=0,a=1}
								return color.r,color.g,color.b,color.a 
							end
						},
						icon_fontshadowoffset_x = {
							name = "Shadow offset X",
							type = "range",
							order	= 39.2,
							min		= -10,
							max		= 10,
							step	= 0.1,
							set = function(info,val)
								if not self.db.profile.cooldownline.icon_fontshadowoffset then self.db.profile.cooldownline.icon_fontshadowoffset = {} end
								self.db.profile.cooldownline.icon_fontshadowoffset[1] = val
								self.UpdateSettings()
							end,
							get = function(info)
								return self.db.profile.cooldownline.icon_fontshadowoffset and self.db.profile.cooldownline.icon_fontshadowoffset[1] or 0
							end,
						},
						icon_fontshadowoffset_y = {
							name = "Shadow offset Y",
							type = "range",
							order	= 39.2,
							min		= -10,
							max		= 10,
							step	= 0.1,
							set = function(info,val)
								if not self.db.profile.cooldownline.icon_fontshadowoffset then self.db.profile.cooldownline.icon_fontshadowoffset = {} end
								self.db.profile.cooldownline.icon_fontshadowoffset[2] = val
								self.UpdateSettings()
							end,
							get = function(info)
								return self.db.profile.cooldownline.icon_fontshadowoffset and self.db.profile.cooldownline.icon_fontshadowoffset[2] or 0
							end,
						},
						
						descSplashBig = {
							type = "header",
							name = L["Big splash"],
							order = 41,
						},
						slash_size = {
							name = L["Size"],
							type = "range",
							order	= 43,
							min		= 1,
							max		= 400,
							step	= 1,
							set = function(info,val) 
								self.db.profile.cooldownline.slash_size = val
								self.UpdateSettings()
							end,
							get = function(info)
								return	self.db.profile.cooldownline.slash_size
							end,
						},
						slash_alpha = {
							name = L["Alpha"],
							type = "range",
							order	= 44,
							min		= 0,
							max		= 1,
							step	= 0.1,
							set = function(info,val) 
								self.db.profile.cooldownline.slash_alpha = val
								self.UpdateSettings()
							end,
							get = function(info)
								return	self.db.profile.cooldownline.slash_alpha
							end,
						},
						slash_show = {
							order = 42,name = L["Show"],type = "toggle",
							set = function(info,val) self.db.profile.cooldownline.slash_show = not self.db.profile.cooldownline.slash_show; self.UpdateSettings() end,
							get = function(info) return self.db.profile.cooldownline.slash_show end
						},
						splash_background_color = {
							order = 42.1,name = L["Background Color"],type = "color", hasAlpha = true,
							set = function(info,r,g,b,a) self.db.profile.cooldownline.splash_background_color={r = r,g = g,b = b, a = a}; self.UpdateSettings()  end,
							get = function(info) return self.db.profile.cooldownline.splash_background_color.r,self.db.profile.cooldownline.splash_background_color.g,self.db.profile.cooldownline.splash_background_color.b,self.db.profile.cooldownline.splash_background_color.a end
						},
						show_only_force = {
							order = 42.2,width = "half",name = "Show forced",type = "toggle", desc = "If checked then show only forced spells. Otherwise do not show splash for forced spells.",
							set = function(info,val) self.db.profile.cooldownline.show_only_force = not self.db.profile.cooldownline.show_only_force; end,
							get = function(info) return self.db.profile.cooldownline.show_only_force end
						},
			
						time_in = {
							name = L["Splash In"],
							desc = L["Big Splash In Desc"],
							type = "range",
							order	= 45,
							min		= 0,
							max		= 1,
							step	= 0.1,
							set = function(info,val) 
								self.db.profile.cooldownline.splash_big.time_in = val
								self.UpdateSettings()
							end,
							get = function(info)
								return	self.db.profile.cooldownline.splash_big.time_in
							end,
						},
						time_out = {
							name = L["Splash Out"],
							desc = L["Big Splash Out Desc"],
							type = "range",
							order	= 46,
							min		= 0,
							max		= 1,
							step	= 0.1,
							set = function(info,val) 
								self.db.profile.cooldownline.splash_big.time_out = val
								self.UpdateSettings()
							end,
							get = function(info)
								return	self.db.profile.cooldownline.splash_big.time_out
							end,
						},
						step_in = {
							name = L["Step In"],
							desc = L["Big Step In Desc"],
							type = "range",
							order	= 47,
							min		= -5,
							max		= 5,
							step	= 0.1,
							set = function(info,val) 
								self.db.profile.cooldownline.splash_big.step_in = val
								self.UpdateSettings()
							end,
							get = function(info)
								return	self.db.profile.cooldownline.splash_big.step_in
							end,
						},
						step_out = {
							name = L["Step Out"],
							desc = L["Big Step Out Desc"],
							type = "range",
							order	= 48,
							min		= -5,
							max		= 5,
							step	= 0.1,
							set = function(info,val) 
								self.db.profile.cooldownline.splash_big.step_out = val
								self.UpdateSettings()
							end,
							get = function(info)
								return	self.db.profile.cooldownline.splash_big.step_out
							end,
						},
						alpha_in = {
							name = L["Alpha In"],
							desc = L["Big Alpha In Desc"],
							type = "range",
							order	= 49,
							min		= -5,
							max		= 5,
							step	= 0.1,
							set = function(info,val) 
								self.db.profile.cooldownline.splash_big.alpha_in = val
								self.UpdateSettings()
							end,
							get = function(info)
								return	self.db.profile.cooldownline.splash_big.alpha_in
							end,
						},
						alpha_out = {
							name = L["Alpha Out"],
							desc = L["Big Alpha Out Desc"],
							type = "range",
							order	= 50,
							min		= -5,
							max		= 5,
							step	= 0.1,
							set = function(info,val) 
								self.db.profile.cooldownline.splash_big.alpha_out = val
								self.UpdateSettings()
							end,
							get = function(info)
								return	self.db.profile.cooldownline.splash_big.alpha_out
							end,
						},
						descSplashSmall = {
							type = "header",
							name = L["Small splash"],
							order = 51,
						},
						--[[
						slash_size = {
							name = L["Size"],
							type = "range",
							order	= 43,
							min		= 1,
							max		= 400,
							step	= 1,
							set = function(info,val) 
								self.db.profile.cooldownline.slash_size = val
								self.UpdateSettings()
							end,
							get = function(info)
								return	self.db.profile.cooldownline.slash_size
							end,
						},
						slash_alpha = {
							name = L["Alpha"],
							type = "range",
							order	= 44,
							min		= 0,
							max		= 1,
							step	= 0.1,
							set = function(info,val) 
								self.db.profile.cooldownline.slash_alpha = val
								self.UpdateSettings()
							end,
							get = function(info)
								return	self.db.profile.cooldownline.slash_alpha
							end,
						},
						]]
						slash_show1 = {
							order = 52,name = L["Show"],type = "toggle",
							set = function(info,val) self.db.profile.cooldownline.slash_show_small = not self.db.profile.cooldownline.slash_show_small; self.UpdateSettings() end,
							get = function(info) return self.db.profile.cooldownline.slash_show_small end
						},
						splashsmall_background_color = {
							order = 52.1,name = L["Background Color"],type = "color", hasAlpha = true,
							set = function(info,r,g,b,a) self.db.profile.cooldownline.splashsmall_background_color={r = r,g = g,b = b, a = a}; self.UpdateSettings()  end,
							get = function(info) return self.db.profile.cooldownline.splashsmall_background_color.r,self.db.profile.cooldownline.splashsmall_background_color.g,self.db.profile.cooldownline.splashsmall_background_color.b,self.db.profile.cooldownline.splashsmall_background_color.a end
						},
						slash_small_alpha = {
							name = L["Alpha"],
							type = "range",
							order	= 53,
							min		= 0,
							max		= 1,
							step	= 0.1,
							set = function(info,val) 
								self.db.profile.cooldownline.slash_small_alpha = val
								self.UpdateSettings()
							end,
							get = function(info)
								return	self.db.profile.cooldownline.slash_small_alpha
							end,
						},
						time_in1 = {
							name = L["Splash In"],
							desc = L["Small Splash In Desc"],
							type = "range",
							order	= 54,
							min		= 0,
							max		= 1,
							step	= 0.1,
							set = function(info,val) 
								self.db.profile.cooldownline.splash_small.time_in = val
								self.UpdateSettings()
							end,
							get = function(info)
								return	self.db.profile.cooldownline.splash_small.time_in
							end,
						},
						time_out1 = {
							name = L["Splash Out"],
							desc = L["Small Splash Out Desc"],
							type = "range",
							order	= 55,
							min		= 0,
							max		= 1,
							step	= 0.1,
							set = function(info,val) 
								self.db.profile.cooldownline.splash_small.time_out = val
								self.UpdateSettings()
							end,
							get = function(info)
								return	self.db.profile.cooldownline.splash_small.time_out
							end,
						},
						step_in1 = {
							name = L["Step In"],
							desc = L["Small Step In Desc"],
							type = "range",
							order	= 56,
							min		= -5,
							max		= 5,
							step	= 0.1,
							set = function(info,val) 
								self.db.profile.cooldownline.splash_small.step_in = val
								self.UpdateSettings()
							end,
							get = function(info)
								return	self.db.profile.cooldownline.splash_small.step_in
							end,
						},
						step_out1 = {
							name = L["Step Out"],
							desc = L["Small Step Out Desc"],
							type = "range",
							order	= 57,
							min		= -5,
							max		= 5,
							step	= 0.1,
							set = function(info,val) 
								self.db.profile.cooldownline.splash_small.step_out = val
								self.UpdateSettings()
							end,
							get = function(info)
								return	self.db.profile.cooldownline.splash_small.step_out
							end,
						},
						alpha_in1 = {
							name = L["Alpha In"],
							desc = L["Small Alpha In Desc"],
							type = "range",
							order	= 58,
							min		= -5,
							max		= 5,
							step	= 0.1,
							set = function(info,val) 
								self.db.profile.cooldownline.splash_small.alpha_in = val
								self.UpdateSettings()
							end,
							get = function(info)
								return	self.db.profile.cooldownline.splash_small.alpha_in
							end,
						},
						alpha_out1 = {
							name = L["Alpha Out"],
							desc = L["Small Alpha Out Desc"],
							type = "range",
							order	= 59,
							min		= -5,
							max		= 5,
							step	= 0.1,
							set = function(info,val) 
								self.db.profile.cooldownline.splash_small.alpha_out = val
								self.UpdateSettings()
							end,
							get = function(info)
								return	self.db.profile.cooldownline.splash_small.alpha_out
							end,
						},
						},
					},
				},
			},
			profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db),
		}
	}

	o.args.profile.args.EIheader = {
			type = "header",
			name = "Export & Import Profile",
			order = -101,
		}
		--[[
	o.args.profile.args.string_text =  {
				type = "input",	order = -100,
				name = "Data",
				width = "full",
				set = function(info,val) end,						
				get = function(info) end
			}
			]]
	o.args.profile.args.Export = {
			type = 'execute',
			order = -99,
			name = 'Export',
			func = function(info, value)
				self:ExportProfile()			
			end,
		}
		
	o.args.profile.args.Import = {
			type = 'execute',
			order = -98,
			name = 'Import',
			func = function(info, value)
				self:ImportProfile()	
			end,
		}
		
	return o
end

do
	local donation = { 
		"Je*** Hui*****(jfd**iz@xs4***.nl) @Curse.com",
	}

	local sites = {
		["curse_com"] = { "Curse.com", "http://www.curse.com/addons/wow/sptimers" },
		["h2p"]		  = { "HowToPriest.com", "http://howtopriest.com/viewtopic.php?f=26&t=5480" },
		["twitter"]	  = { "Twitter", "@Aleyka11" },
		["paypal"]	  = { "PayPal", "https://www.paypal.com/cgi-bin/webscr?hosted_button_id=D6SUMPTFUATY6&item_name=SPTimers+%28from+Curse.com%29&cmd=_s-xclick" , true},
	}
		
	function SPTimers:InitSupports()
		local order_1 = 1
		for k,v in pairs(sites) do
		
			o.args.about.args[k] =  {
				type = "input",	order	= order_1,multiline = v[3],
				name = v[1],
				width = "full",
				set = function(info,val) end,						
				get = function(info) return v[2] end
			}
			order_1 = order_1 + 1
		
		end
		
		local don_text = L["Donations:\n"]
		
		for k,v in ipairs(donation) do
			don_text = don_text..format("%d. %s\n", k, v)
		
		end
		
		o.args.about.args.donations  = {
				order = order_1,
				type = "description",
				name = don_text,
			}

	end
end

	function SPTimers:ClassSpell(spellID)
		
		local opts = self.db.profile.classSpells[self.myCLASS][spellID]
		class_select = spellID
		
		o.args.bars.args.ClassSpells.args.header1 = {
				name	= "",
				order	= 3.2,
				type = "header",
			}
		o.args.bars.args.ClassSpells.args.hide = {
				order = 3.3,name = L["Hide"],type = "toggle", widht = "full",
				set = function(info,val) opts.hide = not opts.hide; end,
				get = function(info) return opts.hide end
			}
			
		o.args.bars.args.ClassSpells.args.haste = {
				order = 3.4,name = L["Hasted"],type = "toggle",
				set = function(info,val) opts.haste = not opts.haste; end,
				get = function(info) return opts.haste end
			}
		if self.myCLASS == "WARLOCK" then
			o.args.bars.args.ClassSpells.args.pandemia = {
					order = 3.5,name = L["Pandemia"],type = "toggle",
					set = function(info,val) opts.pandemia = not opts.pandemia; end,
					get = function(info) return opts.pandemia end
				}
		end
		
		o.args.bars.args.ClassSpells.args.cleu = {
				order = 4,name = L["cleu"], desc = L["cleu_desc"], type = "toggle",
				set = function(info,val) opts.cleu = not opts.cleu; end,
				get = function(info) return opts.cleu end
			}
		o.args.bars.args.ClassSpells.args.channel = {
				order = 5,name = L["Channeling"],type = "toggle",
				set = function(info,val) opts.channel = not opts.channel; self:ScanForChannelingSpell() end,
				get = function(info) return opts.channel end
			}
		o.args.bars.args.ClassSpells.args.duration = {
				type = "input",	width = "half", order	= 6,
				name = L["Duration"],
				set = function(info,val) 
					local num = tonumber(val)				
					if num then opts.duration = num end
				end,
				get = function(info) return opts.duration and tostring(opts.duration) or NO end
			}
		o.args.bars.args.ClassSpells.args.pvpduration = {
				type = "input",	width = "half", order	= 6,
				name = L["PVP duration"],
				set = function(info,val) 
					local num = tonumber(val)				
					if num then opts.pvpduration = num end
				end,
				get = function(info) return opts.pvpduration and tostring(opts.pvpduration) or NO end
			}
		o.args.bars.args.ClassSpells.args.tick = {
				type = "input",width = "half", 	order	= 7,
				name = L["Ticks"],
				desc = L["TicksComm"],
				set = function(info,val) 
					local num = tonumber(val)				
					if num then opts.tick = num end
				end,
				get = function(info) return opts.tick and tostring(opts.tick) or "0" end
			}
		o.args.bars.args.ClassSpells.args.cast = {
				type = "input",width = "half", order	= 8,
				name = L["Cast time"],
				set = function(info,val) 
					local num = tonumber(val)				
					if num then opts.cast = num end
				end,
				get = function(info) return opts.cast and tostring(opts.cast) or "0" end
			}
			
		o.args.bars.args.ClassSpells.args.withgcd = {
				order = 8.1,name = L["Include Global Cooldown"],type = "toggle",
				set = function(info,val) opts.withgcd = not opts.withgcd; end,
				get = function(info) return opts.withgcd end
			}
		o.args.bars.args.ClassSpells.args.tickoverlap = {
				order = 8.2,name = L["Include Tick Overlap"],type = "toggle",
				set = function(info,val) opts.tickoverlap = not opts.tickoverlap; end,
				get = function(info) return opts.tickoverlap end
			}
			
		o.args.bars.args.ClassSpells.args.desc2 = {
				name	= L["Bar color"],
				order	= 9,
				type = "header",
			}
		o.args.bars.args.ClassSpells.args.color = {
				order = 10,name = L["Color"],type = "color",
				set = function(info,r,g,b) opts.color={r,g,b}; end,
				get = function(info) return ( opts.color and opts.color[1] or 0 ),( opts.color and opts.color[2] or 0 ),( opts.color and opts.color[3] or 0 ),1 end
			}
		
		o.args.bars.args.ClassSpells.args.color_on = {
				order = 10,name = L["Own color"],type = "toggle",
				set = function(info,val) opts.color_on = not opts.color_on; end,
				get = function(info) return opts.color_on end
			}
		o.args.bars.args.ClassSpells.args.desc3 = {
				name	= L["Custom Text"],
				order	= 11,
				type = "header",
			}
		o.args.bars.args.ClassSpells.args.custom_text = {
				type = "input",	order	= 12, width = "full",
				name = L["Custom Text"],
				desc = L["Custom Text Desc"],
				set = function(info,val) 
					local num = tostring(val)				
					if num then opts.custom_text = num end
				end,
				get = function(info) return opts.custom_text and tostring(opts.custom_text) or "" end
			}
		o.args.bars.args.ClassSpells.args.custom_text_on= {
				order = 13,name = L["Custom Text On"],type = "toggle", width = "full",
				set = function(info,val) opts.custom_text_on = not opts.custom_text_on; end,
				get = function(info) return opts.custom_text_on end
			}
			
		o.args.bars.args.ClassSpells.args.sortHeader = {
				name	= L["Sorting"],
				order	= 13.1,
				type = "header",
			}
			
		o.args.bars.args.ClassSpells.args.priority = {
				name = L["Priority"],
				type = "range",
				order	= 14,
				min		= -20,
				max		= 20,
				step	= 1,
				set = function(info,val) 
					opts.priority = val
				end,
				get = function(info)
					return opts.priority
				end,
			}
		o.args.bars.args.ClassSpells.args.anchor = {
				name = L["Select Anchor"],
				order = 15,
				desc = L["Select Anchor Desc"],
				type = "select",
				values = function()
					local t = {}							
					for k,v in ipairs(self.db.profile.bars_anchors) do						
						t[k] = v.name or k
					end							
					return t
				end,
				set = function(info,val)
					opts.set_anchor = val
				end,
				get = function(info, val) 
					return opts.set_anchor or 1
				end
			}
		o.args.bars.args.ClassSpells.args.offtargetanchor = {
				name = L["Select offtarget Anchor"],
				order = 15,
				desc = L["Select Anchor offtarget Desc"].." ->'"..L["General"].."'->'"..L["Swap bars when change target"].."'",
				type = "select",
				values = function()
					local t = {}							
					for k,v in ipairs(self.db.profile.bars_anchors) do						
						t[k] = v.name or k
					end							
					return t
				end,
				set = function(info,val)
					opts.offtarge = val
				end,
				get = function(info, val) 
					return opts.offtarge or 1
				end
			}
		o.args.bars.args.ClassSpells.args.group = {
				name = L["Select group"],
				order = 15,
				desc = L["Select group Desc"],
				type = "select",
				values = groupSorting,
				set = function(info,val)
					opts.group = val
				end,
				get = function(info, val) 
					return opts.group or "auto"
				end
			}
			
		o.args.bars.args.ClassSpells.args.filterHeader = {
				name	= L["Filters"],
				order	= 16,
				type = "header",
			}
		o.args.bars.args.ClassSpells.args.whitelist = {
				name = L["White List"],
				order = 17,
				desc = L["White List Desc"],
				type = "select",
				values = filters,
				set = function(info,val)
					opts.whitelist = val
				end,
				get = function(info, val) 
					return opts.whitelist or 1
				end
			}
		o.args.bars.args.ClassSpells.args.blacklist = {
				name = L["Black List"],
				order = 18,
				desc = L["Black List Desc"],
				type = "select",
				values = filters,
				set = function(info,val)
					opts.blacklist = val
				end,
				get = function(info, val) 
					return opts.blacklist or 1
				end
			}
		o.args.bars.args.ClassSpells.args.whitelist_cleu = {
				name = L["White List Combat Log"],
				order = 17,
				desc = L["White List Combat Log Desc"],
				type = "select",
				values = filters_cleu,
				set = function(info,val)
					opts.whitelist_cleu = val
				end,
				get = function(info, val) 
					return opts.whitelist_cleu or 1
				end
			}
		o.args.bars.args.ClassSpells.args.blacklist_cleu = {
				name = L["Black List Combat Log"],
				order = 18,
				desc = L["Black List Combat Log Desc"],
				type = "select",
				values = filters_cleu,
				set = function(info,val)
					opts.blacklist_cleu = val
				end,
				get = function(info, val) 
					return opts.blacklist_cleu or 1
				end
			}
		o.args.bars.args.ClassSpells.args.affilation = {
				name = L["Source"],
				order = 20,
				type = "select",
				values = affiliation,
				set = function(info,val)
					opts.source = val
				end,
				get = function(info, val) 
					return opts.source
				end
			}
		o.args.bars.args.ClassSpells.args.targetType = {
				name = L["Target Type"],
				order = 20,
				type = "select",
				values = targetType,
				set = function(info,val)
					opts.target = val
				end,
				get = function(info, val) 
					return opts.target
				end
			}
		--[[
		o.args.bars.args.ClassSpells.args.anchor_per_unit_desc = {
				name	= L["Anchor per Unit"],
				order	= 21,
				type = "header",
			}
		]]
		
		o.args.bars.args.ClassSpells.args.anchor_per_unit = {		
			type = "group",	order	= 22,
			guiInline = true,
			name	= L["Anchor per Unit"],
			args = {
					enabled = {
						order = 1,name = L["Enabled"],type = "toggle", width = "full", desc = "|cFFff5c5c\n>>>>EXPERIMENTAL!<<<<|r\n\n"..L["Anchor per Unit Desc"],
						set = function(info,val) opts.anchor_per_unit_enabled = not opts.anchor_per_unit_enabled; end,
						get = function(info) return opts.anchor_per_unit_enabled end
					},
					unit = {
						name = L["Unit"],
						order = 2,
					--	desc = L["Select Anchor offtarget Desc"].." ->'"..L["General"].."'->'"..L["Swap bars when change target"].."'",
						type = "select",
						values = {
						--	target = "target",
							focus  = "focus",
							boss1  = "boss1",
							boss2  = "boss2",
							boss3  = "boss3",
							boss4  = "boss4",
							boss5  = "boss5",
							arena1  = "arena1",
							arena2  = "arena2",
							arena3  = "arena3",
							arena4  = "arena4",
							arena5  = "arena5",
							
						},
						set = function(info,val)
							if not opts.anchor_per_unit then 
								opts.anchor_per_unit = {} 
								opts.anchor_per_unit[val] = 1
							end
							unit_per_achor_choose = val
						end,
						get = function(info, val)
							return unit_per_achor_choose
						end
					},
					anchor = {
						name = L["Select Anchor"],
						order = 3,
					--	desc = L["Select Anchor offtarget Desc"].." ->'"..L["General"].."'->'"..L["Swap bars when change target"].."'",
						type = "select",
						values = function()
							local t = {}
							t[0] = NO
							for k,v in ipairs(self.db.profile.bars_anchors) do						
								t[k] = v.name or k
							end				
							return t
						end,
						set = function(info,val)
							if unit_per_achor_choose then 
								opts.anchor_per_unit[unit_per_achor_choose] = val
							end
						end,
						get = function(info, val)
							if unit_per_achor_choose then						
								return opts.anchor_per_unit[unit_per_achor_choose]
							end
							return nil
						end					
					}
				}
			}
		
		o.args.bars.args.ClassSpells.args.SoundFile = {					
			type = "group",	order	= 22.1,
			guiInline = true,
			name	= L["Sound"],
			args = {						
				OnShow = {
					order = 1,type = 'select',name = L["On Show"],
					dialogControl = 'LSM30_Sound',
					values = SPTimers.SharedMedia:HashTable("sound"),
					set = function(info,value) opts.sound_onshow = value end,
					get = function(info) return opts.sound_onshow or "None"; end,
				},
				OnHide = {
					order = 1,type = 'select',name = L["On Hide"],
					dialogControl = 'LSM30_Sound',
					values = SPTimers.SharedMedia:HashTable("sound"),
					set = function(info,value) opts.sound_onhide = value end,
					get = function(info) return opts.sound_onhide or "None"; end,
				},
			}
		}
		
		o.args.bars.args.ClassSpells.args.delete = {
			type = 'execute',
			order = -1,
			name = L['Delete spell'],
			func = function(info, value)
				opts = nil
				class_select = nil
				wipe(o.args.bars.args.ClassSpells.args)
						
				o.args.bars.args.ClassSpells.args.Anchor = {
						name = L["Select Spell"],
						order = 2,
						desc = L["Select Spell"],
						width = "full",
						type = "select",
						values = function()
							local t = {}												
							for k,v in pairs(self.db.profile.classSpells[self.myCLASS]) do						
								t[k] = SpellString(k, 10)
							end									
							return t
						end,
						set = function(info,val)
							SPTimers:ClassSpell(val)
						end,
						get = function(info, val)
							return class_select
						end
					}
				o.args.bars.args.ClassSpells.args.AddNew = {
						type = "input",	order	= 1,
						name = L["Spell ID"],
						desc = L["Change spellID"],
						set = function(info,val)
							local num = tonumber(val)
							if num then
								local spellname = GetSpellInfo(num)										
								if spellname then										
									SPTimers:SearchDBSpell(num, "class")
								end	
							end
						end,
						get = function(info)end
					}
			end,
		}
	end


	function SPTimers:ProcsSpell(spellID)
		
		local opts = self.db.profile.procSpells[spellID]

		proc_select = spellID

		o.args.bars.args.spellList2.args.hide = {
				order = 3,name = L["Hide"],type = "toggle", widht = "full",
				set = function(info,val) opts.hide = not opts.hide; end,
				get = function(info) return opts.hide end
			}			
		o.args.bars.args.spellList2.args.typeHeader1 = {
				name	= L["Color"],
				order	= 4,
				type = "header",
			}
			
		o.args.bars.args.spellList2.args.color = {
				order = 5,name = L["Color"],type = "color",
				set = function(info,r,g,b) opts.color={r,g,b}; end,
				get = function(info) return ( opts.color and opts.color[1] or 0 ),( opts.color and opts.color[2] or 0 ),( opts.color and opts.color[3] or 0 ),1 end
			}
		
		o.args.bars.args.spellList2.args.color_on = {
				order = 6,name = L["Own color"],type = "toggle",
				set = function(info,val) opts.color_on = not opts.color_on; end,
				get = function(info) return opts.color_on end
			}
		o.args.bars.args.spellList2.args.typeHeader2 = {
				name	= L["Bar Text"],
				order	= 7,
				type = "header",
			}
		o.args.bars.args.spellList2.args.custom_text = {
				type = "input",	order	= 8,
				name = L["Custom Text"],
				desc = L["Custom Text Desc"],
				set = function(info,val) 
					local num = tostring(val)				
					if num then opts.custom_text = num end
				end,
				get = function(info) return opts.custom_text and tostring(opts.custom_text) or "" end
			}
		o.args.bars.args.spellList2.args.custom_text_on= {
				order = 9,name = L["Custom Text On"],type = "toggle",
				set = function(info,val) opts.custom_text_on = not opts.custom_text_on; end,
				get = function(info) return opts.custom_text_on end
			}		
		o.args.bars.args.spellList2.args.check_stacks = {
				type = "input",	order	= 10,
				name = L["Check Stacks"],
				desc = L["Check Stacks Desc"],
				set = function(info,val) 
					local num = tonumber(val)				
					if num then opts.checkstaucks = num end
				end,
				get = function(info) return opts.checkstaucks and tostring(opts.checkstaucks) or "" end
			}
		o.args.bars.args.spellList2.args.check_stacks_on= {
				order = 11,name = L["Check Stacks On"],type = "toggle",
				set = function(info,val) opts.checkstaucks_on = not opts.checkstaucks_on; end,
				get = function(info) return opts.checkstaucks_on end
			}
		o.args.bars.args.spellList2.args.sortHeader = {
				name	= L["Sorting"],
				order	= 12,
				type = "header",
			}
			
		o.args.bars.args.spellList2.args.priority = {
				name = L["Priority"],
				type = "range",
				order	= 13,
				min		= -20,
				max		= 20,
				step	= 1,
				set = function(info,val) 
					opts.priority = val
				end,
				get = function(info)
					return opts.priority
				end,
			}
		o.args.bars.args.spellList2.args.anchor = {
				name = L["Select Anchor"],
				order = 14,
				desc = L["Select Anchor Desc"],
				type = "select",
				values = function()
					local t = {}							
					for k,v in ipairs(self.db.profile.bars_anchors) do						
						t[k] = v.name or k
					end							
					return t
				end,
				set = function(info,val)
					opts.set_anchor = val
				end,
				get = function(info, val)
					return opts.set_anchor or 1
				end
			}
		o.args.bars.args.spellList2.args.group = {
				name = L["Select group"],
				order = 15,
				desc = L["Select group Desc"],
				type = "select",
				values = groupSorting,
				set = function(info,val)
					opts.group = val
				end,
				get = function(info, val) 
					return opts.group or "auto"
				end
			}
		o.args.bars.args.spellList2.args.filterHeader = {
				name	= L["Filters"],
				order	= 16,
				type = "header",
			}
		o.args.bars.args.spellList2.args.whitelist = {
				name = L["White List"],
				order = 17,
				type = "select",
				values = filters,
				set = function(info,val)
					opts.whitelist = val
				end,
				get = function(info, val) 
					return opts.whitelist or 1
				end
			}
		o.args.bars.args.spellList2.args.blacklist = {
				name = L["Black List"],
				order = 18,
				type = "select",
				values = filters,
				set = function(info,val)
					opts.blacklist = val
				end,
				get = function(info, val) 
					return opts.blacklist or 1
				end
			}
		o.args.bars.args.spellList2.args.affilation = {
				name = L["Source"],
				order = 19,
				type = "select",
				values = affiliation,
				set = function(info,val)
					opts.source = val
				end,
				get = function(info, val) 
					return opts.source
				end
			}
		o.args.bars.args.spellList2.args.targetType = {
				name = L["Target Type"],
				order = 20,
				type = "select",
				values = targetType,
				set = function(info,val)
					opts.target = val
				end,
				get = function(info, val) 
					return opts.target
				end
			}
		
		o.args.bars.args.spellList2.args.SoundFile = {					
			type = "group",	order	= 20.1,
			guiInline = true,
			name	= L["Sound"],
			args = {						
				OnShow = {
					order = 1,type = 'select',name = L["On Show"],
					dialogControl = 'LSM30_Sound',
					values = SPTimers.SharedMedia:HashTable("sound"),
					set = function(info,value) opts.sound_onshow = value end,
					get = function(info) return opts.sound_onshow or "None" end,
				},
				OnHide = {
					order = 1,type = 'select',name = L["On Hide"],
					dialogControl = 'LSM30_Sound',
					values = SPTimers.SharedMedia:HashTable("sound"),
					set = function(info,value) opts.sound_onhide = value end,
					get = function(info) return opts.sound_onhide or "None" end,
				},
			}
		}
	
		o.args.bars.args.spellList2.args.delete = {
			type = 'execute',
			order = 21,
			name = L['Delete spell'],
			func = function(info, value)
				opts = nil
				class_select = nil
				wipe(o.args.bars.args.spellList2.args)
						
				o.args.bars.args.spellList2.args.Anchor = {
						name = L["Select Spell"],
						order = 2,
						desc = L["Select Spell"],
						width = "full",
						type = "select",
						values = function()
							local t = {}												
							for k,v in pairs(self.db.profile.procSpells) do						
								t[k] = SpellString(k, 10)
							end									
							return t
						end,
						set = function(info,val)
							SPTimers:ClassSpell(val)
						end,
						get = function(info, val)
							return class_select
						end
					}
				o.args.bars.args.spellList2.args.AddNew = {
						type = "input",	order	= 1,
						name = L["Spell ID"],
						desc = L["Change spellID"],
						set = function(info,val)
							local num = tonumber(val)
							if num then
								local spellname = GetSpellInfo(num)										
								if spellname then										
									SPTimers:SearchDBSpell(num, "procs")
								end	
							end
						end,
						get = function(info)end
					}
			end,
		}
	end



	function SPTimers:OthersSpell(spellID)
		
		local opts = self.db.profile.othersSpells[spellID]
		
		others_select = spellID

		o.args.bars.args.spellList3.args.desc2 = {
				name	= L["Color"],
				order	= 9,
				type = "header",
			}
		o.args.bars.args.spellList3.args.hide = {
			order = 9.1,name = L["Hide"],type = "toggle", widht = "full",
			set = function(info,val) opts.hide = not opts.hide; end,
			get = function(info) return opts.hide end
		}
		
		o.args.bars.args.spellList3.args.color = {
				order = 10,width = "half",name = L["Color"],type = "color",
				set = function(info,r,g,b) opts.color={r,g,b}; end,
				get = function(info) return ( opts.color and opts.color[1] or 0 ),( opts.color and opts.color[2] or 0 ),( opts.color and opts.color[3] or 0 ),1 end
			}
		
		o.args.bars.args.spellList3.args.color_on = {
				order = 11,width = "half",name = L["Own color"],type = "toggle",
				set = function(info,val) opts.color_on = not opts.color_on; end,
				get = function(info) return opts.color_on end
			}
		o.args.bars.args.spellList3.args.desc3 = {
				name	= L["StatusBar configurations"],
				order	= 12,
				type = "header",
			}
		o.args.bars.args.spellList3.args.custom_text = {
				type = "input",	order	= 13,
				name = L["Custom Text"],
				desc = L["Custom Text Desc"],
				set = function(info,val) 
					local num = tostring(val)				
					if num then opts.custom_text = num end
				end,
				get = function(info) return opts.custom_text and tostring(opts.custom_text) or "" end
			}
		o.args.bars.args.spellList3.args.custom_text_on= {
				order = 14,name = L["Custom Text On"],type = "toggle",
				set = function(info,val) opts.custom_text_on = not opts.custom_text_on; end,
				get = function(info) return opts.custom_text_on end
			}
		o.args.bars.args.spellList3.args.desc4 = {
				name	= L["Sorting"],
				order	= 15,
				type = "header",
			}
			
		o.args.bars.args.spellList3.args.priority = {
				name = L["Priority"],
				type = "range",
				order	= 16,
				min		= -20,
				max		= 20,
				step	= 1,
				set = function(info,val) 
					opts.priority = val
				end,
				get = function(info)
					return opts.priority
				end,
			}
		o.args.bars.args.spellList3.args.anchor = {
				name = L["Select Anchor"],
				order = 17,
				desc = L["Select Anchor Desc"],
				type = "select",
				values = function()
					local t = {}							
					for k,v in ipairs(self.db.profile.bars_anchors) do						
						t[k] = v.name or k
					end							
					return t
				end,
				set = function(info,val)
					opts.set_anchor = val
				end,
				get = function(info, val) 
					return opts.set_anchor or 1
				end
			}
		o.args.bars.args.spellList3.args.group = {
				name = L["Select group"],
				order = 18,
				desc = L["Select group Desc"],
				type = "select",
				values = groupSorting,
				set = function(info,val)
					opts.group = val
				end,
				get = function(info, val) 
					return opts.group or "auto"
				end
			}
		o.args.bars.args.spellList3.args.filterHeader = {
				name	= L["Filters"],
				order	= 19,
				type = "header",
			}
		o.args.bars.args.spellList3.args.whitelist = {
				name = L["White List"],
				order = 20,
				desc = L["White List Desc"],
				type = "select",
				values = filters,
				set = function(info,val)
					opts.whitelist = val
				end,
				get = function(info, val) 
					return opts.whitelist or 1
				end
			}
		o.args.bars.args.spellList3.args.blacklist = {
				name = L["Black List"],
				order = 21,
				desc = L["Black List Desc"],
				type = "select",
				values = filters,
				set = function(info,val)
					opts.blacklist = val
				end,
				get = function(info, val) 
					return opts.blacklist or 1
				end
			}
		o.args.bars.args.spellList3.args.affilation = {
				name = L["Source"],
				order = 22,
				type = "select",
				values = affiliation,
				set = function(info,val)
					opts.source = val
				end,
				get = function(info, val) 
					return opts.source
				end
			}
		o.args.bars.args.spellList3.args.targetType = {
				name = L["Target Type"],
				order = 23,
				type = "select",
				values = targetType,
				set = function(info,val)
					opts.target = val
				end,
				get = function(info, val) 
					return opts.target
				end
			}
		
		o.args.bars.args.spellList3.args.SoundFile = {					
			type = "group",	order	= 23.1,
			guiInline = true,
			name	= L["Sound"],
			args = {						
				OnShow = {
					order = 1,type = 'select',name = L["On Show"],
					dialogControl = 'LSM30_Sound',
					values = SPTimers.SharedMedia:HashTable("sound"),
					set = function(info,value) opts.sound_onshow = value end,
					get = function(info) return opts.sound_onshow or "None" ; end,
				},
				OnHide = {
					order = 1,type = 'select',name = L["On Hide"],
					dialogControl = 'LSM30_Sound',
					values = SPTimers.SharedMedia:HashTable("sound"),
					set = function(info,value) opts.sound_onhide = value end,
					get = function(info) return opts.sound_onhide or "None"; end,
				},
			}
		}
		
		o.args.bars.args.spellList3.args.delete = {
			type = 'execute',
			order = 24,
			name = L['Delete spell'],
			func = function(info, value)
				opts = nil
				class_select = nil
				wipe(o.args.bars.args.spellList3.args)
						
				o.args.bars.args.spellList3.args.Anchor = {
						name = L["Select Spell"],
						order = 2,
						desc = L["Select Spell"],
						width = "full",
						type = "select",
						values = function()
							local t = {}												
							for k,v in pairs(self.db.profile.othersSpells) do						
								t[k] = SpellString(k, 10)
							end									
							return t
						end,
						set = function(info,val)
							SPTimers:ClassSpell(val)
						end,
						get = function(info, val)
							return class_select
						end
					}
				o.args.bars.args.spellList3.args.AddNew = {
						type = "input",	order	= 1,
						name = L["Spell ID"],
						desc = L["Change spellID"],
						set = function(info,val)
							local num = tonumber(val)
							if num then
								local spellname = GetSpellInfo(num)										
								if spellname then										
									SPTimers:SearchDBSpell(num, "others")
								end	
							end
						end,
						get = function(info)end
					}
			end,
		}
	end


local function get_sort_value(name, value)
	local t = SPTimers.db.profile.bars_anchors[anchor_value].sorting
	
	for k,v in ipairs(t) do	
		if v.name == name then 
			return v[value]
		end
	end
	
end

local function set_sort_value(name, value, arg)
	local t = SPTimers.db.profile.bars_anchors[anchor_value].sorting
	
	for k,v in ipairs(t) do	
		if v.name == name then 
			v[value] = arg
		end
	end
end

local function get_sort_order(val)
	local t = SPTimers.db.profile.bars_anchors[anchor_value].sorting
	
	for k,v in ipairs(t) do	
		if v.name == val then 
			return k
		end
	end
end

local function update_sort_order(name, val)
	local t = SPTimers.db.profile.bars_anchors[anchor_value].sorting	
	local old_val
	local old_tbl
	
	local from, to
	
	
	for i, tb in ipairs(t) do
		if name == tb.name then
			old_val = tb.sort			
			from = tb
		end
		
		if val == tb.sort then
			to = tb
		end
	end
	
	from.sort = val
	to.sort = old_val
	
	table.sort(t, function(x,y)
		return x.sort < y.sort 
	end)
end

function SPTimers:SetAnchorTable(value)
	anchor_value = value

	o.args.bars.args.style.args.desc1 = {
			type = "header",
			name = L["StatusBar configurations"],
			order = 2,
		}
	o.args.bars.args.style.args.Anchor_copy = {
			name = L["Copy settings from"],
			order = 2.1,
			desc = L["Select Anchor"],
			type = "select",
			values = function()
				local t = {}
									
				for k,v in ipairs(self.db.profile.bars_anchors) do
					if k ~= anchor_value then 
						t[k] = v.name or k
					end
				end
									
				return t
			end,
			set = function(info,val)
				if anchor_value and anchor_value ~= val then
					SPTimers:CopySettings(val, anchor_value)
				end
			end,
			get = function(info, val) end
		}
		
	o.args.bars.args.style.args.Anchor_name = {
			type = "input", order	= 2.11,
			name = "Name",
			set = function(info,val) 
				
				if val and val ~= "" then self.db.profile.bars_anchors[anchor_value].name = val end
			end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].name end
		}
		
	o.args.bars.args.style.args.left_icon = {
			order = 3,name = L["Show left icon"],type = "toggle",
			set = function(info,val) self.db.profile.bars_anchors[anchor_value].left_icon = not self.db.profile.bars_anchors[anchor_value].left_icon; self:Visibility()	end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].left_icon end
		}
	o.args.bars.args.style.args.right_icon = {
			order = 4,name = L["Show right icon"],type = "toggle",
			set = function(info,val) self.db.profile.bars_anchors[anchor_value].right_icon = not self.db.profile.bars_anchors[anchor_value].right_icon; self:Visibility() end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].right_icon end
		}	
	o.args.bars.args.style.args.add_up = {
			order = 5,name = L["Grow up"],type = "toggle",
			set = function(info,val) self.db.profile.bars_anchors[anchor_value].add_up = not self.db.profile.bars_anchors[anchor_value].add_up; self:updateSortings(); self:UpdateMoverPosition() end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].add_up end
		}
	o.args.bars.args.style.args.target_name = {
			order = 6,name = L["Show target name"],type = "toggle",
			set = function(info,val) self.db.profile.bars_anchors[anchor_value].target_name = not self.db.profile.bars_anchors[anchor_value].target_name; self:UpdateFormatTexts(anchor_value) end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].target_name end
		}
	o.args.bars.args.style.args.short_name = {
			order = 7,name = L["Short bar text"],type = "toggle",
			set = function(info,val) self.db.profile.bars_anchors[anchor_value].short = not self.db.profile.bars_anchors[anchor_value].short; self:UpdateFormatTexts(anchor_value) end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].short end
		}
	o.args.bars.args.style.args.debug_info = {
			order = 8,name = L["Show debug info"],type = "toggle",
			set = function(info,val) self.db.profile.bars_anchors[anchor_value].debug_info = not self.db.profile.bars_anchors[anchor_value].debug_info; self:UpdateFormatTexts(anchor_value) end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].debug_info end
		}
	o.args.bars.args.style.args.timer_left = {
			order = 9,name = L["Timer text on left"],type = "toggle",
			set = function(info,val) self.db.profile.bars_anchors[anchor_value].lefttext = not self.db.profile.bars_anchors[anchor_value].lefttext; self:Update_TimeText(); self:Update_SpellText()end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].lefttext end
		}
	
	o.args.bars.args.style.args.reverse_fill = {
			order = 9.1,name = L["Revers bar filling"],type = "toggle",
			set = function(info,val) self.db.profile.bars_anchors[anchor_value].reverse_fill = not self.db.profile.bars_anchors[anchor_value].reverse_fill; self:Visibility() end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].reverse_fill end
		}	
		
	o.args.bars.args.style.args.format_s = {
			name = L["Time format"],
			order = 10,
			desc = L["Select timer format Desc"],
			type = "select",						
			values = bar_timer_format,
			set = function(info,val)
				self.db.profile.bars_anchors[anchor_value].fortam_s = val
			end,
			get = function(info, val) 
				return self.db.profile.bars_anchors[anchor_value].fortam_s
			end
			}				
	o.args.bars.args.style.args.bar_number = {
			name = L["Bar Number"],
			desc = L["Set Max bars number"],
			type = "range",
			order	= 11,
			min		= 1,
			max		= 20,
			step	= 1,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].bar_number = val
				self:InitFrames()
			end,
			get =function(info)
				return self.db.profile.bars_anchors[anchor_value].bar_number
			end,
		}
	o.args.bars.args.style.args.bg = {
			order = 12,type = 'select',name = L["Main Texture"],
			dialogControl = 'LSM30_Statusbar',
			values = SPTimers.SharedMedia:HashTable("statusbar"),
			set = function(info,value) self.db.profile.bars_anchors[anchor_value].bar.texture = value; self:Visibility() end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].bar.texture end,
		}
	o.args.bars.args.style.args.bg_bg = {
			order = 13,type = 'select',name = L["Background Texture"], 
			dialogControl = 'LSM30_Statusbar',
			values = SPTimers.SharedMedia:HashTable("statusbar"),
			set = function(info,value) self.db.profile.bars_anchors[anchor_value].bar.bgtexture = value; self:Visibility() end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].bar.bgtexture end,
		}
	o.args.bars.args.style.args.bg_color = {
			order = 14,name = L["Texture Color"],type = "color",
			set = function(info,r,g,b) self.db.profile.bars_anchors[anchor_value].bar.color={r,g,b}; self:Visibility()  end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].bar.color[1],self.db.profile.bars_anchors[anchor_value].bar.color[2],self.db.profile.bars_anchors[anchor_value].bar.color[3],1 end
		}
	o.args.bars.args.style.args.bg_color2 = {
			order = 15,name = L["Background Color"],type = "color", hasAlpha = true,
			set = function(info,r,g,b,a) self.db.profile.bars_anchors[anchor_value].bar.bgcolor={r,g,b,a}; self:Visibility()  end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].bar.bgcolor[1],self.db.profile.bars_anchors[anchor_value].bar.bgcolor[2],self.db.profile.bars_anchors[anchor_value].bar.bgcolor[3],self.db.profile.bars_anchors[anchor_value].bar.bgcolor[4]  end
		}
	o.args.bars.args.style.args.w = {
			name = L["Width"],
			type = "range",
			order	= 16,
			min		= 1,
			max		= 1920, --400,
			step	= 1,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].w = val
				--self:InitFrames()
				
				self:UpdateBarsSize()
			end,
			get =function(info)
				return self.db.profile.bars_anchors[anchor_value].w
			end,
		}
	o.args.bars.args.style.args.h = {
			name = L["Height"],
			type = "range",
			order	= 17,
			min		= 1,
			max		= 200,
			step	= 1,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].h = val
				--self:InitFrames()
				
				self:UpdateBarsSize()
			end,
			get =function(info)
				return self.db.profile.bars_anchors[anchor_value].h
			end,
		}
	
	o.args.bars.args.style.args.gap = {
		name = L["Gap"],
		type = "range",
		order	= 18,
		min		= 0,
		max		= 32,
		step	= 1,
		set = function(info,val) 
			self.db.profile.bars_anchors[anchor_value].gap = val
			self:updateSortings()
		end,
		get =function(info)
			return self.db.profile.bars_anchors[anchor_value].gap
			end,
		}
		
	o.args.bars.args.style.args.icon_gap = {
		name = L["Icon Gap"],
		type = "range",
		order	= 18,
		min		= -32,
		max		= 32,
		step	= 1,
		set = function(info,val) 
			self.db.profile.bars_anchors[anchor_value].icon_gap = val
			self:UpdateAllBorder()
		end,
		get =function(info)
			return self.db.profile.bars_anchors[anchor_value].icon_gap
		end,
		}
	------------------------------------------------------------
	o.args.bars.args.style.args.deskBorder = {
			type = "header",
			name = L["Border configuration"],
			order = 19,
		}
	o.args.bars.args.style.args.border = {
		order = 20,type = 'select',name = L["Border Texture"],
		dialogControl = 'LSM30_Border',
		values = SPTimers.SharedMedia:HashTable("border"),
		set = function(info,value) self.db.profile.bars_anchors[anchor_value].border = value; self:UpdateAllBorder() end,
		get = function(info) return self.db.profile.bars_anchors[anchor_value].border end,
	}
					
	o.args.bars.args.style.args.bordercolor = {
		order = 21,name = L["Border Color"],type = "color", hasAlpha = true,
		set = function(info,r,g,b,a) self.db.profile.bars_anchors[anchor_value].bordercolor={r,g,b,a}; self:UpdateAllBorder()  end,
		get = function(info) return self.db.profile.bars_anchors[anchor_value].bordercolor[1],self.db.profile.bars_anchors[anchor_value].bordercolor[2],self.db.profile.bars_anchors[anchor_value].bordercolor[3],self.db.profile.bars_anchors[anchor_value].bordercolor[4] end
	}
					
	o.args.bars.args.style.args.bordersize = {
		name = L["Border Size"],
		desc = L["Set Border Size"],
		type = "range",
		order	= 22,
		min		= 0,
		max		= 32,
		step	= 1,
		set = function(info,val) 
			self.db.profile.bars_anchors[anchor_value].bordersize = val
			self:UpdateAllBorder()
		end,
		get =function(info)
			return self.db.profile.bars_anchors[anchor_value].bordersize
		end,
	}
	o.args.bars.args.style.args.borderinset = {
		name = L["Border Inset"],
		desc = L["Set Border Inset"],
		type = "range",
		order	= 23,
		min		= 0,
		max		= 32,
		step	= 1,
		set = function(info,val) 
			self.db.profile.bars_anchors[anchor_value].borderinset = val
			self:UpdateAllBorder()
		end,
		get =function(info)
			return self.db.profile.bars_anchors[anchor_value].borderinset
			end,
		}
	---------------------------------------------------------------------
	--[[
	o.args.bars.args.style.args.desc_spark = {
			type = "header",
			name = "Spark Config",
			order = 23.1,
		}
	o.args.bars.args.style.args.spark_color = {
			order = 23.2,name = "Color",type = "color",
			set = function(info,r,g,b) self.db.profile.bars_anchors[anchor_value].castspark.color={r,g,b,1}; self:UpdateAllSparks()  end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].castspark.color[1],self.db.profile.bars_anchors[anchor_value].castspark.color[2],self.db.profile.bars_anchors[anchor_value].castspark.color[3],1 end
		}
	o.args.bars.args.style.args.spark_width = {
			name = "Width",
			type = "range",
			order	= 23.3,
			min		= 0,
			max		= 100,
			step	= 0.1,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].castspark.w = val
				self:UpdateAllSparks()
			end,
			get = function(info)
				return	self.db.profile.bars_anchors[anchor_value].castspark.w
			end,
		}
	o.args.bars.args.style.args.spark_height = {
			name = "Height",
			type = "range",
			order	= 23.4,
			min		= 0,
			max		= 100,
			step	= 0.1,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].castspark.h = val
				self:UpdateAllSparks()
			end,
			get = function(info)
				return	self.db.profile.bars_anchors[anchor_value].castspark.h
			end,
		}
	o.args.bars.args.style.args.spark_alpha = {
			name = "Alpha",
			type = "range",
			order	= 23.5,
			min		= 0,
			max		= 1,
			step	= 0.1,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].castspark.alpha = val
				self:UpdateAllSparks()
			end,
			get = function(info)
				return	self.db.profile.bars_anchors[anchor_value].castspark.alpha
			end,
		}
		
	o.args.bars.args.style.args.desc_ticks = {
			type = "header",
			name = "Ticks Config",
			order = 23.6,
		}
	o.args.bars.args.style.args.ticks_alpha = {
			name = "Alpha",
			type = "range",
			order	= 23.8,
			min		= 0,
			max		= 1,
			step	= 0.1,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].dotticks.alpha = val
				self:UpdateAllTiks()
			end,
			get = function(info)
				return	self.db.profile.bars_anchors[anchor_value].dotticks.alpha
			end,
		}
	o.args.bars.args.style.args.ticks_color = {
			order = 23.7,name = "Color",type = "color",
			set = function(info,r,g,b) self.db.profile.bars_anchors[anchor_value].dotticks.color={r,g,b,1}; self:UpdateAllTiks()  end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].dotticks.color[1],self.db.profile.bars_anchors[anchor_value].dotticks.color[2],self.db.profile.bars_anchors[anchor_value].dotticks.color[3],1 end
		}
		]]
	-------------------------------------------------------------------	
	o.args.bars.args.style.args.desc2 = {
			type = "header",
			name = L["Stack Text configuration"],
			order = 24,
		}
	o.args.bars.args.style.args.stack_textcolor = {
			order = 25,name = L["Text color"],type = "color",
			set = function(info,r,g,b) self.db.profile.bars_anchors[anchor_value].stack.textcolor={r,g,b}; self:Update_StackText()  end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].stack.textcolor[1],self.db.profile.bars_anchors[anchor_value].stack.textcolor[2],self.db.profile.bars_anchors[anchor_value].stack.textcolor[3],1 end
		}
	o.args.bars.args.style.args.stack_font = {
			order = 26,name = L["Font"],type = 'select',
			dialogControl = 'LSM30_Font',
			values = SPTimers.SharedMedia:HashTable("font"),
			set = function(info,key) self.db.profile.bars_anchors[anchor_value].stack.font = key; self:Update_StackText() end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].stack.font end,
		}
	o.args.bars.args.style.args.stack_font_size = {
			name = L["Size"],
			type = "range",
			order	= 27,
			min		= 1,
			max		= 32,
			step	= 1,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].stack.size = val
				self:Update_StackText()
			end,
			get = function(info)
				return	self.db.profile.bars_anchors[anchor_value].stack.size
			end,
		}
	o.args.bars.args.style.args.stack_justifu = {
			type = "select",	order = 28,
			name = L["Justify"],
			values = justifu,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].stack.justify = val
				self:Update_StackText()
			end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].stack.justify end
		}			
	o.args.bars.args.style.args.stack_text_flaggs = {
			type = "select",	order = 29,
			name = L["Flags"],
			values = text_flaggs,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].stack.flags = val
				self:Update_StackText()
			end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].stack.flags end
		}
	o.args.bars.args.style.args.stack_font_alpha = {
			name = L["Alpha"],
			type = "range",
			order	= 29.1,
			min		= 0,
			max		= 1,
			step	= 0.1,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].stack.alpha = val
				self:Update_StackText()
			end,
			get = function(info)
				return	self.db.profile.bars_anchors[anchor_value].stack.alpha
			end,
		}
		
	o.args.bars.args.style.args.stack_shadow_color = {
			order = 29.2,name = "Shadow color",type = "color", hasAlpha = true,
			set = function(info,r,g,b,a) 
				
				self.db.profile.bars_anchors[anchor_value].stack.shadow={r,g,b,a}; 
				self:Update_StackText()  
			end,
			get = function(info)
				local color = self.db.profile.bars_anchors[anchor_value].stack.shadow or { 0, 0, 0, 1}			
				return color[1],color[2],color[3],color[4] 
			end
		}
		
	o.args.bars.args.style.args.stack_shadow_offset_x = {
			name = "Shadow Offset X",
			type = "range",
			order	= 29.3,
			min		= -10,
			max		= 10,
			step	= 0.1,
			set = function(info,val)
				if not self.db.profile.bars_anchors[anchor_value].stack.offset then self.db.profile.bars_anchors[anchor_value].stack.offset = {} end
				self.db.profile.bars_anchors[anchor_value].stack.offset[1] = val
				self:Update_StackText()
			end,
			get = function(info)
				return	self.db.profile.bars_anchors[anchor_value].stack.offset and self.db.profile.bars_anchors[anchor_value].stack.offset[1] or 0
			end,
		}
		
	o.args.bars.args.style.args.stack_shadow_offset_y = {
			name = "Shadow Offset Y",
			type = "range",
			order	= 29.4,
			min		= -10,
			max		= 10,
			step	= 0.1,
			set = function(info,val)
				if not self.db.profile.bars_anchors[anchor_value].stack.offset then self.db.profile.bars_anchors[anchor_value].stack.offset = {} end
				self.db.profile.bars_anchors[anchor_value].stack.offset[2] = val
				self:Update_StackText()
			end,
			get = function(info)
				return	self.db.profile.bars_anchors[anchor_value].stack.offset and self.db.profile.bars_anchors[anchor_value].stack.offset[2] or 0
			end,
		}
		
	o.args.bars.args.style.args.desc3 = {
			type = "header",
			name = L["Timer Text configuration"],
			order = 30,
		}	
	o.args.bars.args.style.args.timer_textcolor = {
			order = 31,name = L["Text color"],type = "color",
			set = function(info,r,g,b) self.db.profile.bars_anchors[anchor_value].timer.textcolor={r,g,b}; self:Update_TimeText() end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].timer.textcolor[1],self.db.profile.bars_anchors[anchor_value].timer.textcolor[2],self.db.profile.bars_anchors[anchor_value].timer.textcolor[3],1 end
		}
	o.args.bars.args.style.args.timer_font = {
			order = 32,name = L["Font"],type = 'select',
			dialogControl = 'LSM30_Font',
			values = SPTimers.SharedMedia:HashTable("font"),
			set = function(info,key) self.db.profile.bars_anchors[anchor_value].timer.font = key; self:Update_TimeText() end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].timer.font end,
		}
	o.args.bars.args.style.args.timer_font_size = {
			name = L["Size"],
			type = "range",
			order	= 33,
			min		= 1,
			max		= 32,
			step	= 1,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].timer.size = val
				self:Update_TimeText()
			end,
			get = function(info)
				return	self.db.profile.bars_anchors[anchor_value].timer.size
			end,
		}
	o.args.bars.args.style.args.timer_justifu = {
			type = "select",	order = 34,
			name = L["Justify"],
			values = justifu,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].timer.justify = val
				self:Update_TimeText()
			end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].timer.justify end
		}			
	o.args.bars.args.style.args.timer_text_flaggs = {
			type = "select",	order = 35,
			name = L["Flags"],
			values = text_flaggs,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].timer.flags = val
				self:Update_TimeText()
			end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].timer.flags end
		}
	o.args.bars.args.style.args.timer_font_alpha = {
			name = L["Alpha"],
			type = "range",
			order	= 35.1,
			min		= 0,
			max		= 1,
			step	= 0.1,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].timer.alpha = val
				self:Update_TimeText()
			end,
			get = function(info)
				return	self.db.profile.bars_anchors[anchor_value].timer.alpha
			end,
		}
		
	o.args.bars.args.style.args.timer_shadow_color = {
			order = 35.2,name = "Shadow color",type = "color", hasAlpha = true,
			set = function(info,r,g,b,a) self.db.profile.bars_anchors[anchor_value].timer.shadow={r,g,b,a}; self:Update_TimeText()  end,
			get = function(info) 
				local color = self.db.profile.bars_anchors[anchor_value].timer.shadow or { 0, 0, 0, 1}			
				return color[1],color[2],color[3],color[4]
			end
		}
		
	o.args.bars.args.style.args.timer_shadow_offset_x = {
			name = "Shadow Offset X",
			type = "range",
			order	= 35.3,
			min		= -10,
			max		= 10,
			step	= 0.1,
			set = function(info,val)
				if not self.db.profile.bars_anchors[anchor_value].timer.offset then self.db.profile.bars_anchors[anchor_value].timer.offset = {} end
				self.db.profile.bars_anchors[anchor_value].timer.offset[1] = val
				self:Update_TimeText()
			end,
			get = function(info)
				return self.db.profile.bars_anchors[anchor_value].timer.offset and self.db.profile.bars_anchors[anchor_value].timer.offset[1] or 0
			end,
		}
		
	o.args.bars.args.style.args.timer_shadow_offset_y = {
			name = "Shadow Offset Y",
			type = "range",
			order	= 35.4,
			min		= -10,
			max		= 10,
			step	= 0.1,
			set = function(info,val)
				if not self.db.profile.bars_anchors[anchor_value].timer.offset then self.db.profile.bars_anchors[anchor_value].timer.offset = {} end
				self.db.profile.bars_anchors[anchor_value].timer.offset[2] = val
				self:Update_TimeText()
			end,
			get = function(info)
				return self.db.profile.bars_anchors[anchor_value].timer.offset and self.db.profile.bars_anchors[anchor_value].timer.offset[2] or 0
			end,
		}
		
	o.args.bars.args.style.args.desc4 = {
			type = "header",
			name = L["Spell Text configuration."],
			order = 36,
		}	
	o.args.bars.args.style.args.spell_textcolor = {
			order = 37,name = L["Text color"],type = "color",
			set = function(info,r,g,b) self.db.profile.bars_anchors[anchor_value].spell.textcolor={r,g,b}; self:Update_SpellText()  end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].spell.textcolor[1],self.db.profile.bars_anchors[anchor_value].spell.textcolor[2],self.db.profile.bars_anchors[anchor_value].spell.textcolor[3],1 end
		}
	o.args.bars.args.style.args.spell_font = {
			order = 38,name = L["Font"],type = 'select',
			dialogControl = 'LSM30_Font',
			values = SPTimers.SharedMedia:HashTable("font"), --SPTimers.SharedMedia:HashTable("font"),
			set = function(info,key) 
				self.db.profile.bars_anchors[anchor_value].spell.font = key
				self:Update_SpellText() 
			end,
			get = function(info) 
				--print("spell_text_flaggs", self.db.profile.bars_anchors[anchor_value].spell.font, anchor_value)
				return self.db.profile.bars_anchors[anchor_value].spell.font 
			end,
		}
	o.args.bars.args.style.args.spell_font_size = {
			name = L["Size"],
			type = "range",
			order	= 39,
			min		= 1,
			max		= 32,
			step	= 1,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].spell.size = val
				self:Update_SpellText()
			end,
			get = function(info)
				return	self.db.profile.bars_anchors[anchor_value].spell.size
			end,
		}
	o.args.bars.args.style.args.spell_justifu = {
			type = "select",	order = 40,
			name = L["Justify"],
			values = justifu,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].spell.justify = val
				self:Update_SpellText()
			end,
			get = function(info) return self.db.profile.bars_anchors[anchor_value].spell.justify end
		}		
	o.args.bars.args.style.args.spell_text_flaggs = {
			type = "select",	order = 41,
			name = L["Flags"],
			values = text_flaggs,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].spell.flags = val
				self:Update_SpellText()
			end,
			get = function(info)
				return self.db.profile.bars_anchors[anchor_value].spell.flags 
			end
		}
	o.args.bars.args.style.args.spell_font_alpha = {
			name = L["Alpha"],
			type = "range",
			order	= 41.1,
			min		= 0,
			max		= 1,
			step	= 0.1,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].spell.alpha = val
				self:Update_SpellText()
			end,
			get = function(info)
				return	self.db.profile.bars_anchors[anchor_value].spell.alpha
			end,
		}
	o.args.bars.args.style.args.spell_text_offset = {
			name = L["Offset X"],
			type = "range",
			order	= 41.2,
			min		= -300,
			max		= 300,
			step	= 1,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].spell.offsetx = val
				self:Update_SpellText()
			end,
			get = function(info)
				return	self.db.profile.bars_anchors[anchor_value].spell.offsetx
			end,
		}
		
	o.args.bars.args.style.args.spell_shadow_color = {
			order = 41.3,name = "Shadow color",type = "color", hasAlpha = true,
			set = function(info,r,g,b,a) self.db.profile.bars_anchors[anchor_value].spell.shadow={r,g,b,a}; self:Update_SpellText()  end,
			get = function(info) 
				local color = self.db.profile.bars_anchors[anchor_value].spell.shadow or { 0, 0, 0, 1}			
				return color[1],color[2],color[3],color[4]
			end
		}

	o.args.bars.args.style.args.spell_shadow_offset_x = {
			name = "Shadow Offset X",
			type = "range",
			order	= 41.4,
			min		= -10,
			max		= 10,
			step	= 0.1,
			set = function(info,val)
				if not self.db.profile.bars_anchors[anchor_value].spell.offset then self.db.profile.bars_anchors[anchor_value].spell.offset = {} end
				self.db.profile.bars_anchors[anchor_value].spell.offset[1] = val
				self:Update_SpellText()
			end,
			get = function(info)
				return self.db.profile.bars_anchors[anchor_value].spell.offset and self.db.profile.bars_anchors[anchor_value].spell.offset[1] or 0
			end,
		}
		
	o.args.bars.args.style.args.spell_shadow_offset_y = {
			name = "Shadow Offset Y",
			type = "range",
			order	= 41.5,
			min		= -10,
			max		= 10,
			step	= 0.1,
			set = function(info,val)
				if not self.db.profile.bars_anchors[anchor_value].spell.offset then self.db.profile.bars_anchors[anchor_value].spell.offset = {} end
				self.db.profile.bars_anchors[anchor_value].spell.offset[2] = val
				self:Update_SpellText()
			end,
			get = function(info)
				return	self.db.profile.bars_anchors[anchor_value].spell.offset and self.db.profile.bars_anchors[anchor_value].spell.offset[2] or 0
			end,
		}
		
	o.args.bars.args.style.args.descRaidIcon = {
			type = "header",
			name = L["Raid Mark configuration"],
			order = 41.6,
		}
	o.args.bars.args.style.args.raidiconsize = {
			name = L["Size"],
			type = "range",
			order	= 41.7,
			min		= 1,
			max		= 32,
			step	= 1,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].raidiconsize = val
				self:UpdateRaidIcons()
			end,
			get = function(info)
				return	self.db.profile.bars_anchors[anchor_value].raidiconsize or 10
			end,
		}
	
	o.args.bars.args.style.args.raidicon_x = {
			name = L["Offset X"],
			type = "range",
			order	= 41.8,
			min		= -1000,
			max		= 1000,
			step	= 1,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].raidicon_x = val
				self:UpdateRaidIcons()
			end,
			get = function(info)
				return	self.db.profile.bars_anchors[anchor_value].raidicon_x or 0
			end,
		}
	
	o.args.bars.args.style.args.raidicon_y = {
			name = L["Offset Y"],
			type = "range",
			order	= 41.9,
			min		= -50,
			max		= 50,
			step	= 1,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].raidicon_y = val
				self:UpdateRaidIcons()
			end,
			get = function(info)
				return	self.db.profile.bars_anchors[anchor_value].raidicon_y or 5
			end,
		}
		
	o.args.bars.args.style.args.raidicon_alpha = {
			name = L["Alpha"],
			type = "range",
			order	= 41.10,
			min		= 0,
			max		= 1,
			step	= 0.1,
			set = function(info,val) 
				self.db.profile.bars_anchors[anchor_value].raidicon_alpha = val
				self:UpdateRaidIcons()
			end,
			get = function(info)
				return	self.db.profile.bars_anchors[anchor_value].raidicon_alpha or 1
			end,
		}
		
	o.args.bars.args.style.args.sortingdest = {
			type = "header",
			name = L["Sorting group configuration"],
			order = 42,
		}
		
		
	o.args.bars.args.style.args.sort_func = {
				name = L["Sorting"],
				order = 42.1,
				type = "select",						
				values = sorting_timer_table,
				set = function(info,val)
					self.db.profile.bars_anchors[anchor_value].sort_func = val; self:UpdateSortingFunction()
				end,
				get = function(info, val) 
					return self.db.profile.bars_anchors[anchor_value].sort_func or 1
				end
			}
		
	for k,v in ipairs({"target", "player", "procs", "offtargets"}) do
		
		o.args.bars.args.style.args["sorting_group_"..v] = {
		
						type = "group",	order	= 43+k*2,
						guiInline = true,
						name	= v,
						args = {
		
						}
		}
		
		
		o.args.bars.args.style.args["sorting_group_"..v].args.gap = {
			name = L["Gap"],
			type = "range",
			order	= 1,
			min		= 0,
			max		= 50,
			step	= 1,
			set = function(info,val) 
			--	self.db.profile.bars_anchors[anchor_value].sorting[i].gap = val
				
				set_sort_value(v, "gap", val)
			end,
			get = function(info)
				return get_sort_value(v, "gap")-- self.db.profile.bars_anchors[anchor_value].sorting[i].gap
			end,
		}
		
		o.args.bars.args.style.args["sorting_group_"..v].args.alpha = {
			name = L["Alpha"],
			type = "range",
			order	= 2,
			min		= 0,
			max		= 1,
			step	= 0.1,
			set = function(info,val) 
			--	self.db.profile.bars_anchors[anchor_value].sorting[i].alpha = val
				set_sort_value(v, "alpha", val)
			end,
			get = function(info)
				return get_sort_value(v, "alpha") -- self.db.profile.bars_anchors[anchor_value].sorting[i].alpha
			end,
		}
		
		if v ~= "offtargets" then
			o.args.bars.args.style.args["sorting_group_"..v].args.position = {
				name = L["Position"],
				order = 3,
				type = "select",						
				values = { "1", "2", "3" },
				set = function(info,val)
					update_sort_order(v, val)
				end,
				get = function(info, val) 
					return get_sort_value(v, "sort")
				end
			}						
		end
	end
	--[[
	sorting = {
							{name = "target", 		gap = 10, alpha = 1, },
							{name = "player", 		gap = 10, alpha = 1, },
							{name = "procs",		gap = 15, alpha = .7 },
							{name = "offtargets",	gap = 6, alpha = .7,},
						},
						
		]]

	if #self.db.profile.bars_anchors > 1 then
		o.args.bars.args.style.args.delete = {
				type = 'execute',
				--width = "half",
				order = -1,
				name = L['Remove Anchor'],
				func = function(info, value)
					
					if #self.db.profile.bars_anchors > 1 then
						SPTimers:DeleteAnchor(anchor_value)				
						anchor_value = nil
					
						self:SetAnchorTable(1)
					end
				end,
			}
	else
		o.args.bars.args.delete = nil
	end
end

function SPTimers:Cooldown(spellname)
	local opts = self.db.profile.classCooldowns[SPTimers.myCLASS][spellname]
	
	
	self.UpdateSpellCooldowns()
	cooldown_select = spellname
	
	o.args.coolline.args.ClassSpells.args.header1 = {
			name	= "",
			order	= 3,
			type = "header",
		}

	o.args.coolline.args.ClassSpells.args.hide = {
			order = 4,name = L["Hide"], type = "toggle",
			set = function(info,val) opts.hide = not opts.hide; self.UpdateSpellCooldowns() end,
			get = function(info) return opts.hide end
		}
	
	o.args.coolline.args.ClassSpells.args.annonce = {
			order = 4.1,name = L["Reporting"], type = "toggle",
			desc = L["Turn on/off spell cooldown report"],
			set = function(info,val) opts.reporting = not opts.reporting end,
			get = function(info) return opts.reporting end
		}
	o.args.coolline.args.ClassSpells.args.hide_splash = {
			order = 4.2,name = "Forced big splash", type = "toggle",
			set = function(info,val) opts.hide_splash = not opts.hide_splash end,
			get = function(info) return opts.hide_splash end
		}
		
	o.args.coolline.args.ClassSpells.args.colorr = {
			order = 5,name = L["Color"],type = "color",
			set = function(info,r,g,b) opts.color ={r,g,b}; end,
			get = function(info) return opts.color and opts.color[1] or 0 , opts.color and opts.color[2] or 0, opts.color and opts.color[3] or 0 end,
		}

	o.args.coolline.args.ClassSpells.args.color_on = {
			order = 6,name = L["Own color"],type = "toggle",
			set = function(info,val) opts.color_on = not opts.color_on; end,
			get = function(info) return opts.color_on end
		}
	o.args.coolline.args.ClassSpells.args.SoundFile = {					
		type = "group",	order	= 10,
		guiInline = true,
		name	= L["Sound"],
		args = {						
			OnShow = {
				order = 1,type = 'select',name = L["On Show"],
				dialogControl = 'LSM30_Sound',
				values = SPTimers.SharedMedia:HashTable("sound"),
				set = function(info,value) opts.sound_onshow = value end,
				get = function(info) return opts.sound_onshow or "None"; end,
			},
			OnHide = {
				order = 1,type = 'select',name = L["On Hide"],
				dialogControl = 'LSM30_Sound',
				values = SPTimers.SharedMedia:HashTable("sound"),
				set = function(info,value) opts.sound_onhide = value end,
				get = function(info) return opts.sound_onhide or "None"; end,
			},
		}
	}
end

function SPTimers:ICooldown(spellname)
	local opts = self.db.profile.internal_cooldowns[spellname]
	
	ICD_select = spellname
	
	o.args.coolline.args.ICD.args.header1 = {
			name	= "",
			order	= 3,
			type = "header",
		}

	o.args.coolline.args.ICD.args.hide = {
			order = 4,name = L["Hide"], type = "toggle",
			set = function(info,val) opts.hide = not opts.hide; end,
			get = function(info) return opts.hide end
		}
	
	o.args.coolline.args.ICD.args.reporting = {
			order = 4.1,name = L["Reporting"], type = "toggle",
			desc = L["Turn on/off spell cooldown report"],
			set = function(info,val) opts.reporting = not opts.reporting end,
			get = function(info) return opts.reporting end
		}
		
	o.args.coolline.args.ICD.args.hide_splash = {
			order = 4.2,name = "Forced big splash", type = "toggle",
			set = function(info,val) opts.hide_splash = not opts.hide_splash end,
			get = function(info) return opts.hide_splash end
		}
		
	o.args.coolline.args.ICD.args.colorr = {
			order = 5,name = L["Color"],type = "color",
			set = function(info,r,g,b) opts.color ={r,g,b}; end,
			get = function(info) return opts.color and opts.color[1] or 0 , opts.color and opts.color[2] or 0, opts.color and opts.color[3] or 0 end,
		}

	o.args.coolline.args.ICD.args.color_on = {
			order = 6,name = L["Own color"],type = "toggle",
			set = function(info,val) opts.color_on = not opts.color_on; end,
			get = function(info) return opts.color_on end
		}
	o.args.coolline.args.ICD.args.ICD = {
			type = "input", order	= 7,
			name = L["ICD"],
			set = function(info,val) 
				local num = tonumber(val)				
				if num then opts.icd = num end
			end,
			get = function(info) return opts.icd and tostring(opts.icd) or "" end
		}
	o.args.coolline.args.ICD.args.spellID = {
			type = "input", order	= 8,
			name = L["SpellID"],
			set = function(info,val) 
				local num = tonumber(val)				
				if num then opts.spellid = num end
			end,
			get = function(info) return opts.spellid and tostring(opts.spellid) or "" end
		}
	
	o.args.coolline.args.ICD.args.checkID = {
			order = 9,name = L["Check ID"], type = "toggle",
			set = function(info,val) opts.checkID = not opts.checkID; end,
			get = function(info) return opts.checkID end
		}
		
	o.args.coolline.args.ICD.args.SoundFile = {					
		type = "group",	order	= 10,
		guiInline = true,
		name	= L["Sound"],
		args = {						
			OnShow = {
				order = 1,type = 'select',name = L["On Show"],
				dialogControl = 'LSM30_Sound',
				values = SPTimers.SharedMedia:HashTable("sound"),
				set = function(info,value) opts.sound_onshow = value end,
				get = function(info) return opts.sound_onshow or "None"; end,
			},
			OnHide = {
				order = 1,type = 'select',name = L["On Hide"],
				dialogControl = 'LSM30_Sound',
				values = SPTimers.SharedMedia:HashTable("sound"),
				set = function(info,value) opts.sound_onhide = value end,
				get = function(info) return opts.sound_onhide or "None"; end,
			},
		}
	}
end